/* Definitions of IRC message functions and list of messages.
 *
 * Auspice Services is copyright (c) 2000-2001 In Mean
 *     E-mail: auspice@pleaselook.com
 * SirvNET Services is copyright (c) 1998-2000 Trevor Klingbeil.
 *     E-mail: <priority1@dal.net>
 * Originally based on EsperNet Services(c) by Andy Church.
 * This program is free but copyrighted software; see the file COPYING for
 * details.
 */

#include "../inc/services.h"

int serv_com = 0;
int day_serv_com = 0;
int hour_serv_com = 0;
int min_serv_com = 0;
/* List of messages is at the bottom of the file. */

/*************************************************************************/
static void m_nickcoll(char *source, int ac, char **av)
{
    do_quit(source, ac, av);

}
/*************************************************************************/
static void m_user(char *source, int ac, char **av)
{
    send_cmd (server_name, "446 %s :USERS has been disabled", source);
}
/*************************************************************************/
static void m_netinfo(char *source, int ac, char **av)
{
    send_cmd (server_name, "NETINFO 0 %lu 2300 * * * * :%s",
                time(NULL), av[7]);
}
/*************************************************************************/
static void m_time(char *source, int ac, char **av)
{
        time_t t;
        struct tm *tm;
        char buf[94];

        time(&t);
        tm = localtime(&t);
        strftime(buf, sizeof(buf), "%A %B %d %Y -- %H:%M %Z", tm);

        send_cmd (server_name, "391 %s %s :%s", source, server_name, buf);

}
/*************************************************************************/
static void m_stats(char *source, int ac, char **av)
{
        if (stricmp (av[0], "u") == 0)
        {
            int tmpnow;
            tmpnow = time(NULL) - start_time;

            send_cmd (server_name, "242 %s :Server Up %d day%s, %d:%02d:%02d",
                   source, tmpnow/86400, (tmpnow/86400 == 1) ? "" : "s",
                   (tmpnow/3600)%24, (tmpnow/60)%60, tmpnow%60);
 	    send_cmd (server_name, "250 %s :Highest connection count: 0 (0 clients)", source);
	send_cmd (server_name, "219 %s %s :End of /STATS report", source, av[0]);

	} 
}
/*************************************************************************/
static void m_setname(char *source, int ac, char **av)
{
        User *u;
        NickInfo *ni;

        /* First, update the user structure. */
        u = finduser (source);
        u->realname = sstrdup(av[0]);

        /* Now, update the nickinfo. */
        ni = findnick (u->nick);
        ni->last_realname = sstrdup (u->realname);
}

/*************************************************************************/

static void m_server(char *source, int ac, char **av)
{
    if (ac < 1)
	return;
    check_jupes();
    check_zlines();
}

/*************************************************************************/

static void m_squit(char *source, int ac, char **av)
{
    if (ac < 1)
	return;
    check_jupe_squit(source, av[0]);
}


/*************************************************************************/

static void m_ping(char *source, int ac, char **av)
{
    if (ac < 1)
	return;
    send_cmd(server_name, "PONG %s %s", ac>1 ? av[1] : server_name, av[0]);
}

/*************************************************************************/
#ifndef SKELETON

static void m_away(char *source, int ac, char **av)
{
    if (ac == 0 || *av[0] == 0)		/* un-away */
	check_memos(source);
}
#endif

/*************************************************************************/

static void m_join(char *source, int ac, char **av)
{
    if (ac != 1)
	return;
    do_join(source, ac, av);
}

/*************************************************************************/

static void m_sjoin(char *source, int ac, char **av)
{
   do_sjoin(source, ac, av);
}

/*************************************************************************/

static void m_kick(char *source, int ac, char **av)
{
    if (ac != 3)
	return;
    if (stricmp(av[1],s_ChanServ)==0) {
	send_cmd(s_AbuseServ, "JOIN %s", av[0]);
	send_cmd(s_AbuseServ, "MODE %s +o %s", av[0], av[1]);
    }
    do_kick(source, ac, av);
}

/*************************************************************************/

static void m_kill(char *source, int ac, char **av)
{
    if (ac != 2)
	return;
    do_kill(source, ac, av);
}

/*************************************************************************/

static void m_mode(char *source, int ac, char **av)
{
    if (*av[0] == '#' || *av[0] == '&') {
	if (ac < 2)
	    return;
	do_cmode(source, ac, av);
    } else {
	if (ac != 2)
	    return;
	do_umode(source, ac, av);
    }
}

/*************************************************************************/

static void m_motd(char *source, int ac, char **av)
{
    FILE *f;
    char buf[BUFSIZE];
    char buf2[BUFSIZE];
    char *x;
    int i, all=0, avail=0;
    time_t uptime = time(NULL) - start_time;

    f = fopen(MOTD_FILENAME, "r");
    send_cmd(server_name, "375 %s :- %s Message of the Day",
		source, server_name);
   if (f) {
	while (fgets(buf, sizeof(buf), f)) {
	    buf[strlen(buf)-1] = 0;
	    send_cmd(server_name, "372 %s :- %s", source, buf);
	}
	fclose(f);
    }

    send_cmd(server_name, "372 %s :-", source);

    send_cmd(server_name, "372 %s :- This list of people can retreive"
                " lost passwords:", source);

    send_cmd(server_name, "372 %s :-", source);

    *buf = 0;
    *buf2 = 0;

    for (i = 0; i < MAX_SERVADMINS; i++) {
        if (services_admins[i]) {
            all++;
            if (finduser(services_admins[i])) {
                strcat(buf2, services_admins[i]);
                strcat(buf2, " ");
                avail++;
            }
            if (!finduser(services_admins[i])) {
                strcat(buf, services_admins[i]);
                strcat(buf, " ");
            }
        }
    }
    for (i = 0; i < MAX_SERVROOTS; i++) {
        if (services_roots[i]) {
            all++;
            if (finduser(services_roots[i])) {
                strcat(buf2, services_roots[i]);
                strcat(buf2, " ");
                avail++;
            }
            if (!finduser(services_roots[i])) {
                strcat(buf, services_roots[i]);
                strcat(buf, " ");
            }
        }
    }
    if (!*buf && !*buf2)
        strcpy(buf, "List: Not Currently Available");

    send_cmd(server_name, "372 %s :- \2%s \n\2", source, buf2);
    send_cmd(server_name, "372 %s :-", source);
    send_cmd(server_name, "372 %s :- %s \n", source, buf);

    send_cmd(server_name, "372 %s :-", source);
    send_cmd(server_name, "372 %s :- %d of %d nicks available.",
         source, avail, all);

    send_cmd(server_name, "372 %s :- Note: The nicks that are in \2bold\2"
                 " are currently online", source);

    send_cmd(server_name, "372 %s :-", source);

    if (MOTD_UPDATE && !readonly) {
       i = (UPDATE_TIMEOUT - (time(NULL) - last_update));
       send_cmd(server_name, "372 %s :- Next DataBase Update: %d Minute%s, %d second%s",
           source, i/60, i==1 ? "" : "s",
           i%60, i==1 ? "" : "s");
       i=0;
    }

    if (MOTD_UPTIME) {
      if (uptime > 86400)
          send_cmd(server_name, "372 %s :- "
                        "Services up \2%d\2 day%s, \2%02d:%02d\2", source,
                        uptime/86400, (uptime/86400 == 1) ? "" : "s",
                        (uptime/3600) % 24, (uptime/60) % 60);
      else if (uptime > 3600)
          send_cmd(server_name, "372 %s :- "
                        "Services up \2%d hour%s, %d minute%s\2", source,
                        uptime/3600, uptime/3600==1 ? "" : "s",
                        (uptime/60) % 60, (uptime/60)%60==1 ? "" : "s");
      else
          send_cmd(server_name, "372 %s :- "
                        "Services up \2%d minute%s, %d second%s\2", source,
                        uptime/60, uptime/60==1 ? "" : "s",
                        uptime%60, uptime%60==1 ? "" : "s");
    }


    send_cmd(server_name, "372 %s :-", source);

    send_cmd(server_name, "372 %s :- Auspice Services are copyright (c) "
                "2000-2001 In Mean.", source);
    send_cmd(server_name, "376 %s :End of /MOTD command.", source);
}
/*************************************************************************/

static void m_nick(char *source, int ac, char **av)
{
    if (ac == 8) {
	/* Get rid of the useless extra parameter. */
	av[6] = av[7];
	ac--;
    }

    if ((!*source && ac != 7) || (*source && ac != 2))
	return;
    do_nick(source, ac, av);
}

/*************************************************************************/

static void m_part(char *source, int ac, char **av)
{
    if (ac < 1 || ac > 2)
	return;
    do_part(source, ac, av);
}

/*************************************************************************/

static void m_privmsg(char *source, int ac, char **av)
{
    time_t starttime, stoptime;	/* When processing started and finished */
    time_t now = time(NULL);
    char *nick;
    char *tnick;
    User *u = finduser(source);


    if (ac != 2)
	return;


    nick = sstrdup(av[0]);
    if(strchr(nick, '@')) {
	tnick = strtok(nick, "@");
	av[0] = sstrdup(tnick);
    }

    /* Check if we have a flooder.  Operators are ignored. */
    if (allow_flood == 1)
        do_update_flood(u);

    /* Check if we should ignore.  Operators always get through. */
    if ((allow_ignore == 1) && !is_oper(source)) {
         if (check_ignore(u->nick, u->username, u->host) != 0)
              return;
    }


    if (u && (u->flags & U_NOSERV)) {
	log("Access denied to services from %s", source);
	return;
    }
//    if (s_Agent)
//	send_cmd(s_Agent, "PRIVMSG #repeat :%s(%s)> %s", source, av[0], av[1]);

    if ((debug == 1) && ((stricmp(av[0], s_OperServ) == 0)
       || (stricmp(av[0], s_OperServ) == 0)
       || (stricmp(av[0], s_RootServ) == 0)
       || (stricmp(av[0], s_NickServ) == 0)
       || (stricmp(av[0], s_MemoServ) == 0)
       || (stricmp(av[0], s_ChanServ) == 0)
       || (stricmp(av[0], s_HelpServ) == 0)
       || (stricmp(av[0], s_HostServ) == 0)
       || (stricmp(av[0], s_Agent) == 0)
       || (stricmp(av[0], s_ServicesX) == 0)
       || (stricmp(av[0], s_ServicesW) == 0))
       && (!stricmp(av[1], "*identify*") ==0))
            log("%s: %s", source, inbuf);

    if ((sndebug == 1) && ((stricmp(av[0], s_OperServ) == 0)
       || (stricmp(av[0], s_OperServ) == 0)
       || (stricmp(av[0], s_RootServ) == 0)
       || (stricmp(av[0], s_NickServ) == 0)
       || (stricmp(av[0], s_MemoServ) == 0)
       || (stricmp(av[0], s_ChanServ) == 0)
       || (stricmp(av[0], s_HelpServ) == 0)
       || (stricmp(av[0], s_HostServ) == 0)
       || (stricmp(av[0], s_Agent) == 0)
       || (stricmp(av[0], s_ServicesX) == 0)
       || (stricmp(av[0], s_ServicesW) == 0))
       && (!stricmp(av[1], "*identify*") ==0))
           send_cmd(s_OperServ, "PRIVMSG #Snoop :%s: %s", source, inbuf);

    serv_com += 1;

    if (now > d_check_time) {
       d_check_time = time(NULL) + 86400;
       day_serv_com = 1;
    } else {
        day_serv_com += 1;
    }

    if (now > h_check_time) {
       h_check_time = time(NULL) + 3600;
       hour_serv_com = 1;
    } else {
        hour_serv_com += 1;
    }

   if (now > m_check_time) {
      m_check_time = time(NULL) + 60;
      min_serv_com = 1;
   } else {
       min_serv_com += 1;
   }

    starttime = time(NULL);

    if (stricmp(av[0], s_OperServ) == 0) {
        if (is_oper(source) && !(u->flags & U_ABUSIVE))
	    operserv(source, av[1]);
    } else if (stricmp(av[0], s_AbuseServ) == 0) {
	if (is_abuse(source) || is_services_coder(source))
	    abuseserv(source, av[1]);
	else {
            if (is_oper(source))
               notice(s_AbuseServ, source, "Access denied.");
	}
    } else if (stricmp(av[0], s_RootServ) == 0) {
        if (is_services_root(source))
            rootserv(source, av[1]);
	else {
            if (is_oper(source))
               notice(s_RootServ, source, "Access denied.");
	}

    } else if (stricmp(av[0], s_NickServ) == 0) {
	nickserv(source, av[1]);
    } else if (stricmp(av[0], s_ChanServ) == 0) {
	chanserv(source, av[1]);
    } else if (stricmp(av[0], s_MemoServ) == 0) {
	memoserv(source, av[1]);
    } else if (stricmp(av[0], s_HostServ) == 0) {
	hostserv(source, av[1]);
    } else if (stricmp(av[0], s_Agent) == 0) {
	agent(source, av[1]);
    } else if (stricmp(av[0], s_HelpServ) == 0) {
	helpserv(s_HelpServ, source, av[1]);
    } else if (stricmp(av[0], s_ServicesX) == 0) {
	servicesX(source, av[1]);
    } else if (stricmp(av[0], s_ServicesW) == 0) {
	servicesW(source, av[1]);
    } else if (snchan && (stricmp(av[0], snchan) == 0)) {
	send_cmd(s_OperServ, "PRIVMSG #Snoop :<%s/%s> %s", source, snchan, av[1]);
    }

    /* Add to ignore list if the command took a significant amount of time. */
/*
    if (allow_ignore) {
	stoptime = time(NULL);
	if (stoptime > starttime && *source && !strchr(source, '.'))
            add_ignore(mask, source, 10);
    }
*/
}

/*************************************************************************/

static void m_quit(char *source, int ac, char **av)
{
    if (ac != 1)
	return;
    do_quit(source, ac, av);
}

/*************************************************************************/

static void m_topic(char *source, int ac, char **av)
{
    if (ac != 4)
	return;
    do_topic(source, ac, av);
}

/*************************************************************************/
/*************************************************************************/

Message messages[] = {

    { "436",       m_nickcoll },
    { "401",	   NULL },
    { "HELP",	   NULL },
#ifndef SKELETON
    { "AWAY",      m_away },
#else
    { "AWAY",      NULL },
#endif
    { "JOIN",      m_join },
    { "SJOIN",     m_sjoin },
    { "KICK",      m_kick },
    { "KILL",      m_kill },
    { "SVSKILL",   m_kill },
    { "MODE",      m_mode },
    { "SVSMODE",   m_mode },
    { "MOTD",      m_motd },
    { "NICK",      m_nick },
    { "SVSNICK",   m_nick },
    { "NOTICE",    NULL },
    { "PART",      m_part },
    { "PASS",      NULL },
    { "PING",      m_ping },
    { "PRIVMSG",   m_privmsg },
    { "QUIT",      m_quit },
    { "SERVER",    m_server },
    { "SQUIT",     m_squit },
    { "TOPIC",     m_topic },
    { "WALLOPS",   NULL },
    { "VERSION",   m_version },
    { "AKILL",     NULL },
    { "GLOBOPS",   NULL },
    { "CHATOPS",   NULL },
    { "GNOTICE",   NULL },
    { "GOPER",     NULL },
    { "RAKILL",    NULL },
    { "ADMIN",     m_admin },
    { "NETINFO",   m_netinfo },
//    { "SETNAME",   m_setname },
    { "USERS",     m_user },
    { "INFO",      m_info },
    { "CREDITS",   m_credits },
    { "STATS",     m_stats },
    { "TIME",      m_time },
    { "WHOIS",     NULL },
    { NULL }

};

/*************************************************************************/

Message *find_message(const char *name)
{
    Message *m;

    for (m = messages; m->name; m++) {
	if (stricmp(name, m->name) == 0)
	    return m;
    }
    return NULL;
}

/*************************************************************************/
