/* ChanServ functions.
 * 
 * Auspice Services is copyright (c) 2000-2001 In Mean
 *     E-mail: auspice@pleaselook.com
 *
 * SirvNET Services is copyright (c) 1998-2000 Trevor Klingbeil.
 *     E-mail: <priority1@dal.net>
 * Originally based on EsperNet Services(c) by Andy Church.
 * This program is free but copyrighted software; see the file COPYING for
 * details.
 */

/*************************************************************************/

#include "../inc/services.h"
#include "../inc/timeout.h"
#include "../inc/encrypt.h"

# define MODE_SENDER s_ChanServ

/*************************************************************************/

static ChannelInfo *chanlists[256];

static int def_levels[][2] = {
    { CA_AUTOOP,             5 },
    { CA_AUTOHOP,            4 },
    { CA_AUTOVOICE,          3 },
    { CA_AUTODEOP,          -1 },
    { CA_NOJOIN,            -1 },
    { CA_INVITE,             5 },
    { CA_AKICK,             10 },
    { CA_SET,               13 },
    { CA_CLEAR,             13 },
    { CA_UNBAN,              5 },
    { CA_OPDEOP,             5 },
    { CA_ACCESS_LIST,        3 },
    { CA_ACCESS_CHANGE,      1 },
    { -1 }
};

typedef struct {
    int what;
    char *name;
    char *desc;
} LevelInfo;
static LevelInfo levelinfo[] = {
    { CA_AUTOOP,        "AUTOOP",     "Automatic channel operator status" },
    { CA_AUTOHOP,       "AUTOHOP",    "Automatic channel Half operator status"},
    { CA_AUTOVOICE,     "AUTOVOICE",  "Automatic mode +v" },
    { CA_AUTODEOP,      "AUTODEOP",   "Channel operator status disallowed" },
    { CA_NOJOIN,        "NOJOIN",     "Not allowed to join channel if RESTRICTED" },
    { CA_INVITE,        "INVITE",     "Allowed to use INVITE command" },
    { CA_AKICK,         "AKICK",      "Allowed to use AKICK command" },
    { CA_SET,           "SET",        "Allowed to use SET command (not FOUNDER/PASSWORD)" },
    { CA_CLEAR,         "CLEAR",      "Allowed to use CLEAR command" },
    { CA_UNBAN,         "UNBAN",      "Allowed to use UNBAN command" },
    { CA_OPDEOP,        "OPDEOP",     "Allowed to use OP/DEOP commands" },
    { CA_ACCESS_LIST,   "ACC-LIST",   "Allowed to view the access list" },
    { CA_ACCESS_CHANGE, "ACC-CHANGE", "Allowed to modify the access list" },
    { -1 }
};
static int levelinfo_maxwidth = 0;

/*************************************************************************/

/* Help messages for IRC operators. */

#include "cs-help.c"

/*************************************************************************/

void do_aglist(const char *source);
void do_glist(const char *source);
void do_doop(const char *source);
static void alpha_insert_chan(ChannelInfo *ci);
static ChannelInfo *makechan(const char *chan);
static int delchan(ChannelInfo *ci);
static void reset_levels(ChannelInfo *ci);
static int is_founder(User *user, NickInfo *ni, ChannelInfo *ci, int set);
static int is_identified(User *user, ChannelInfo *ci);
static void reset_all_levels();
static void do_ohelp(const char *source);
static void do_ostopic(const char *source);
static void do_help(const char *source);
static void do_register(const char *source);
static void do_identify(const char *source);
static void do_delete(const char *source);
static void do_drop(const char *source);
static void do_set(const char *source);
static void do_set_founder(User *u, ChannelInfo *ci);
static void do_set_password(User *u, ChannelInfo *ci, char *param);
static void do_set_desc(User *u, ChannelInfo *ci, char *param);
static void do_set_welcome(User *u, ChannelInfo *ci, char *param);
static void do_set_url(User *u, ChannelInfo *ci, char *param);
static void do_memolev(User *u, ChannelInfo *ci, char *param);
static void do_set_email(User *u, ChannelInfo *ci, char *param);
static void do_set_topic(User *u, ChannelInfo *ci, char *param);
static void do_set_mlock(User *u, ChannelInfo *ci, char *param);
static void do_set_keeptopic(User *u, ChannelInfo *ci, char *param);
static void do_set_topiclock(User *u, ChannelInfo *ci, char *param);
static void do_set_private(User *u, ChannelInfo *ci, char *param);
static void do_set_opguard(User *u, ChannelInfo *ci, char *param);
static void do_set_auth(User *u, ChannelInfo *ci, char *param);
static void do_set_chanjoin(User *u, ChannelInfo *ci, char *param);
static void do_set_restricted(User *u, ChannelInfo *ci, char *param);
static void do_set_ident(User *u, ChannelInfo *ci, char *param);
static void do_set_unsecure(User *u, ChannelInfo *ci, char *param);
static void do_cfounder(const char *source);
static void do_avoice(const char *source);
static void do_hop(const char *source);
static void do_sop(const char *source);
static void do_aop(const char *source);
static void do_akick(const char *source);
static void do_info(const char *source);
static void do_list(const char *source);
static void do_flist(const char *source);
static void do_invite(const char *source);
static void do_op(const char *source);
static void do_voice (const char *source);
static void do_devoice (const char *source);
static void do_halfop (const char *source);
static void do_dehalfop (const char *source);
static void do_deop(const char *source);
static void do_unban(const char *source);
static void do_clear(const char *source);
static void do_wipe(const char *source);
static void do_count(const char *source);
static void do_getpass(const char *source);
static void do_sendpass(const char *source);
static void do_forbid(const char *source);
static void do_freeze(const char *source);
static void do_suspend(const char *source);
static void do_why(const char *source);
static void do_level(const char *source);
static void do_unhold(const char *source, const char *chan);
static void do_unmark(const char *source, const char *chan);
static void do_unforbid(const char *source, const char *chan);
static void do_unfreeze(const char *source, const char *chan);
static void do_reopen(const char *source, const char *chan);
void do_cs_join(const char *chan);
void do_cs_part(const char *chan);
static void do_setpass(const char *source);
static void do_setfounder(const char *source);

/*************************************************************************/
/*************************************************************************/

#ifndef SKELETON

/* Display total number of registered channels and info about each; or, if
 * a specific channel is given, display information about that channel
 * (like /msg ChanServ INFO <channel>).  If count_only != 0, then only
 * display the number of registered channels (the channel parameter is
 * ignored).
 */

void listchans(int count_only, const char *chan)
{
    long count = 0;
    ChannelInfo *ci;
    int i;

    if (count_only) {

	for (i = 33; i < 256; ++i)
	    for (ci = chanlists[i]; ci; ci = ci->next)
		++count;
	printf("%d channels registered.\n", count);

    } else if (chan) {

	struct tm tm;
	NickInfo *ni;
	char *t;

	if (!(ci = cs_findchan(chan))) {
	    printf("Channel %s not registered.\n", chan);
	    return;
	}
	if (ni = findnick(ci->founder))
	    t = ni->last_usermask;
	else
	    t = NULL;
	if (ci->flags & CI_VERBOTEN) {
	    printf("Channel %s is FORBIDden.\n");
	} else {
	    printf("Information about channel %s:\n", ci->name);
	    printf("        Founder: %s%s%s%s\n",
			ci->founder, t ? " (" : "", t ? t : "", t ? ")" : "");
	    printf("    Description: %s\n", ci->desc);
	    tm = *localtime(&ci->time_registered);
	    printf("     Registered: %s %2d %02d:%02d:%02d '%d\n",
			month_name(tm.tm_mon+1), tm.tm_mday, tm.tm_hour,
			tm.tm_min, tm.tm_sec, tm.tm_year);
	    tm = *localtime(&ci->last_used);
	    printf("      Last used: %s %2d %02d:%02d:%02d '%d\n",
			month_name(tm.tm_mon+1), tm.tm_mday, tm.tm_hour,
			tm.tm_min, tm.tm_sec, tm.tm_year);
	    if (ci->last_topic) {
		printf("     Last topic: %s\n", ci->last_topic);
		printf("   Topic set by: %s\n", ci->last_topic_setter);
	    }
	    if (ci->url)
		printf("            URL: %s\n", ci->url);
	    if (ci->email)
		printf(" E-mail address: %s\n", ci->email);
	    printf("        Options: ");
	    if (!ci->flags) {
		printf("None\n");
	    } else {
		int need_comma = 0;
		static const char commastr[] = ", ";
		if (ci->flags & CI_PRIVATE) {
		    printf("Private");
		    need_comma = 1;
		}
		if (ci->flags & CI_KEEPTOPIC) {
		    printf("%sTopic Retention", need_comma ? commastr : "");
		    need_comma = 1;
		}
		if (ci->flags & CI_TOPICLOCK) {
		    printf("%sTopic Lock", need_comma ? commastr : "");
		    need_comma = 1;
		}
		if (ci->flags & CI_OPGUARD) {
		    printf("%sOp Guard", need_comma ? commastr : "");
		    need_comma = 1;
		}
		if (ci->flags & CI_RESTRICTED) {
		    printf("%sRestricted Access", need_comma ? commastr : "");
		    need_comma = 1;
		}
		if (ci->flags & CI_IDENT) {
		    printf("%sIdent", need_comma ? commastr : "");
		    need_comma = 1;
		}
                if (ci->flags & CI_HELDCHAN) {
                    printf("%sHeld", need_comma ? commastr : "");
                    need_comma = 1;
                }
                if (ci->flags & CI_MARKCHAN) {
                    printf("%sMarked", need_comma ? commastr : "");
                    need_comma = 1;
                }
	    }
	    printf("      Mode lock: ");
	    if (ci->mlock_on || ci->mlock_key || ci->mlock_limit) {
                printf("+%s%s%s%s%s%s%s%s",
			(ci->mlock_on & CMODE_I) ? "i" : "",
			(ci->mlock_key         ) ? "k" : "",
			(ci->mlock_limit       ) ? "l" : "",
			(ci->mlock_on & CMODE_M) ? "m" : "",
			(ci->mlock_on & CMODE_N) ? "n" : "",
			(ci->mlock_on & CMODE_P) ? "p" : "",
			(ci->mlock_on & CMODE_S) ? "s" : "",
			(ci->mlock_on & CMODE_T) ? "t" : "",
                        (ci->mlock_on & CMODE_H) ? "H" : "");
	     }
	    if (ci->mlock_off)
		printf("-%s%s%s%s%s%s%s%s",
			(ci->mlock_off & CMODE_I) ? "i" : "",
			(ci->mlock_off & CMODE_K) ? "k" : "",
			(ci->mlock_off & CMODE_L) ? "l" : "",
			(ci->mlock_off & CMODE_M) ? "m" : "",
			(ci->mlock_off & CMODE_N) ? "n" : "",
			(ci->mlock_off & CMODE_P) ? "p" : "",
			(ci->mlock_off & CMODE_S) ? "s" : "",
			(ci->mlock_off & CMODE_T) ? "t" : "",
                        (ci->mlock_off & CMODE_H) ? "H" : "");
	    if (ci->mlock_key)
		printf(" %s", ci->mlock_key);
	    if (ci->mlock_limit)
		printf(" %ld", ci->mlock_limit);
	    printf("\n");
	}

    } else {

	for (i = 33; i < 256; ++i) {
	    for (ci = chanlists[i]; ci; ci = ci->next) {
		printf("%-20s  %s\n", ci->name,
			ci->flags & CI_VERBOTEN ? "Disallowed (FORBID)"
			                        : ci->desc);
		++count;
	    }
	}
	printf("%d channels registered.\n", count);

    }
}

#endif	/* SKELETON */

/*************************************************************************/

/* Return information on memory use.  Assumes pointers are valid. */

void get_chanserv_stats(long *nrec, long *memuse)
{
    long count = 0, mem = 0;
    int i, j;
    ChannelInfo *ci;

    for (i = 0; i < 256; i++) {
	for (ci = chanlists[i]; ci; ci = ci->next) {
	    count++;
	    mem += sizeof(*ci);
	    if (ci->desc)
		mem += strlen(ci->desc)+1;
	    mem += ci->accesscount * sizeof(ChanAccess);
	    mem += ci->akickcount * sizeof(AutoKick);
	    for (j = 0; j < ci->akickcount; j++) {
		if (ci->akick[j].name)
		    mem += strlen(ci->akick[j].name)+1;
		if (ci->akick[j].reason)
		    mem += strlen(ci->akick[j].reason)+1;
	    }
	    if (ci->mlock_key)
		mem += strlen(ci->mlock_key)+1;
	    if (ci->last_topic)
		mem += strlen(ci->last_topic)+1;
	    if (ci->levels)
		mem += sizeof(*ci->levels) * CA_SIZE;
            if (ci->url)
                mem += strlen(ci->url)+1;
            if (ci->email)
                mem += strlen(ci->email)+1;
            if (ci->welcome)
                mem += strlen(ci->welcome)+1;
            if (ci->hold)
                mem += strlen(ci->hold)+1;
            if (ci->mark)
                mem += strlen(ci->mark)+1;
            if (ci->forbid)
                mem += strlen(ci->forbid)+1;
            if (ci->freeze)
                mem += strlen(ci->freeze)+1;
	}
    }
    *nrec = count;
    *memuse = mem;
}

/*************************************************************************/
/*************************************************************************/

/* Main ChanServ routine. */

void chanserv(const char *source, char *buf)
{
    char *cmd, *s;

    cmd = strtok(buf, " ");

    if (!cmd) {
	return;

    } else if (stricmp(cmd, "\1PING") == 0) {
	if (!(s = strtok(NULL, "")))
	    s = "\1";
	notice(s_ChanServ, source, "\1PING %s", s);

#ifdef SKELETON

    } else {
	notice(s_ChanServ, source, "%s is currently offline.", s_ChanServ);

    }

#else

    } else if (stricmp (cmd, "\1VERSION\1") == 0) {

        if (!(s = strtok (NULL, "")))
            s = "\1";
        notice(s_ChanServ, source, "%sVERSION Auspice IRC Services %s", s, s);

    } else if (stricmp(cmd, "HELP") == 0) {
	do_help(source);

    } else if (stricmp(cmd, "REGISTER") == 0) {
	do_register(source);

    } else if (stricmp(cmd, "IDENTIFY") == 0) {
	do_identify(source);

    } else if (stricmp(cmd, "ID") == 0) {
	do_identify(source);

    } else if (stricmp(cmd, "DROP") == 0) {
	do_drop(source);

    } else if (stricmp(cmd, "SET") == 0) {
	do_set(source);

    } else if (stricmp(cmd, "CFOUNDER") == 0) {
        do_cfounder(source);

    } else if (stricmp(cmd, "CF") == 0) {
        do_cfounder(source);

    } else if (stricmp(cmd, "VOP") == 0) {
        do_avoice(source);

    } else if (stricmp(cmd, "HOP") == 0) {
        do_hop(source);

    } else if (stricmp(cmd, "SOP") == 0) {
        do_sop(source);

    } else if (stricmp(cmd, "AOP") == 0) {
	do_aop(source);

    } else if (stricmp(cmd, "AKICK") == 0) {
	do_akick(source);

    } else if (stricmp(cmd, "INVITE") == 0) {
	do_invite(source);

    } else if (stricmp(cmd, "UNBAN") == 0) {
	do_unban(source);

    } else if (stricmp(cmd, "INFO") == 0) {
	do_info(source);

    } else if (stricmp(cmd, "WHY") == 0) {
        do_why(source);

    } else if (stricmp(cmd, "OP") == 0) {
	do_op(source);

    } else if (stricmp(cmd, "DEOP") == 0) {
	do_deop(source);

    } else if (stricmp(cmd, "VOICE") == 0) {
	do_voice(source);

    } else if (stricmp(cmd, "DEVOICE") == 0) {
	do_devoice(source);

    } else if (stricmp(cmd, "HALFOP") == 0) {
	do_halfop(source);

    } else if (stricmp(cmd, "DEHALFOP") == 0) {
	do_dehalfop(source);

    } else if (stricmp(cmd, "CLEAR") == 0) {
	do_clear(source);

    } else if (stricmp(cmd, "COUNT") == 0) {
        do_count(source);

    } else if (!is_services_helpop(source) && !is_oper(source)) {
        notice(s_ChanServ,
           "Unrecognized command \2%s\2.  Type \"/msg %s HELP\" for help.",
           cmd, s_ChanServ);
	return;        
    } else if (!is_oper(source)) {
            notice(s_ChanServ,source,"Permission Denied.");
            return;

    } else if (stricmp(cmd, "SENDPASS") == 0) {
	do_sendpass(source);

    } else if (stricmp(cmd, "FLIST") == 0) {
	do_flist(source);

    } else if (stricmp(cmd, "OHELP") == 0) {
	do_ohelp(source);

    } else if (stricmp(cmd, "RACCESS") == 0) {
        notice(s_ChanServ, source,
               "RACCESS has been replaced with LEVEL");
        do_level(source);


    } else if (stricmp(cmd, "LEVEL") == 0) {
        do_level(source);


    } else if (!is_services_oper(source)) {
	notice(s_ChanServ, source,
		"Unrecognized command \2%s\2.  Type \"/msg %s HELP\" for help.",
		cmd, s_ChanServ);

    } else if (stricmp(cmd, "SUSPEND") == 0) {
        do_suspend(source);

    } else if (stricmp(cmd, "FREEZE") == 0) {
        do_freeze(source);

    } else if (stricmp(cmd, "TOPIC") == 0) {
        do_ostopic(source);

    } else if (!is_services_admin(source)) {
            notice(s_ChanServ,source,"Permission Denied.");
            return;

    } else if (stricmp(cmd, "FORBID") == 0) {
        do_forbid(source);

    } else if (stricmp(cmd, "WIPE") == 0) {
         do_wipe(source);

    } else if (stricmp(cmd, "HOLD") == 0) {
	char *chan = strtok(NULL, " ");
	ChannelInfo *ci;
	User *u = finduser(source);

	if ((!chan) || (stricmp(chan, "-") == 0)) {
            notice(s_ChanServ,source,"Syntax: \2HOLD\2 [-]<#channel>");
	    notice(s_ChanServ,source,"\2/msg %s OHELP HOLD\2 for more information.",s_ChanServ);
            return;
        }
        if (*chan == '-') {
           *chan++;
           do_unhold(source, chan);

	} else if (!(ci = cs_findchan(chan))) {
	    notice(s_ChanServ, source, "Channel %s is not registered.", chan);
        } else if (!(ci->flags & CI_HELDCHAN)) {
            ci->flags |= CI_HELDCHAN;
            if (ci->hold)
                  free(ci->hold);
            ci->hold = sstrdup(u->nick);
            wallops(s_ChanServ,"%s set the HELD flag for \2%s\2",u->nick,chan);
            slog("CS +H %s (%s!%s@%s)",
                chan, u->nick, u->username, u->host);
            do_break_log("CS_F", "CS +H %s (%s!%s@%s)",
                chan, u->nick, u->username, u->host);

	} else {
	    notice(s_ChanServ,source,"Channel \2%s\2 is already held",chan);
	}

    } else if (stricmp(cmd, "MARK") == 0) {
        char *chan = strtok(NULL, " ");
        ChannelInfo *ci;
        User *u = finduser(source);

        if ((!chan) || (stricmp(chan, "-") == 0)) {	
            notice(s_ChanServ,source,"Syntax: \2MARK\2 [-]<#channel>");
            notice(s_ChanServ,source,
		"\2/msg %s OHELP MARK\2 for more information",s_ChanServ);
            return;
        }

        if (*chan == '-') {
           *chan++;
           do_unmark(source, chan);

        } else if (!(ci = cs_findchan(chan))) {
            notice(s_ChanServ, source, "Channel %s is not registered.",chan);
        } else if (!(ci->flags & CI_MARKCHAN)) {
            ci->flags |= CI_MARKCHAN;
            if (ci->mark)
                free(ci->mark);
            ci->mark = sstrdup(u->nick);
            slog("CS +M %s (%s!%s@%s)",
                chan, u->nick, u->username, u->host);
            do_break_log("CS_F", "CS +M %s (%s!%s@%s)",
                chan, u->nick, u->username, u->host);
	    wallops(s_ChanServ,
                "%s set the MARK flag for \2%s\2",u->nick,chan);

	} else {
	    notice(s_ChanServ,source,"Channel \2%s\2 already marked.",chan);
	}

    } else if (stricmp(cmd, "GETPASS") == 0) {
	do_getpass(source);

    } else if (!is_services_root(source)) {
           notice(s_ChanServ,source,"Permission Denied.");
           return;
/*
    } else if (stricmp(cmd, "RESETALLLEVELS") == 0) {
        reset_all_levels();
	notice(s_ChanServ, source, "All levels reset.");
        slog("All ChanServ access levels reset.");
*/

    } else if (stricmp(cmd, "DELETE") == 0) {
        do_delete(source);

    } else if (stricmp(cmd, "SETPASS") == 0) {
        do_setpass(source);

    } else if (stricmp(cmd, "SETFOUNDER") == 0) {
        do_setfounder(source);

    } else if (stricmp(cmd, "LIST") == 0) {
	do_list(source);

    } else {
        User *u = finduser(source);
	notice(s_ChanServ, source,
		"Unrecognized command \2%s\2.  Type \"/msg %s HELP\" for help.",
		cmd, s_ChanServ);
        slog("CS *C %s (%s@%s) [%s]",
            source, u->username, u->host, cmd);

    }

#endif	/* SKELETON */

}

/*************************************************************************/

#ifndef SKELETON

/* Load/save data files. */

void load_cs_dbase(void)
{
    FILE *f = fopen(CHANSERV_DB, "r");
    int ver, i, j, len;
    ChannelInfo *ci;

    if (!(f = open_db(s_ChanServ, CHANSERV_DB, "r")))
	return;

    switch (ver = get_file_version(f, CHANSERV_DB)) {
      case 5:
        for (i = 33; i < 256; ++i) {

            while (fgetc(f) == 1) {

                ci = smalloc(sizeof(ChannelInfo));
                if (1 != fread(ci, sizeof(ChannelInfo), 1, f))
                    fatal_perror("Read error on %s", CHANSERV_DB);
#ifdef USE_ENCRYPTION
                if (!(ci->flags & (CI_ENCRYPTEDPW | CI_VERBOTEN))) {
                    if (debug)
                        log("debug: %s: encrypting password for %s on load",
                                s_ChanServ, ci->name);
                    if (encrypt_in_place(ci->founderpass, PASSMAX) < 0)
                        fatal("%s: load database: Can't encrypt %s password!",
                                s_ChanServ, ci->name);
                    ci->flags |= CI_ENCRYPTEDPW;
                }
#else
                if (ci->flags & CI_ENCRYPTEDPW) {
                    /* Bail: it makes no sense to continue with encrypted
                     * passwords, since we won't be able to verify them */
                    fatal("%s: load database: password for %s encrypted "
                          "but encryption disabled, aborting",
                          s_ChanServ, ci->name);
                }
#endif
                /* Can't guarantee the file is in a particular order...
                 * (Well, we can, but we don't have to depend on it.) */
                alpha_insert_chan(ci);
                if (ver < 3) {
                    NickInfo *ni = findnick(ci->founder);
                    if (ni && ni->channelcount+1 > ni->channelcount)
                        ni->channelcount++;
                }
                ci->desc = read_string(f, CHANSERV_DB);
                if (ci->url)
                    ci->url = read_string(f, CHANSERV_DB);
                if (ci->email)
                    ci->email = read_string(f, CHANSERV_DB);
                if (ci->mlock_key)
                    ci->mlock_key = read_string(f, CHANSERV_DB);
                if (ci->last_topic)
                    ci->last_topic = read_string(f, CHANSERV_DB);
		if (ci->welcome)
		    ci->welcome = read_string(f, CHANSERV_DB);
                if (ci->hold)
                    ci->hold = read_string(f, CHANSERV_DB);
                if (ci->mark)
                    ci->mark = read_string(f, CHANSERV_DB);
                if (ci->freeze)
                    ci->freeze = read_string(f, CHANSERV_DB);
                if (ci->forbid)
                    ci->forbid = read_string(f, CHANSERV_DB);

                if (ci->accesscount) {
                    ChanAccess *access;
                    access = smalloc(sizeof(ChanAccess) * ci->accesscount);
                    ci->access = access;
                    if (ci->accesscount != fread(access, sizeof(ChanAccess),
                                                        ci->accesscount, f))
                        fatal_perror("Read error on ", CHANSERV_DB);
                    for (j = 0; j < ci->accesscount; ++j, ++access)
                        access->name = read_string(f, CHANSERV_DB);
                    j = 0; access = ci->access;
                    /* Clear out unused entries */
                    while (j < ci->accesscount) {
                        if (access->is_nick < 0)
//			|| (access->is_nick == 0 && !findnick(access->name)))
			{

                            --ci->accesscount;
                            free(access->name);
                            if (j < ci->accesscount)
                                bcopy(access+1, access, sizeof(*access) *
                                                        (ci->accesscount - j));
                        } else {
                            access->is_nick = 1;
                            ++j; ++access;
                        }
                    }
                    if (ci->accesscount)
                        ci->access = srealloc(ci->access,
                                        sizeof(ChanAccess) * ci->accesscount);
                    else {
                        free(ci->access);
                        ci->access = NULL;
                    }
                } /* if (ci->accesscount) */

                if (ci->akickcount) {
                    AutoKick *akick;
                    akick = smalloc(sizeof(AutoKick) * ci->akickcount);
                    ci->akick = akick;
                    if (ci->akickcount !=
                            fread(akick, sizeof(AutoKick), ci->akickcount, f))
                        fatal_perror("Read error on %s", CHANSERV_DB);
                    for (j = 0; j < ci->akickcount; ++j, ++akick) {
                        akick->name = read_string(f, CHANSERV_DB);
                        if (akick->reason)
                            akick->reason = read_string(f, CHANSERV_DB);
                    }
                    j = 0; akick = ci->akick;
                    while (j < ci->akickcount) {
                        if (akick->is_nick < 0) {
                            --ci->akickcount;
                            free(akick->name);
                            if (akick->reason)
                                free(akick->reason);
                            if (j < ci->akickcount)
                                bcopy(akick+1, akick, sizeof(*akick) *
                                                        (ci->akickcount - j));
                        } else {
                            ++j; ++akick;
                        }
                    }
                    if (ci->akickcount) {
                        ci->akick = srealloc(ci->akick,
                                        sizeof(AutoKick) * ci->akickcount);
                    } else {
                        free(ci->akick);
                        ci->akick = NULL;
                    }
                } /* if (ci->akickcount) */
                if (ci->levels) {
                    int n_entries;
                    ci->levels = NULL;
                    reset_levels(ci);
                    n_entries = fgetc(f)<<8 | fgetc(f);
                    if (n_entries < 0)
                        fatal_perror("Read error on %s", CHANSERV_DB);
                    /* Ignore earlier, incompatible levels list */
                    if (n_entries == 6) {
                        fseek(f, sizeof(short) * n_entries, SEEK_CUR);
                    } else
                    if (n_entries <= CA_SIZE) {
                        fread(ci->levels, sizeof(short), n_entries, f);
                    } else {
                        fread(ci->levels, sizeof(short), CA_SIZE, f);
                        fseek(f, sizeof(short) * (n_entries - CA_SIZE),
                                                                SEEK_CUR);
                    }
                } else {
                    reset_levels(ci);
                }

            } /* while (fgetc(f) == 1) */

        } /* for (i) */

        break; /* case 1, etc. */

      case 4:
      case 3:
      case 2:
      case 1:

	for (i = 33; i < 256; ++i) {

	    while (fgetc(f) == 1) {

		ci = smalloc(sizeof(ChannelInfo));
		if (1 != fread(ci, sizeof(ChannelInfo), 1, f))
		    fatal_perror("Read error on %s", CHANSERV_DB);
#ifdef USE_ENCRYPTION
		if (!(ci->flags & (CI_ENCRYPTEDPW | CI_VERBOTEN))) {
		    if (debug)
			log("debug: %s: encrypting password for %s on load",
				s_ChanServ, ci->name);
		    if (encrypt_in_place(ci->founderpass, PASSMAX) < 0)
			fatal("%s: load database: Can't encrypt %s password!",
				s_ChanServ, ci->name);
		    ci->flags |= CI_ENCRYPTEDPW;
		}
#else
		if (ci->flags & CI_ENCRYPTEDPW) {
		    /* Bail: it makes no sense to continue with encrypted
		     * passwords, since we won't be able to verify them */
		    fatal("%s: load database: password for %s encrypted "
		          "but encryption disabled, aborting",
		          s_ChanServ, ci->name);
		}
#endif
		/* Can't guarantee the file is in a particular order...
		 * (Well, we can, but we don't have to depend on it.) */
		alpha_insert_chan(ci);
		if (ver < 3) {
		    NickInfo *ni = findnick(ci->founder);
		    if (ni && ni->channelcount+1 > ni->channelcount)
			ni->channelcount++;
		}
		ci->desc = read_string(f, CHANSERV_DB);
		if (ci->url)
		    ci->url = read_string(f, CHANSERV_DB);
		if (ci->email)
		    ci->email = read_string(f, CHANSERV_DB);
		if (ci->mlock_key)
		    ci->mlock_key = read_string(f, CHANSERV_DB);
		if (ci->last_topic)
		    ci->last_topic = read_string(f, CHANSERV_DB);

		if (ci->accesscount) {
		    ChanAccess *access;
		    access = smalloc(sizeof(ChanAccess) * ci->accesscount);
		    ci->access = access;
		    if (ci->accesscount != fread(access, sizeof(ChanAccess),
							ci->accesscount, f))
			fatal_perror("Read error on ", CHANSERV_DB);
		    for (j = 0; j < ci->accesscount; ++j, ++access)
			access->name = read_string(f, CHANSERV_DB);
		    j = 0; access = ci->access;
		    /* Clear out unused entries */
		    while (j < ci->accesscount) {
			if (access->is_nick < 0)
//			 ||(access->is_nick == 0 && !findnick(access->name)))
			{

			    --ci->accesscount;
			    free(access->name);
			    if (j < ci->accesscount)
				bcopy(access+1, access,	sizeof(*access) *
							(ci->accesscount - j));
			} else {
			    access->is_nick = 1;
			    ++j; ++access;
			}
		    }
		    if (ci->accesscount)
			ci->access = srealloc(ci->access,
					sizeof(ChanAccess) * ci->accesscount);
		    else {
			free(ci->access);
			ci->access = NULL;
		    }
		} /* if (ci->accesscount) */

		if (ci->akickcount) {
		    AutoKick *akick;
		    akick = smalloc(sizeof(AutoKick) * ci->akickcount);
		    ci->akick = akick;
		    if (ci->akickcount !=
			    fread(akick, sizeof(AutoKick), ci->akickcount, f))
			fatal_perror("Read error on %s", CHANSERV_DB);
		    for (j = 0; j < ci->akickcount; ++j, ++akick) {
			akick->name = read_string(f, CHANSERV_DB);
			if (akick->reason)
			    akick->reason = read_string(f, CHANSERV_DB);
		    }
		    j = 0; akick = ci->akick;
		    while (j < ci->akickcount) {
			if (akick->is_nick < 0) {
			    --ci->akickcount;
			    free(akick->name);
			    if (akick->reason)
				free(akick->reason);
			    if (j < ci->akickcount)
				bcopy(akick+1, akick, sizeof(*akick) *
							(ci->akickcount - j));
			} else {
			    ++j; ++akick;
			}
		    }
		    if (ci->akickcount) {
			ci->akick = srealloc(ci->akick,
					sizeof(AutoKick) * ci->akickcount);
		    } else {
			free(ci->akick);
			ci->akick = NULL;
		    }
		} /* if (ci->akickcount) */

		if (ci->levels) {
		    int n_entries;
		    ci->levels = NULL;
		    reset_levels(ci);
		    n_entries = fgetc(f)<<8 | fgetc(f);
		    if (n_entries < 0)
			fatal_perror("Read error on %s", CHANSERV_DB);
		    /* Ignore earlier, incompatible levels list */
		    if (n_entries == 6) {
			fseek(f, sizeof(short) * n_entries, SEEK_CUR);
		    } else
		    if (n_entries <= CA_SIZE) {
			fread(ci->levels, sizeof(short), n_entries, f);
		    } else {
			fread(ci->levels, sizeof(short), CA_SIZE, f);
			fseek(f, sizeof(short) * (n_entries - CA_SIZE),
								SEEK_CUR);
		    }
		} else {
		    reset_levels(ci);
		}

	    } /* while (fgetc(f) == 1) */

	} /* for (i) */

	break; /* case 1, etc. */

      default:
	fatal("Unsupported version number (%d) on %s", i, CHANSERV_DB);

    } /* switch (version) */

    close_db(f, CHANSERV_DB);
}

/*************************************************************************/

void save_cs_dbase(void)
{
    FILE *f;
    int i, j, len;
    ChannelInfo *ci;

    if (!(f = open_db(s_ChanServ, CHANSERV_DB, "w")))
	return;

    for (i = 33; i < 256; ++i) {
	for (ci = chanlists[i]; ci; ci = ci->next) {
	    fputc(1, f);
	    if (1 != fwrite(ci, sizeof(ChannelInfo), 1, f))
		fatal_perror("Write error on %s", CHANSERV_DB);
	    write_string(ci->desc ? ci->desc : "", f, CHANSERV_DB);
	    if (ci->url)
		write_string(ci->url, f, CHANSERV_DB);
	    if (ci->email)
		write_string(ci->email, f, CHANSERV_DB);
	    if (ci->mlock_key)
		write_string(ci->mlock_key, f, CHANSERV_DB);
	    if (ci->last_topic)
		write_string(ci->last_topic, f, CHANSERV_DB);
	    if (ci->welcome)
		write_string(ci->welcome, f, CHANSERV_DB);
            if (ci->hold)
                write_string(ci->hold, f, CHANSERV_DB);
            if (ci->mark)
                write_string(ci->mark, f, CHANSERV_DB);
            if (ci->freeze)
                write_string(ci->freeze, f, CHANSERV_DB);
            if (ci->forbid)
                write_string(ci->forbid, f, CHANSERV_DB);

	    if (ci->accesscount) {
		ChanAccess *access = ci->access;
		if (ci->accesscount !=
			fwrite(access, sizeof(ChanAccess), ci->accesscount, f))
		    fatal_perror("Write error on %s", CHANSERV_DB);
		for (j = 0; j < ci->accesscount; ++j, ++access) {
		    if (access->in_use)
			write_string(access->name, f, CHANSERV_DB);
		    else
			write_string("", f, CHANSERV_DB);
		}
	    }

	    if (ci->akickcount) {
		AutoKick *akick = ci->akick;
		if (ci->akickcount !=
			fwrite(akick, sizeof(AutoKick), ci->akickcount, f))
		    fatal_perror("Write error on %s", CHANSERV_DB);
		for (j = 0; j < ci->akickcount; ++j, ++akick) {
		    if (akick->is_nick >= 0) {
			write_string(akick->name, f, CHANSERV_DB);
			if (akick->reason)
			    write_string(akick->reason, f, CHANSERV_DB);
		    } else {
			write_string("", f, CHANSERV_DB);
			if (akick->reason)
			    write_string("", f, CHANSERV_DB);
		    }
		}
	    }

	    if (ci->levels) {
		fputc(CA_SIZE >> 8, f);
		fputc(CA_SIZE & 0xFF, f);
		fwrite(ci->levels, sizeof(short), CA_SIZE, f);
	    }

	} /* for (chanlists[i]) */

	fputc(0, f);

    } /* for (i) */

    close_db(f, CHANSERV_DB);
}

/*************************************************************************/

/* Check the current modes on a channel; if they conflict with a mode lock,
 * fix them. */

void check_modes(const char *chan)
{
    Channel *c = findchan(chan);
    ChannelInfo *ci = cs_findchan(chan);
    char newmodes[32], *newkey = NULL;
    long newlimit = 0;
    char *end = newmodes;
    int modes;
    int set_limit = 0, set_key = 0;

    if (!c || !ci)
	return;

    if (ci->flags & CI_FREEZECHAN)
        return;

    *end++ = '+';
    modes = ~c->mode & ci->mlock_on;

     if (modes & CMODE_r) {
        c->mode |= CMODE_r;  /* Registered Channel */
        *end++ = 'r';
     }

    if (modes & CMODE_I) {
	*end++ = 'i';
	c->mode |= CMODE_I;
    }
    if (modes & CMODE_M) {
	*end++ = 'm';
	c->mode |= CMODE_M;
    }
    if (modes & CMODE_N) {
	*end++ = 'n';
	c->mode |= CMODE_N;
    }
    if (modes & CMODE_P) {
	*end++ = 'p';
	c->mode |= CMODE_P;
    }
    if (modes & CMODE_S) {
	*end++ = 's';
	c->mode |= CMODE_S;
    }
    if (modes & CMODE_T) {
	*end++ = 't';
	c->mode |= CMODE_T;
    }

    if (modes & CMODE_C) {
	*end++ = 'c';
	c->mode |= CMODE_C;
    }
    if (modes & CMODE_Q) {
	*end++ = 'q';
	c->mode |= CMODE_Q;
    }
    if (modes & CMODE_H) {
	*end++ = 'H';
	c->mode |= CMODE_H;
    }
    if (ci->mlock_limit && ci->mlock_limit != c->limit) {
	*end++ = 'l';
	newlimit = ci->mlock_limit;
	c->limit = newlimit;
	set_limit = 1;
    }
    if (!c->key && ci->mlock_key) {
	*end++ = 'k';
	newkey = ci->mlock_key;
	c->key = sstrdup(newkey);
	set_key = 1;
    } else if (c->key && ci->mlock_key && stricmp(c->key, ci->mlock_key) != 0) {
	change_cmode (s_ChanServ, c->name, "-k", c->key);
/*	send_cmd(MODE_SENDER, "MODE %s -k %s", c->name, c->key); */
	free (c->key);
	*end++ = 'k';
	newkey = ci->mlock_key;
	c->key = sstrdup(newkey);
	set_key = 1;
    }

    if (end[-1] == '+')
	--end;

    *end++ = '-';
    modes = c->mode & ci->mlock_off;
    if (modes & CMODE_I) {
	*end++ = 'i';
	c->mode &= ~CMODE_I;
    }
    if (modes & CMODE_M) {
	*end++ = 'm';
	c->mode &= ~CMODE_M;
    }
    if (modes & CMODE_N) {
	*end++ = 'n';
	c->mode &= ~CMODE_N;
    }
    if (modes & CMODE_P) {
	*end++ = 'p';
	c->mode &= ~CMODE_P;
    }
    if (modes & CMODE_S) {
	*end++ = 's';
	c->mode &= ~CMODE_S;
    }
    if (modes & CMODE_T) {
	*end++ = 't';
	c->mode &= ~CMODE_T;
    }
    if (modes & CMODE_C) {
	*end++ = 'c';
	c->mode &= ~CMODE_C;
    }
    if (modes & CMODE_H) {
        *end++ = 'H';
        c->mode &= ~CMODE_H;
    }
    if (c->key && (ci->mlock_off & CMODE_K)) {
	*end++ = 'k';
	newkey = sstrdup(c->key);
	free(c->key);
	c->key = NULL;
	set_key = 1;
    }
    if (c->limit && (ci->mlock_off & CMODE_L)) {
	*end++ = 'l';
	c->limit = 0;
    }

    if (end[-1] == '-')
	--end;

    if (end == newmodes)
	return;
    *end = 0;
    if (set_limit) {
	if (set_key)
	    send_cmd(MODE_SENDER, "MODE %s %s %d :%s", c->name,
				newmodes, newlimit, newkey ? newkey : "");
	else
	    send_cmd(MODE_SENDER, "MODE %s %s %d", c->name, newmodes, newlimit);
    } else if (set_key)
	send_cmd(MODE_SENDER, "MODE %s %s :%s", c->name,
				newmodes, newkey ? newkey : "");
    else
	send_cmd(MODE_SENDER, "MODE %s %s", c->name, newmodes);

    if (newkey && !c->key)
	free(newkey);
}

/*************************************************************************/

/* Check whether a user is allowed to be opped on a channel; if they
 * aren't, deop them.  If serverop is 1, the +o was done by a server.
 * Return 1 if the user is allowed to be opped, 0 otherwise. */

int check_valid_op(User *user, const char *chan, int serverop)
{
    ChannelInfo *ci = cs_findchan(chan);
    NickInfo *ni;

    ni = findnick(user->nick);
    
    if (!ci)
      return 1;

    if (!ni || (ni && ni->flags & NI_NEVEROP)) {
       if (ci->flags & CI_LEAVEOPS)
           return 1;
    }

    if (rec_services_root(user->nick))
 	return 1;

    if (ci->flags & CI_VERBOTEN)
	return 0;
	/* check_kick() will get them out; we needn't explain. */

    if (ci->flags & CI_FREEZECHAN)
        return 0;

    if (serverop && !check_access(user, ci, CA_AUTOOP)) {
	notice(s_ChanServ, user->nick,
		"This channel has been registered with %s.", s_ChanServ);
	return 0;
    }

    if (ni && (ni->flags & NI_NEVEROP))
	return 0;


    if (check_access(user, ci, CA_AUTODEOP)) {
#if 0	/* This is probably a bad idea.  People could flood other people,
	 * and stuff. */
	notice(s_ChanServ, user->nick,
		"You are not allowed chanop status on channel %s.", chan);
#endif
	return 0;
    }

    if (ci->flags & CI_OPGUARD) {
         if (!check_opguard(user, chan))
            return 0;
         else
            return 1;
    }


    return 1;
}

/*************************************************************************/

/* Check whether a user should be opped on a channel, and if so, do it.
 * Return 1 if the user was opped, 0 otherwise.  (Updates the channel's
 * last used time if the user was opped.) */

int check_should_op(User *user, const char *chan)
{
    ChannelInfo *ci = cs_findchan(chan);
    NickInfo *ni;

    if (ci && ci->flags & CI_FREEZECHAN)
        return 0;

    if (ci && ci->welcome)
        notice(s_ChanServ, user->nick, "%s Welcome: %s", ci->name, ci->welcome);

    
    if (!ci || (ci->flags & CI_VERBOTEN) || !ci->levels[CA_AUTOOP] ||
		*chan == '+')
	return 0;

    ni = findnick(user->nick);
    if (ci->flags & CI_IDENT) {
        if (!ni || !(ni->flags & NI_IDENTIFIED))
	    return 0;
   }
    
    if (!ni || !(ni->flags & NI_NEVEROP)) {
       if (check_access(user, ci, CA_AUTOOP)) {

	    if ((ci->mark) && (CHANX == 1) && (ci->hold))
 		   change_cmode (s_ServicesX, chan, "+o", user->nick);
	    else
		   change_cmode (s_ChanServ, chan, "+o", user->nick);

/*	   if (check_access(user, ci, CA_SET)) */
	   ci->last_used = time(NULL);
           return 1;
      }
    }
    return 0;
}

/*************************************************************************/

/* Check whether a user should be voiced on a channel, and if so, do it.
 * Return 1 if the user was voiced, 0 otherwise. */

int check_should_voice(User *user, const char *chan)
{
    ChannelInfo *ci = cs_findchan(chan);
    NickInfo *ni;

    if (!ci || (ci->flags & CI_VERBOTEN) || !ci->levels[CA_AUTOVOICE] ||
		*chan == '+')
	return 0;

    if (ci->flags & CI_FREEZECHAN)
        return 0;

    ni = findnick(user->nick);
    if (ci->flags & CI_IDENT) {
        if (ni && !(ni->flags & NI_IDENTIFIED))
	    return 0;
    }
    if (ni && (ni->flags & NI_NEVEROP))
         return 0;

    if (get_access(user, ci) == 3) {
         send_cmd(MODE_SENDER, "MODE %s +v %s", chan, user->nick);
         return 1;
    }
    return 0;
}

/*************************************************************************/

/* Check whether a user should be halfop on a channel, and if so, do it.
 * Return 1 if the user was halfop, 0 otherwise. */

int check_should_halfop(User *user, const char *chan)
{
    ChannelInfo *ci = cs_findchan(chan);
    NickInfo *ni;

    if (!ci || (ci->flags & CI_VERBOTEN) || !ci->levels[CA_AUTOHOP] ||
		*chan == '+')
	return 0;

    if (ci->flags & CI_FREEZECHAN)
        return 0;

    ni = findnick(user->nick);
    if (ci->flags & CI_IDENT) {
        if (ni && !(ni->flags & NI_IDENTIFIED))
	    return 0;
    }
    if (ni && (ni->flags & NI_NEVEROP))
         return 0;

    if (get_access(user, ci) == 4) {
         send_cmd(MODE_SENDER, "MODE %s +h %s", chan, user->nick);
         return 1;
    }
    return 0;
}

/*************************************************************************/
/* Tiny helper routine to get ChanServ out of a channel after it went in. */

static void timeout_leave(Timeout *to)
{
    char *chan = to->data;
    send_cmd(s_ChanServ, "PART %s", chan);
    free(to->data);
}


static void timeout_unban(Timeout *to)
{
    char *chan = to->data;
    send_cmd(s_ChanServ, "MODE %s -b *!*@*", chan);
    free(to->data);
}


/* Check whether a user is permitted to be on a channel.  If so, return 0;
 * else, kickban the user with an appropriate message (could be either
 * AKICK or restricted access) and return 1.  Note that this is called
 * _before_ the user is added to internal channel lists (so do_kick() is
 * not called).
 */

int check_kick(User *user, const char *chan)
{
    ChannelInfo *ci = cs_findchan(chan);
    C_SuspendData *c_spn = get_c_suspend(chan);
    AutoKick *akick;
    int i, bad;
    NickInfo *ni;
    char *av[3], *mask, *reason;
    int check_level;
    Timeout *t;

    send_cmd(s_ChanServ, "PRIVMSG #Debug :K 1");

    if (!ci)
	return 0;

    send_cmd(s_ChanServ, "PRIVMSG #Debug :K 2");

    if (ci->flags & CI_VERBOTEN) {
	mask = create_mask(user);
	reason = "This channel may not be used.";
	goto kick;
    }


    if (c_spn && c_spn->time > time(NULL)) {
       mask = create_mask(user);
       reason = "Channel is currently suspended";
       goto kick;
    }

    send_cmd(s_ChanServ, "PRIVMSG #Debug :K 3");

    if (ci->flags & CI_FREEZECHAN)
        return 0;

    if (is_services_root(user->nick))
	return 0;



    if ((ci->flags & CI_CODERONLY) && !is_services_coder(user->nick)) {
        mask = create_mask(user);
        reason = "Only Services Masters are allowed to use this channel.";
        send_cmd(s_ChanServ, "MODE %s +A", chan);
        goto kick;
    }

    if ((ci->flags & CI_OPERONLY) && !is_oper(user->nick)) {
	mask = create_mask(user);
	reason = "Only IRC Operators are allowed to use this channel.";
        send_cmd(s_ChanServ, "MODE %s +O", chan);
	goto kick;
    }
    if ((ci->flags & CI_SOPONLY) && !rec_services_oper(user->nick)) {
	mask = create_mask(user);
	reason = "Only Services Opers are allowed to use this channel.";
	goto kick;
    }
    if ((ci->flags & CI_SAONLY) && !rec_services_admin(user->nick)) {
	mask = create_mask(user);
	reason = "Only Services Admins are allowed to use this channel.";
        send_cmd(s_ChanServ, "MODE %s +O", chan);
	goto kick;
    }
    if ((ci->flags & CI_SRAONLY) && !rec_services_root(user->nick)) {
	mask = create_mask(user);
	reason = "Only Services Roots are allowed to use this channel.";
	goto kick;
    }

    if ((ci->flags & CI_ABUSEONLY) && !rec_abuse(user->nick)) {
        mask = create_mask(user);
        reason = "Only Abuse Team Members are allowed to use this channel.";
        goto kick;
    }

    if ((ci->flags & CI_RESTRICTED) && !rec_services_root(user->nick)) {
         if (get_access(user, ci) < 3) {
             mask = create_mask(user);
             reason = "You are not permitted to be on this channel.";
	     goto kick;
         }
    }

    send_cmd(s_ChanServ, "PRIVMSG #Debug :K 5");

    ni = findnick(user->nick);
    if ((ni == NULL) || !(ni->flags & (NI_IDENTIFIED | NI_RECOGNIZED)))
	ni = NULL;

    send_cmd(s_ChanServ, "PRIVMSG #Debug :K 5b");

    if (get_access(user, ci) > 0)
        return 0;

    send_cmd(s_ChanServ, "PRIVMSG #Debug :K 6");

    for (akick = ci->akick, i = 0; i < ci->akickcount; ++akick, ++i) {
	if ((akick->is_nick > 0
			&& ni && stricmp(akick->name, user->nick) == 0) ||
			(!akick->is_nick && match_usermask(akick->name, user))
	) {
	    mask = akick->is_nick ? create_mask(user) : sstrdup(akick->name);
	    reason = akick->reason ? akick->reason : DEF_AKICK_REASON;
	    goto kick;
	}
    }

    send_cmd(s_ChanServ, "PRIVMSG #Debug :K 7");

    return 0;

kick:


    if (debug) {
	log("debug: channel: AutoKicking %s!%s@%s",
		user->nick, user->username, user->host);
    }
    /* Remember that the user has not been added to our channel user list
     * yet, so we check whether the channel does not exist */


    av[0] = sstrdup(chan);
    send_cmd(s_ChanServ, "JOIN %s", chan);
    strcpy(av[0], chan);
    av[1] = sstrdup("+b");
    av[2] = mask;
    send_cmd(s_ChanServ, "MODE %s +b %s  %lu", chan, av[2], time(NULL));
    do_cmode(s_ChanServ, 3, av);
    send_cmd(s_ChanServ, "KICK %s %s :%s", chan, user->nick, reason);
    av[1] = sstrdup(user->nick);
    do_kick(s_ChanServ, 3, av);
    free(av[0]);    free(av[1]);    free(av[2]);
    t = add_timeout(CHANNEL_INHABIT, timeout_leave, 0);
    t->data = sstrdup(chan);
/* 
   should be fine to leave out timeout_unban since the empty
   channel will remove the mode anyway.
*/
    send_cmd(s_ChanServ, "PRIVMSG #Debug :K 8");

    return 1;
}

/*************************************************************************/

/* Record the current channel topic in the ChannelInfo structure. */

void record_topic(const char *chan)
{
    Channel *c;
    ChannelInfo *ci;

    if (readonly)
	return;
    c = findchan(chan);
    ci = cs_findchan(chan);
    if (!c || !ci)
	return;
    if (ci->flags & CI_FREEZECHAN)
        return;
    if (ci->last_topic)
	free(ci->last_topic);
    if (c->topic)
	ci->last_topic = sstrdup(c->topic);
    else
	ci->last_topic = NULL;
    strscpy(ci->last_topic_setter, c->topic_setter, NICKMAX);
    ci->last_topic_time = c->topic_time;
}

/*************************************************************************/

/* Restore the topic in a channel when it's created, if we should. */

void restore_topic(const char *chan)
{
    Channel *c = findchan(chan);
    ChannelInfo *ci = cs_findchan(chan);


    if (!c || !ci)
	return;

    if (!(ci->flags & CI_KEEPTOPIC)) {
       return;
    }

    if (c->topic)
	free(c->topic);
    if (ci->last_topic) {
	c->topic = sstrdup(ci->last_topic);
	strscpy(c->topic_setter, ci->last_topic_setter, NICKMAX);
	c->topic_time = ci->last_topic_time;
    } else {
	c->topic = NULL;
	strscpy(c->topic_setter, s_ChanServ, NICKMAX);
    }
    if (c->topic != NULL)
      if (TOPIC_PROBS == 1)
       send_cmd(MODE_SENDER, "TOPIC %s %s %lu :%s", chan,
		c->topic_setter, c->topic_time, c->topic ? c->topic :"");
      else
       send_cmd(MODE_SENDER, "TOPIC %s %s %lu :%s (%s)", chan,
		c->topic_setter, c->topic_time, c->topic ? c->topic :"",
                ci->last_topic_setter);

}

/*************************************************************************/

/* See if the topic is locked on the given channel, and return 1 (and fix
 * the topic) if so. */

int check_topiclock(const char *source, const char *chan)
{
    Channel *c = findchan(chan);
    ChannelInfo *ci = cs_findchan(chan);
    User *u = finduser(source);
    int ulev;

    if (!c || !ci || (!(ci->flags & CI_TOPICLOCK)))
	return 0;


    if ((u && (ci->topic_allow <= (ulev = get_access(u, ci))))
        && (!(ci->flags & CI_FREEZECHAN))) 
        return 0;


    if (c->topic)
	free(c->topic);
    if (ci->last_topic)
	c->topic = sstrdup(ci->last_topic);
    else
	c->topic = NULL;
    strscpy(c->topic_setter, ci->last_topic_setter, NICKMAX);
    c->topic_time = ci->last_topic_time;
    send_cmd(MODE_SENDER, "TOPIC %s %s %lu :%s", chan, c->topic_setter,
          c->topic_time, c->topic ? c->topic : "");
    return 1;
}

/*************************************************************************/

/* Remove all channels which have expired. */

void expire_chans()
{
    ChannelInfo *ci, *next;
    NickInfo *ni;
    int i;
    const time_t expire_time = CHANNEL_EXPIRE*24*60*60;
    time_t now = time(NULL);
    long count = 0;
    long xcount = 0;

    for (i = 33; i < 256; ++i) {
	for (ci = chanlists[i]; ci; ci = next) {
	    ++count;
	    next = ci->next;
            if ((now - ci->last_used >= expire_time && !(ci->flags & CI_VERBOTEN) && !(ci->flags & CI_HELDCHAN))
               || (!(ni = findnick(ci->founder)) && (!(ci->flags & CI_VERBOTEN)))) {
		++xcount;
                slog("CS X %s", ci->name);
                do_break_log("CS_X", "CS X %s", ci->name);
		log("Expiring channel %s", ci->name);
		delchan(ci);
	    }
	}
    }
  if (DISPLAY_UPDATES == 1)
     wallops(SERVER_NAME, "Completed Channel Expire (%d/%d)",xcount,count);
}

/*************************************************************************/

/* Remove a (deleted or expired) nickname from all channel access lists. */

void cs_remove_nick(const char *nick)
{
    int i, j;
    ChannelInfo *ci;
    ChanAccess *ca;

    for (i = 33; i < 256; i++) {
	for (ci = chanlists[i]; ci; ci = ci->next) {
	    for (ca = ci->access, j = ci->accesscount; j > 0; ca++, j--) {
		if (ca->in_use && stricmp(ca->name, nick) == 0)
		    ca->in_use = 0;
	    }
	}
    }
}

/*************************************************************************/
/*********************** ChanServ private routines ***********************/
/*************************************************************************/

/* Return the ChannelInfo structure for the given channel, or NULL if the
 * channel isn't registered. */

ChannelInfo *cs_findchan(const char *chan)
{
    ChannelInfo *ci;

    for (ci = chanlists[tolower(chan[1])]; ci; ci = ci->next) {
	if (stricmp(ci->name, chan) == 0)
	    return ci;
    }
    return NULL;
}

/*************************************************************************/

/* Insert a channel alphabetically into the database. */

static void alpha_insert_chan(ChannelInfo *ci)
{
    ChannelInfo *ci2, *ci3;
    char *chan = ci->name;

    for (ci3 = NULL, ci2 = chanlists[tolower(chan[1])];
			ci2 && stricmp(ci2->name, chan) < 0;
			ci3 = ci2, ci2 = ci2->next)
	;
    ci->prev = ci3;
    ci->next = ci2;
    if (!ci3)
	chanlists[tolower(chan[1])] = ci;
    else
	ci3->next = ci;
    if (ci2)
	ci2->prev = ci;
}

/*************************************************************************/

/* Add a channel to the database.  Returns a pointer to the new ChannelInfo
 * structure if the channel was successfully registered, NULL otherwise.
 * Assumes channel does not already exist. */

static ChannelInfo *makechan(const char *chan)
{
    ChannelInfo *ci;
    Channel *c;
    char *s, *end;

    ci = scalloc(sizeof(ChannelInfo), 1);
    strscpy(ci->name, chan, CHANMAX);
    ci->time_registered = time(NULL);
    reset_levels(ci);
    alpha_insert_chan(ci);
    return ci;
}

/*************************************************************************/

/* Remove a channel from the ChanServ database.  Return 1 on success, 0
 * otherwise. */

static int delchan(ChannelInfo *ci)
{
    int i;

    if (ci->next)
	ci->next->prev = ci->prev;
    if (ci->prev)
	ci->prev->next = ci->next;
    else
	chanlists[tolower(ci->name[1])] = ci->next;
    if (ci->desc)
	free(ci->desc);
    if (ci->mlock_key)
	free(ci->mlock_key);
    if (ci->last_topic)
	free(ci->last_topic);
    for (i = 0; i < ci->accesscount; ++i)
	free(ci->access[i].name);
    if (ci->access)
	free(ci->access);
    for (i = 0; i < ci->akickcount; ++i) {
	free(ci->akick[i].name);
	if (ci->akick[i].reason)
	    free(ci->akick[i].reason);
    }
    if (ci->akick)
	free(ci->akick);
    if (ci->levels)
	free(ci->levels);
    if (ci->welcome)
        free(ci->welcome);
    if (ci->hold)
        free(ci->hold);
    if (ci->mark)
        free(ci->mark);
    if (ci->freeze)
        free(ci->freeze);
    if (ci->forbid)
        free(ci->forbid);

    free(ci);
    return 1;
}

/*************************************************************************/

/* Reset defined access level values to their default state. */

static void reset_levels(ChannelInfo *ci)
{
    int i;

    if (ci->levels)
	free(ci->levels);
    ci->levels = smalloc(CA_SIZE * sizeof(*ci->levels));
    for (i = 0; def_levels[i][0] >= 0; i++)
	ci->levels[def_levels[i][0]] = def_levels[i][1];
}

/*************************************************************************/

/* Does the given user have founder access to the channel? */

static int is_founder(User *user, NickInfo *ni, ChannelInfo *ci, int set)
{
    struct u_chaninfolist *c;
    int i;

    if (is_identified(user, ci))
	return 1;

    ni = findnick(user->nick);
    if (set && (ci->flags & CI_UNSECURE)) {
       if (ni && (ni->flags & NI_RECOGNIZED)
              && (stricmp(ni->nick, ci->founder) == 0))
          return 1;
    }
    
    if (ni && ((ni->flags & NI_RECOGNIZED) && (!(ci->flags & CI_IDENT)))
           && !set && (stricmp(ni->nick, ci->founder) == 0))
         return 1;


    for (i=0; user->id_nicks[i] && i < MAX_IDS; i++) {
       ni = findnick(user->id_nicks[i]);

       if (ni && !set && ((ni->flags & NI_IDENTIFIED)
	   || ((ni->flags & NI_RECOGNIZED) && (!(ci->flags & CI_IDENT)))) 
	   && (stricmp(user->nick, ci->founder) == 0))
           return 1;
       else if (ni && set && ((ni->flags & NI_IDENTIFIED)
           || ((ni->flags & NI_RECOGNIZED) && (ci->flags & CI_UNSECURE)))
              && (stricmp(user->nick, ci->founder) == 0))
           return 1;
    }

    return 0;
}

/*************************************************************************/

/* Has the given user password-identified as founder for the channel? */

static int is_identified(User *user, ChannelInfo *ci)
{
    struct u_chaninfolist *c;

    for (c = user->founder_chans; c; c = c->next) {
	if (c->chan == ci)
	    return 1;
    }
    return 0;
}

/*************************************************************************/

/* Return the access level the given user has on the channel.  If the
 * channel doesn't exist, the user isn't on the access list, or the channel
 * is CS_IDENT and the user hasn't IDENTIFY'd with NickServ, return 0. */

int get_access(User *user, ChannelInfo *ci)
{
    NickInfo *ni, *ni2;
    ChanAccess *access;
    char **naccess;
    int i, i2, a=0, k=0, checked=0;

    if (!ci)
        return 0;


    ni = findnick(user->nick);

    for (access = ci->access, i2 = 0; i2 < ci->accesscount; ++access, ++i2) {
       if (ni && ((ni->flags & NI_RECOGNIZED) && (!(ci->flags & CI_IDENT)))
             && (!(ni->flags & NI_IDENTIFIED))
             && (stricmp(ni->nick, access->name) == 0)) {
                if (access->level > a)
                    a = access->level;
                if (access->level < 0)
                    k = access->level;
       } else if (!(ci->flags & CI_IDENT)) {
           if (match_usermask(access->name, user)) {
               if (access->level > a)
                  a = access->level;
               if (access->level < k)
                  k = access->level;
           } else {
               ni2 = findnick(access->name);
               if (ni2) {
                   for (naccess = ni2->access, i = 0; i < ni2->accesscount; ++naccess, ++i) {
                      if (is_on_access(user, ni2)) {
                          if (access->level > a)
                               a = access->level;
                      }
                   }
               }
           }
       }
    }

    if (is_founder(user, ni, ci, 0))
        return 15;

    for (i=0; i < MAX_IDS && user->id_nicks[i]; i++) {
      for (access = ci->access, i2 = 0; i2 < ci->accesscount; ++access, ++i2) {
          ni = findnick(user->id_nicks[i]);
          if (ni && (ni->flags & NI_IDENTIFIED)
              && stricmp(ni->nick, access->name) == 0) {
                   if (access->level > a)
                       a = access->level;
                   if (access->level < 0)
                       k = access->level;
          }
       }
    }

  if (a > 0)
     return a;
  else if ((a <= 0) && (k < 0))
     return k;
  else
     return 0;
}


/*************************************************************************/

/* Return 1 if the user's access level on the given channel falls into the
 * given category, 0 otherwise.  Note that this may seem slightly confusing
 * in some cases: for example, check_access(..., CA_NOJOIN) returns true if
 * the user does _not_ have access to the channel (i.e. matches the NOJOIN
 * criterion). */

int check_access(User *user, ChannelInfo *ci, int what)
{
    int level = get_access(user, ci);

    if (level == 15)
	return (what==CA_AUTODEOP || what==CA_NOJOIN) ? 0 : 1;
    if (ci->levels[what] == ACCESS_INVALID)
	return 0;
    if (what == CA_AUTODEOP || what == CA_NOJOIN)
	return level <= ci->levels[what];
    else
	return level >= ci->levels[what];
}

/*************************************************************************/
/*********************** ChanServ command routines ***********************/
/*************************************************************************/

static void do_ohelp(const char *source)
{
    char *cmd = strtok(NULL, "");
    char buf[256];
    int is_servadmin = is_services_oper(source);

    if (!cmd)
        slog("CS - %s requested ohelp: (index)", source);

    if (cmd) {

        slog("CS - %s requested ohelp: %s",
              source, cmd);

        if (stricmp(cmd, "DELETE") == 0) {
            notice_list(s_ChanServ, source, oper_delete_help);
            return;
        } else if (stricmp(cmd, "FLIST") == 0) {
            notice_list(s_ChanServ, source, flist_help);
            return;
        } else if (stricmp(cmd, "TOPIC") == 0) {
            notice_list(s_ChanServ, source, ostopic_help);
            return;
        } else if (stricmp(cmd, "GETPASS") == 0) {
            notice_list(s_ChanServ, source, getpass_help);
            return;
        } else if (stricmp(cmd, "FREEZE") == 0) {
            notice_list(s_ChanServ, source, freeze_help);
            return;
        } else if (stricmp(cmd, "FORBID") == 0) {
            notice_list(s_ChanServ, source, forbid_help);
            return;
        } else if (stricmp(cmd, "MARK") == 0) {
            notice_list(s_ChanServ, source, mark_help);
            return;
        } else if (stricmp(cmd, "HOLD") == 0) {
            notice_list(s_ChanServ, source, hold_help);
            return;
        } else if (stricmp(cmd, "MARK") == 0) {
            notice_list(s_ChanServ, source, mark_help);
            return;
        } else if (stricmp(cmd, "HOLD") == 0) {
            notice_list(s_ChanServ, source, hold_help);
            return;
        } else if (stricmp(cmd, "SETPASS") == 0) {
            notice_list(s_ChanServ, source, setpass_help);
            return;
        } else if (stricmp(cmd, "SETFOUNDER") == 0) {
            notice_list(s_ChanServ, source, setfounder_help);
            return;
        } else if (stricmp(cmd, "SUSPEND") == 0) {
            notice_list(s_ChanServ, source, suspend_help);
            return;
        } else if (stricmp(cmd, "WIPE") == 0) {
            notice_list(s_ChanServ, source, wipe_help);
            return;
        } else if (stricmp(cmd, "SENDPASS") == 0) {
            notice_list(s_ChanServ, source, sendpass_help);
            return;
        } else if (stricmp(cmd, "LEVEL") == 0) {
            notice_list(s_ChanServ, source, level_help);
            return;
        } else if (stricmp(cmd, "RACCESS") == 0) {
            notice_list(s_ChanServ, source, level_help);
            return;
        }

    }
    if (!cmd) {
        notice_list(s_ChanServ, source, cs_servhelper_help);
        if (is_oper(source))
                notice_list(s_ChanServ, source, cs_servircop_help);
        if (is_services_oper(source))
                notice_list(s_ChanServ, source, cs_servop_help);
        if (is_services_admin(source))
                notice_list(s_ChanServ, source, cs_servadmin_help);
        if (is_services_root(source))
                notice_list(s_ChanServ, source, cs_servroot_help);
    }

    notice(s_ChanServ, source,
       "\2END OF OHELP\2");
}

static void do_help(const char *source)
{
    char *cmd = strtok(NULL, "");
    char buf[256];

    snprintf(buf, sizeof(buf), "%s%s", s_ChanServ, cmd ? " " : "");
    strscpy(buf+strlen(buf), cmd ? cmd : "", sizeof(buf)-strlen(buf));
    helpserv(s_ChanServ, source, buf);
}

/*************************************************************************/

static void do_register(const char *source)
{
    char *chan = strtok(NULL, " ");
    char *pass = strtok(NULL, " ");
    char *desc = strtok(NULL, "");
    char *keepmlocks = sstrdup(DEF_MLOCKS);
    NickInfo *ni = findnick(source);
    ChannelInfo *ci;
    User *u = finduser(source);
    struct u_chaninfolist *c;
    int add = 1, x=0;

    if (readonly) {
	notice(s_ChanServ, source,
		"Sorry, channel registration is temporarily disabled.");
	return;
    }

    if (!desc) {

	notice(s_ChanServ, source,
                "Syntax: \2REGISTER\2 <#channel> <password> <description>");
	notice(s_ChanServ, source,
		"\2/msg %s HELP REGISTER\2 for more information.", s_ChanServ);
        return;

    } else if (*chan == '&') {

	notice(s_ChanServ, source, "Local channels cannot be registered.");

    } else if (chan[1] == 0) {

	notice(s_ChanServ, source, "\2%s\2 is not a valid channel name.", chan);

    } else if (!ni) {

	notice(s_ChanServ, source, "Your must register your nickname first.");
	notice(s_ChanServ, source,
		"\2/msg %s HELP\2 for information on registering nicknames.",
		s_NickServ);

    } else if (!(ni->flags & (NI_RECOGNIZED | NI_IDENTIFIED))) {

	notice(s_ChanServ, source,
		"Please identify with %s first, using the command:",
		s_NickServ);
	notice(s_ChanServ, source,
		"/msg %s IDENTIFY \37password\37", s_NickServ);

    } else if (ci = cs_findchan(chan)) {

	if (ci->flags & CI_VERBOTEN) {
            slog("CS *R %s (%s!%s@%s) [FORBIDDEN]",
                chan, source, u->username, u->host);
            do_break_log("CS_R", "CS *R %s (%s!%s@%s) [FORBIDDEN]",
                chan, source, u->username, u->host);
	    log("%s: Attempt to register FORBIDden channel %s by %s!%s@%s",
			s_ChanServ, chan, source, u->username, u->host);
	    notice(s_ChanServ, source,
			"Channel \2%s\2 may not be registered.", chan);
	} else {
	    notice(s_ChanServ, source,
			"Channel \2%s\2 already registered!", chan);
	}

    } else if (!u) {

	log("%s: Attempt to register channel %s from nonexistent nick %s",
						s_ChanServ, chan, source);
	notice(s_ChanServ, source, "Sorry, registration failed.");

    } else if (!is_chanop(source, chan)) {

	notice(s_ChanServ, source,
		"You must be a channel operator to register the channel.");

    } else if ((CHANNEL_MAXREG > 0) && (ni->channelcount >= CHANNEL_MAXREG)
		&& !(is_services_admin(source))) {
	notice(s_ChanServ, source,
		"Sorry, you have already %s your limit of \2%d\2 channels.",
		ni->channelcount > CHANNEL_MAXREG ? "exceeded" : "reached",
		CHANNEL_MAXREG);

    } else {

	if (ci = makechan(chan)) {
	    Channel *c;
	    if (!(c = findchan(chan))) {
		log("%s: Channel %s not found for REGISTER", s_ChanServ, chan);
		notice(s_ChanServ, source, "Sorry, registration failed.");
		return;
	    }
	    ci->flags = CI_KEEPTOPIC;
	    ci->last_used = ci->time_registered;
	    strscpy(ci->founder, source, NICKMAX);
	    strscpy(ci->founderpass, pass, PASSMAX);
#ifdef USE_ENCRYPTION
	    memset(pass, 0, strlen(pass));
	    if (encrypt_in_place(ci->founderpass, PASSMAX) < 0) {
		log("%s: Couldn't encrypt password for %s (register)",
			s_ChanServ, chan);
		notice(s_ChanServ, source, "Sorry, registration failed.");
		delchan(ci);
		return;
	    }
	    ci->flags |= CI_ENCRYPTEDPW;
#endif
	    ci->desc = sstrdup(desc);
            ci->welcome = NULL;
            ci->flags |= CI_MEMO_AV;

            if (DEF_MLOCKS == "+nt") {
              ci->mlock_on |= CMODE_N; ci->mlock_on |= CMODE_T;
            } else {
                while (*DEF_MLOCKS) {
                   x++;
                   switch(*DEF_MLOCKS++) {

                   case '+':
                      add = 1; break;

                   case '-':
                      add = 0; break;

                   case 'n':
                     if (add)
                        ci->mlock_on |= CMODE_N;
                     else
                        ci->mlock_off |= CMODE_N;
                     break;

                   case 't':
                     if (add)
                        ci->mlock_on |= CMODE_T;
                     else
                        ci->mlock_off |= CMODE_T;
                     break;

                   case 'i':
                     if (add)
                        ci->mlock_on |= CMODE_I;
                     else
                        ci->mlock_off |= CMODE_I;
                     break;

                   case 'm':
                     if (add)
                        ci->mlock_on |= CMODE_M;
                     else
                        ci->mlock_off |= CMODE_M;
                     break;

                   case 'p':
                     if (add)
                        ci->mlock_on |= CMODE_P;
                     else
                        ci->mlock_off |= CMODE_P;
                     break;

                   case 's':
                     if (add)
                        ci->mlock_on |= CMODE_S;
                     else
                        ci->mlock_off |= CMODE_S;
                     break;

                   case 'c':
                     if (add)
                        ci->mlock_on |= CMODE_C;
                     else
                        ci->mlock_off |= CMODE_C;
                     break;

/*                   case 'r':
                     if (add)
                        ci->mlock_on |= CMODE_r;
                     else
                        ci->mlock_off |= CMODE_r;
                     break;
*/
/* +r can not set with mlock */

                   case 'Q':
                     if (add)
                        ci->mlock_on |= CMODE_Q;
                     else
                        ci->mlock_off |= CMODE_Q;
                     break;
                   }
                }
            }
            DEF_MLOCKS = sstrdup(keepmlocks);
            ci->mlock_on |= CMODE_r;
/*
              ci->mlock_off |= CMODE_I; ci->mlock_off |= CMODE_M;
              ci->mlock_off |= CMODE_P; ci->mlock_off |= CMODE_S;
*/

	    if (c->topic) {
		ci->last_topic = sstrdup(c->topic);
		strscpy(ci->last_topic_setter, c->topic_setter, NICKMAX);
		ci->last_topic_time = c->topic_time;
	    }
	    if (ni->channelcount+1 > ni->channelcount)
		ni->channelcount++;
            slog("CS R %s (%s!%s@%s)",
                chan, u->nick, u->username, u->host);
            do_break_log("CS_R", "CS R %s (%s!%s@%s)",
                chan, u->nick, u->username, u->host);
	    log("%s: Channel %s registered by %s!%s@%s", s_ChanServ, chan,
			source, u->username, u->host);
            notice(s_ChanServ, source,  CS_REGISTER,  chan, source);
            notice(s_ChanServ, source,  CS_REGISTER2,  ci->founderpass);
            if (x > 1)
              notice(s_ChanServ, source,  CS_REGISTER3,  chan, DEF_MLOCKS);
            notice(s_ChanServ, source, MISC_OFHELP, AOCHANS);
	    if (REGNOTICE == 1)
	    wallops(s_ChanServ, "Channel \2%s\2 is registered under nick %s.", chan, source);
#if REG_SAVES > 0
            if (regcnt++ >= REG_SAVES) {
                wallops(SERVER_NAME,
                     "Automatic DataBase save on %d new registrations",
                            regcnt);
                save_data = 1;
            }
#endif
            show_next_db(source, s_ChanServ);
	} else {
	    notice(s_ChanServ, source, "Sorry, registration failed.");
	}
	c = smalloc(sizeof(*c));
	c->next = u->founder_chans;
	c->prev = NULL;
	if (u->founder_chans)
	    u->founder_chans->prev = c;
	u->founder_chans = c;
	c->chan = ci;

    }
    check_modes(chan);
}

/*************************************************************************/

static void do_identify(const char *source)
{
    char *chan = strtok(NULL, " ");
    char *pass = strtok(NULL, " ");
    ChannelInfo *ci;
    struct u_chaninfolist *c;
    User *u = finduser(source);

    if (!pass) {

	notice(s_ChanServ, source,
                "Syntax: \2IDENTIFY\2 <#channel> <password>");
	notice(s_ChanServ, source,
		"\2/msg %s HELP IDENTIFY\2 for more information.", s_ChanServ);

    } else if (!(ci = cs_findchan(chan))) {

	notice(s_ChanServ, source, "Channel %s is not registered.", chan);

    } else if (!u) {

	log("%s: IDENTIFY from nonexistent nick %s for %s",
						s_ChanServ, source, chan);
	notice(s_ChanServ, source, "Sorry, identification failed.");

    } else {

	int res;
	int is_force = (is_services_root(source) && !stricmp(pass, "force"));

	if (is_force || (res = check_password(pass, ci->founderpass)) == 1) {
	    if (!is_identified(u, ci)) {
		c = smalloc(sizeof(*c));
		c->next = u->founder_chans;
		c->prev = NULL;
		if (u->founder_chans)
		    u->founder_chans->prev = c;
		u->founder_chans = c;
		c->chan = ci;
 		change_cmode (s_ChanServ, chan, "+q", u->nick);
		log("%s: %s!%s@%s identified for %s %s", s_ChanServ,
			source, u->username, u->host, chan,
			is_force ? "[Force]" : "");
                if (EXTRA_SNOOP && !is_force) {
                     slog("CS I %s (%s!%s@%s) %s",
                           chan, u->nick, u->username, u->host,
      			   is_force ? "[Force]" : "");
                     do_break_log("CS_I", "CS I %s (%s!%s@%s) %s",
                             chan, u->nick, u->username, u->host,
   			     is_force ? "[Force]" : "");
                }
	    }
            notice(s_ChanServ, source,  CS_IDENTIFY,  chan);
	} else if (res < 0) {
            if (EXTRA_SNOOP) {
                 slog("CS *I %s (%s!%s@%s)",
                     chan, u->nick, u->username, u->host);
                 do_break_log("CS_I", "CS *I %s (%s!%s@%s)",
                     chan, u->nick, u->username, u->host);
            }
	    log("%s: check_password failed for %s", s_ChanServ, chan);
	    notice(s_ChanServ, source, "Sorry, identification failed.");
	} else {
	    log("%s: Failed IDENTIFY for %s by %s!%s@%s",
			s_ChanServ, chan, source, u->username, u->host);
            notice(s_ChanServ, source, CS_BADPASS);
	    bad_password(u, chan);
	}

    }
}
/*************************************************************************/

static void do_delete(const char *source)
{
    char *chan = strtok(NULL, " ");
    ChannelInfo *ci;
    User *u = finduser(source);
    int is_servop = is_services_root(source);

    if (readonly) {
        notice(s_ChanServ, source,
                "Sorry, channel deletion is temporarily disabled. (Read Only)");
	return;
    }

    if (!chan) {
       notice(s_ChanServ, source, "\2Delete\2: You MUST specify a channel");
       return;
    }

    if (!(ci = cs_findchan(chan))) {
        if (chan)
             notice(s_ChanServ, source, "Channel %s isn't registered.", chan);
        return;
    }
    delchan(ci);
    slog("CS +D %s (%s!%s@%s)", chan, source, u->username, u->host);
    do_break_log("CS_G", "CS +D %s (%s!%s@%s)", chan, source, u->username, u->host);
    log("%s: %s!%s@%s deleted channel %s", s_ChanServ,
            source, u->username, u->host, chan);
        notice(s_ChanServ, source, CS_DELETE, chan);
        wallops(s_ChanServ, "%s deleted channel \2%s\2", source, chan);
}

/*************************************************************************/

static void do_drop(const char *source)
{
    char *chan = strtok(NULL, " ");
    C_SuspendData *c_spn;
    ChannelInfo *ci;
    NickInfo *ni;
    Channel *c;
    User *u = finduser(source);
    Timeout *t;

    if (readonly) {
	notice(s_ChanServ, source,
		"Sorry, channel de-registration is temporarily disabled.");
	return;
    }

    if (!chan) {
        notice(s_ChanServ, source, "Syntax: \2DROP\2 <#channel>");
	notice(s_ChanServ, source,
		"\2/msg %s HELP DROP\2 for more information.", s_ChanServ);
        return;
    }

    c_spn = get_c_suspend(chan);

    if (!(ci = cs_findchan(chan))) {

	notice(s_ChanServ, source, "Channel %s is not registered.", chan);

    } else if (ci->flags & CI_FREEZECHAN) {
         notice(s_ChanServ, source, "Channel %s is currently frozen", chan);

    } else if (c_spn && c_spn->time > time(NULL)) {
       char *av[3];
       send_cmd(s_ChanServ, "JOIN %s", chan);
       send_cmd(s_ChanServ, "MODE %s +b *!*@*", chan);
       send_cmd(s_ChanServ, "MODE %s +O", chan);
       send_cmd(s_ChanServ, "KICK %s %s :Channel is currently suspended",
              chan, u->nick);
       av[0] = sstrdup(chan);
       av[1] = sstrdup(u->nick);
       av[2] = sstrdup("Channel is cirrently Suspended");
       do_kick(s_ChanServ, 3, av);
       free(av[0]); free(av[1]); free(av[2]);
      t = add_timeout(CHANNEL_INHABIT, timeout_leave, 0);
      t->data = sstrdup(chan);


    } else if (!u || !is_identified(u, ci)) {

            slog("CS *D %s (%s!%s@%s) [UNIDENTIFIED]",
                chan, u->nick, u->username, u->host);
            do_break_log("CS_O", "CS *D %s (%s!%s@%s) [UNIDENTIFIED]",
                chan, u->nick, u->username, u->host);

	notice(s_ChanServ, source,
		"Password authentication required for that command.");
	notice(s_ChanServ, source,
		"Retry after typing \2/msg %s IDENTIFY %s <password>.",
		s_ChanServ, chan);

    } else if (!u) {

	log("%s: DROP %s from nonexistent oper %s", s_ChanServ, chan, source);
	notice(s_ChanServ, source, "Can't find your user record!");

    } else {

	if (readonly) {
	    notice(s_ChanServ, source, "Warning: Services is in read-only "
	                               "mode; changes will not be saved.");
	}
	ni = findnick(ci->founder);
	if (ni) {
	    if (ni->channelcount > 0)
		ni->channelcount--;
	} else {
	    log("%s: Can't find founder nickname record for dropping %s",
			s_ChanServ, chan);
	}
	do_cs_part(chan);
        if ((c = findchan(chan))) {
            c->mode &= ~CMODE_r;
            send_cmd(s_ChanServ, "MODE %s -r", chan);
        }
	delchan(ci);
        slog("CS D %s (%s!%s@%s)",
              chan, u->nick, u->username, u->host);
        do_break_log("CS_O", "CS D %s (%s!%s@%s)",
              chan, u->nick, u->username, u->host);
	log("%s: Channel %s dropped by %s!%s@%s", s_ChanServ, chan,
			source, u->username, u->host);
        notice(s_ChanServ, source,  CS_DROP,  chan);
    }
}

/*************************************************************************/

/* Main SET routine.  Calls other routines as follows:
 *	do_set_command(User *command-sender, ChannelInfo *ci, char *param);
 * Additional parameters can be retrieved using strtok(NULL, toks).
 * (Exception is do_set_held(), which takes only source nick and channel
 * name.)
 */
static void do_set(const char *source)
{
    NickInfo *ni = findnick(source);
    char *chan = strtok(NULL, " ");
    char *cmd  = strtok(NULL, " ");
    char *param;
    ChannelInfo *ci;
    User *u = finduser(source);
    int i, access=0;

    if (readonly) {
	notice(s_ChanServ, source,
		"Sorry, channel option setting is temporarily disabled.");
	return;
    }

    if (cmd) {
	if (stricmp(cmd, "DESC") == 0 || stricmp(cmd, "TOPIC") == 0 ||
            stricmp(cmd,"WELCOME") == 0)
	    param = strtok(NULL, "");
        else if (stricmp(cmd, "FOUNDER") == 0)
            param = sstrdup(source);
	else
	    param = strtok(NULL, " ");

    } else
        param = NULL;

    if (!param) {

	notice(s_ChanServ, source,
                "Syntax: \2SET <#chan> <option> [<parameters>]");
		
	notice(s_ChanServ, source,
		"\2/msg %s HELP SET\2 for more information.", s_ChanServ);

    } else if (!ni) {
        return;

    } else if (!(ci = cs_findchan(chan))) {

	notice(s_ChanServ, source, "Channel %s is not registered.", chan);

    } else if (!u || (1 != (access = is_founder(u, ni, ci, 1))
                  && (get_access(u, ci) != 13))) {

	notice(s_ChanServ, source, "Access denied.");


    } else if (stricmp(cmd, "FOUNDER") == 0) {

        if (access != 1) {
	    notice(s_ChanServ, source,
		"Password authentication required for that command.");
	    notice(s_ChanServ, source,
		"Retry after typing \2/msg %s IDENTIFY %s \37password\37.",
		s_ChanServ, chan);
	} else {
	    do_set_founder(u, ci);
	}

    } else if (stricmp(cmd, "PASSWORD") == 0) {

        if (access != 1) {
	    notice(s_ChanServ, source,
		"Password authentication required for that command.");
	    notice(s_ChanServ, source,
		"Retry after typing \2/msg %s IDENTIFY %s \37password\37.",
		s_ChanServ, chan);
	} else {
	    do_set_password(u, ci, param);
	}

    } else if (stricmp(cmd, "WELCOME") == 0) {

        do_set_welcome(u, ci, param);

    } else if (stricmp(cmd, "DESC") == 0) {

	do_set_desc(u, ci, param);

    } else if (stricmp(cmd, "URL") == 0) {

	do_set_url(u, ci, param);

    } else if (stricmp(cmd, "EMAIL") == 0) {

	do_set_email(u, ci, param);

    } else if (stricmp(cmd, "TOPIC") == 0) {

	do_set_topic(u, ci, param);

    } else if (stricmp(cmd, "MEMO") == 0) {

        do_memolev(u, ci, param);

    } else if (stricmp(cmd, "MLOCK") == 0) {

	do_set_mlock(u, ci, param);

    } else if (stricmp(cmd, "KEEPTOPIC") == 0) {

	do_set_keeptopic(u, ci, param);

    } else if (stricmp(cmd, "TOPICLOCK") == 0) {

	do_set_topiclock(u, ci, param);

    } else if (stricmp(cmd, "PRIVATE") == 0) {

	do_set_private(u, ci, param);

    } else if (stricmp(cmd, "AUTH") == 0) {

	do_set_auth(u, ci, param);

    } else if (stricmp(cmd, "OPGUARD") == 0) {

        do_set_opguard(u, ci, param);

    } else if (stricmp(cmd, "JOIN") == 0) {
    	if ((CJOIN==0) && (!is_services_root(source))) {
           notice(s_ChanServ,source,"This options is disabled.");
           return;
	}
        do_set_chanjoin(u, ci, param);

    } else if (stricmp(cmd, "RESTRICTED") == 0) {

	do_set_restricted(u, ci, param);

    } else if (stricmp(cmd, "IDENT") == 0) {

	do_set_ident(u, ci, param);

    } else if (stricmp(cmd, "UNSECURE") == 0) {

	do_set_unsecure(u, ci, param);

    } else {

	notice(s_ChanServ, source,
			"Unknown SET option \2%s\2.", strupper(cmd));

    }
}

/*************************************************************************/

static void do_set_founder(User *u, ChannelInfo *ci)
{
    NickInfo *ni = findnick(u->nick);

    if (!ni) {
       notice(s_ChanServ, u->nick,
           "You must be using a registered nick");
    } else if (!(ni->flags & NI_IDENTIFIED)) {
       notice(s_ChanServ, u->nick,
           "You must first identify to your nick before claiming  founder");
    } else {

        slog("CS F %s (%s!%s@%s) [%s -> %s]",
           ci->name, u->nick, u->username, u->host, ci->founder, u->nick);
        do_break_log("CS_O", "CS F %s (%s!%s@%s) [%s -> %s]",
           ci->name, u->nick, u->username, u->host, ci->founder, u->nick);
        strscpy(ci->founder, u->nick, NICKMAX);
        notice(s_ChanServ, u->nick,
		"You are now the Founder of %s.", ci->name);
        show_u_next_db(u, s_ChanServ);
    }
}

/*************************************************************************/

static void do_set_password(User *u, ChannelInfo *ci, char *param)
{
#ifdef USE_ENCRYPTION
    int len = strlen(param);
    if (len > PASSMAX)
	len = PASSMAX;
    if (encrypt(param, len, ci->founderpass, PASSMAX) < 0) {
	memset(param, 0, strlen(param));
        slog("CS *P %s (%s!%s@%s)",
          ci->name, u->nick, u->username, u->host);
        do_break_log("CS_O", "CS *P %s (%s!%s@%s)",
          ci->name, u->nick, u->username, u->host);
	log("%s: Failed to encrypt password for %s (set)",
		s_ChanServ, ci->name);
	notice(s_ChanServ, u->nick, "Sorry, couldn't set password.");
	return;
    }
    memset(param, 0, strlen(param));
    notice(s_ChanServ, u->nick, "%s password changed.", ci->name);
        slog("CS P %s (%s!%s@%s)",
          ci->name, u->nick, u->username, u->host);
        do_break_log("CS_O", "CS P %s (%s!%s@%s)",
          ci->name, u->nick, u->username, u->host);
#else
    strscpy(ci->founderpass, param, PASSMAX);
    slog("CS P %s (%s!%s@%s)",
       ci->name, u->nick, u->username, u->host);
    do_break_log("CS_O", "CS P %s (%s!%s@%s)",
       ci->name, u->nick, u->username, u->host);

    notice(s_ChanServ, u->nick,
		"%s password changed to \2%s\2.", ci->name, ci->founderpass);
    show_u_next_db(u, s_ChanServ);
#endif
}

/*************************************************************************/

static void do_set_welcome(User *u, ChannelInfo *ci, char *param)
{
    if (ci->welcome)
        free(ci->welcome);
    if (!stricmp(param,"none")) {
	ci->welcome = NULL;
        notice(s_ChanServ, u->nick, "Welcome message disabled for %s.",
            ci->name);
        slog("CS -W %s (%s!%s@%s)",
                ci->name, u->nick, u->username, u->host);
    } else {
        ci->welcome = sstrdup(param);
        notice(s_ChanServ, u->nick,
                    "Welcome Message for %s changed to \"\2%s\2\".", ci->name, ci->welcome);
        slog("CS W %s (%s!%s@%s) [%s]",
           ci->name, u->nick, u->username, u->host, ci->welcome);

        show_u_next_db(u, s_ChanServ);
    }
}

/*************************************************************************/

static void do_set_desc(User *u, ChannelInfo *ci, char *param)
{
    free(ci->desc);
    ci->desc = sstrdup(param);
    notice(s_ChanServ, u->nick,
		"Description of %s changed to \2%s\2.", ci->name, param);
    show_u_next_db(u, s_ChanServ);
}

/*************************************************************************/

static void do_set_url(User *u, ChannelInfo *ci, char *param)
{
    if (ci->url)
	free(ci->url);
    ci->url = sstrdup(param);
    notice(s_ChanServ, u->nick,
		"URL for %s changed to \2%s\2.", ci->name, param);
    show_u_next_db(u, s_ChanServ);
}

/*************************************************************************/

static void do_set_email(User *u, ChannelInfo *ci, char *param)
{
    if (!strchr(param, '@')) {
        notice(s_ChanServ, u->nick,
             "E-Mail addressed must contain an '@'");
        return;
    }

    if (ci->email)
	free(ci->email);
    ci->email = sstrdup(param);
    notice(s_ChanServ, u->nick,
		"E-mail address for %s changed to \2%s\2.", ci->name, param);
    show_u_next_db(u, s_ChanServ);
}

/*************************************************************************/

static void do_set_topic(User *u, ChannelInfo *ci, char *param)
{
    char *source = u->nick, *chan = ci->name;
    Channel *c = findchan(chan);
    int ulev2 = 0, clev = 0;

    if (!c) {
	log("%s: SET TOPIC for %s from %s: channel not found!",
		s_ChanServ, chan, source);
	notice(s_ChanServ, source, "Sorry, can't set topic.");
	return;
    }

    if (ci->topic_allow > 0) {
        if (ci->topic_allow > (ulev2 = get_access(u, ci))) {
             notice(s_ChanServ, source,
                 "Access Denied");

             return;
        }
    }


    if (ci->last_topic)
	free(ci->last_topic);
    if (*param)
	ci->last_topic = sstrdup(param);
    else
	ci->last_topic = NULL;

    if (c->topic) {
	free(c->topic);
	--c->topic_time;	/* to get around TS8 */
    } else
	c->topic_time = time(NULL);

    if (*param)
	c->topic = sstrdup(param);
    else
	c->topic = NULL;

    strscpy(ci->last_topic_setter, source, NICKMAX);
    strscpy(c->topic_setter, source, NICKMAX);
    ci->last_topic_time = c->topic_time;

    send_cmd(MODE_SENDER, "TOPIC %s %s %lu :%s",
                chan, source, c->topic_time, param);

}

/*************************************************************************/

static void do_set_mlock(User *u, ChannelInfo *ci, char *param)
{
    char *source = u->nick, *chan = ci->name;
    char *s, modebuf[32], *end, c;
    int add = -1;	/* 1 if adding, 0 if deleting, -1 if neither */

    ci->mlock_on = ci->mlock_off = ci->mlock_limit = 0;
    if (ci->mlock_key) {
	free(ci->mlock_key);
	ci->mlock_key = NULL;
    }
        ci->mlock_on |= CMODE_r;

    while (*param) {
	if (*param != '+' && *param != '-' && add < 0) {
	    ++param;
	    continue;
	}
	switch (c = *param++) {
	  case '+':
	    add = 1;
	    break;
	  case '-':
	    add = 0;
	    break;
	  case 'i':
	    if (add)
		ci->mlock_on |= CMODE_I;
	    else
		ci->mlock_off |= CMODE_I;
	    break;
	  case 'k':
	    if (add) {
		if (!(s = strtok(NULL, " "))) {
		    notice(s_ChanServ, source,
				"Parameter required for MLOCK +k.");
		    break;
		}
		ci->mlock_key = sstrdup(s);
	    } else {
		if (ci->mlock_key) {
		    free(ci->mlock_key);
		    ci->mlock_key = NULL;
		}
		ci->mlock_off |= CMODE_K;
	    }
	    break;
	  case 'l':
	    if (add) {
		if (!(s = strtok(NULL, " "))) {
		    notice(s_ChanServ, source,
				"Parameter required for MLOCK +l.");
		    break;
		}
		if (atol(s) <= 0) {
		    notice(s_ChanServ, source,
			"Parameter for MLOCK +l must be a positive number.");
		    break;
		}
		ci->mlock_limit = atol(s);
	    } else
		ci->mlock_off |= CMODE_L;
	    break;
	  case 'm':
	    if (add)
		ci->mlock_on |= CMODE_M;
	    else
		ci->mlock_off |= CMODE_M;
	    break;
	  case 'n':
	    if (add)
		ci->mlock_on |= CMODE_N;
	    else
		ci->mlock_off |= CMODE_N;
	    break;
	  case 'p':
	    if (add)
		ci->mlock_on |= CMODE_P;
	    else
		ci->mlock_off |= CMODE_P;
	    break;
	  case 's':
	    if (add)
		ci->mlock_on |= CMODE_S;
	    else
		ci->mlock_off |= CMODE_S;
	    break;
	  case 't':
	    if (add)
		ci->mlock_on |= CMODE_T;
	    else
		ci->mlock_off |= CMODE_T;
	    break;
	  case 'c':
	    if (add)
		ci->mlock_on |= CMODE_C;
	    else
		ci->mlock_off |= CMODE_C;
	    break;
          case 'R':
	    if (add)
                ci->mlock_on |= CMODE_RN;
	    else
                ci->mlock_off |= CMODE_RN;
	    break;
          case 'V':
	    if (add)
                ci->mlock_on |= CMODE_NI;
	    else
                ci->mlock_off |= CMODE_NI;
	    break;
          case 'Q':
	   if (is_services_admin(source))
	    if (add)
                ci->mlock_on |= CMODE_Q;
	    else
                ci->mlock_off |= CMODE_Q;
	   break;
/*
          case 'S':
	    if (add)
                ci->mlock_on |= CMODE_ST;
	    else
                ci->mlock_off |= CMODE_ST;
	    break;
          case 'K':
	    if (add)
                ci->mlock_on |= CMODE_NK;
	    else
                ci->mlock_off |= CMODE_NK;
	    break;
          case 'G':
	    if (add)
                ci->mlock_on |= CMODE_G;
	    else
                ci->mlock_off |= CMODE_G;
	    break;
*/
          case 'O':
	   if (is_services_admin(source))
	    if (add)
                ci->mlock_on |= CMODE_OP;
	    else
                ci->mlock_off |= CMODE_OP;
	   break;
          case 'A':
	   if (is_services_admin(source))
	    if (add)
                ci->mlock_on |= CMODE_AD;
	    else
                ci->mlock_off |= CMODE_AD;
	   break;
          case 'H':
	   if (is_services_admin(source))
            if (add)
                ci->mlock_on |= CMODE_H;
            else
                ci->mlock_off |= CMODE_H;
            break;
/* No No No why you can remove or set +r while it just for services */
          case 'r':
//	    if (add)
//              ci->mlock_on |= CMODE_r;
//	    else
//              ci->mlock_off |= CMODE_r;
	    break;

	  default:
	    notice(s_ChanServ, source,
			"Unknown mode character \2%c\2 ignored.", c);
	    break;
	} /* switch */
    } /* while (*param) */

    end = modebuf;
    *end = 0;
    if (ci->mlock_on)
        end += sprintf(end,"+%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s",
				(ci->mlock_on & CMODE_I) ? "i" : "",
                                (ci->mlock_on & CMODE_H) ? "H" : "",
				(ci->mlock_key         ) ? "k" : "",
				(ci->mlock_limit       ) ? "l" : "",
				(ci->mlock_on & CMODE_M) ? "m" : "",
				(ci->mlock_on & CMODE_N) ? "n" : "",
				(ci->mlock_on & CMODE_P) ? "p" : "",
				(ci->mlock_on & CMODE_S) ? "s" : "",
				(ci->mlock_on & CMODE_T) ? "t" : "",
				(ci->mlock_on & CMODE_C) ? "c" : "",
			/*	(ci->mlock_on & CMODE_G) ? "G" : "",
				(ci->mlock_on & CMODE_ST) ? "S" : "",
				(ci->mlock_on & CMODE_NK) ? "K" : "", */
				(ci->mlock_on & CMODE_NI) ? "V" : "",
				(ci->mlock_on & CMODE_AD) ? "A" : "",
				(ci->mlock_on & CMODE_OP) ? "O" : "",
				(ci->mlock_on & CMODE_Q)  ? "Q" : "",
				(ci->mlock_on & CMODE_RN) ? "R" : "",
                                (ci->mlock_on & CMODE_r) ? "r" : "");
    if (ci->mlock_off)
        end += sprintf(end, "-%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s",
				(ci->mlock_off & CMODE_I) ? "i" : "",
                                (ci->mlock_off & CMODE_H) ? "H" : "",
				(ci->mlock_off & CMODE_K) ? "k" : "",
				(ci->mlock_off & CMODE_L) ? "l" : "",
				(ci->mlock_off & CMODE_M) ? "m" : "",
				(ci->mlock_off & CMODE_N) ? "n" : "",
				(ci->mlock_off & CMODE_P) ? "p" : "",
				(ci->mlock_off & CMODE_S) ? "s" : "",
				(ci->mlock_off & CMODE_T) ? "t" : "",
				(ci->mlock_off & CMODE_C) ? "c" : "",
			/*	(ci->mlock_off & CMODE_G) ? "G" : "",
				(ci->mlock_off & CMODE_ST) ? "S" : "",
				(ci->mlock_off & CMODE_NK) ? "K" : "", */
				(ci->mlock_off & CMODE_NI) ? "V" : "",
				(ci->mlock_off & CMODE_AD) ? "A" : "",
				(ci->mlock_off & CMODE_OP) ? "O" : "",
				(ci->mlock_off & CMODE_Q) ? "Q" : "",
				(ci->mlock_off & CMODE_RN) ? "R" : "",
                                (ci->mlock_off & CMODE_r) ? "r" : "");
    if (*modebuf) {
	notice(s_ChanServ, source,
		"Mode lock on channel %s changed to \2%s\2.", chan, modebuf);
        show_u_next_db(u, s_ChanServ);
    } else {
	notice(s_ChanServ, source,
		"Mode lock on channel %s removed.", chan);
    show_u_next_db(u, s_ChanServ);
    }
  check_modes(chan);
}

/*************************************************************************/

static void do_set_keeptopic(User *u, ChannelInfo *ci, char *param)
{
    char *source = u->nick, *chan = ci->name;

    if (stricmp(param, "ON") == 0) {

	ci->flags |= CI_KEEPTOPIC;
	notice(s_ChanServ, source,
		"Topic retention option is now \2ON\2.");
        show_u_next_db(u, s_ChanServ);

    } else if (stricmp(param, "OFF") == 0) {

	ci->flags &= ~CI_KEEPTOPIC;
	notice(s_ChanServ, source,
		"Topic retention option is now \2OFF\2.");
        show_u_next_db(u, s_ChanServ);

    } else {

	notice(s_ChanServ, source,
                "Syntax: \2SET\2 %s KEEPTOPIC [ON|OFF]", chan);
	notice(s_ChanServ, source,
		"\2/msg %s HELP SET KEEPTOPIC\2 for more information.",
		s_ChanServ);
    }
}

/*************************************************************************/

static void do_set_topiclock(User *u, ChannelInfo *ci, char *param)
{
    char *source = u->nick, *chan = ci->name;

    if (stricmp(param, "VOP") == 0) {

	ci->flags |= CI_TOPICLOCK;
        ci->topic_allow = 3;
	notice(s_ChanServ, source,
                "Topic lock option is now set to \2VOP\2.");
        show_u_next_db(u, s_ChanServ);

    } else if (stricmp(param, "HOP") == 0) {

	ci->flags |= CI_TOPICLOCK;
        ci->topic_allow = 4;
	notice(s_ChanServ, source,
                "Topic lock option is now set to \2HOP\2.");
        show_u_next_db(u, s_ChanServ);

    } else if (stricmp(param, "AOP") == 0) {

	ci->flags |= CI_TOPICLOCK;
        ci->topic_allow = 5;
	notice(s_ChanServ, source,
                "Topic lock option is now set to \2AOP\2.");
        show_u_next_db(u, s_ChanServ);

    } else if (stricmp(param, "SOP") == 0) {

	ci->flags |= CI_TOPICLOCK;
        ci->topic_allow = 10;
	notice(s_ChanServ, source,
                "Topic lock option is now set to \2SOP\2.");
        show_u_next_db(u, s_ChanServ);

    } else if ((stricmp(param, "CFOUNDER") == 0) || (stricmp(param,"CF") == 0)) {

	ci->flags |= CI_TOPICLOCK;
        ci->topic_allow = 13;
	notice(s_ChanServ, source,
                "Topic lock option is now set to \2Co-Founder\2.");
        show_u_next_db(u, s_ChanServ);

    } else if (stricmp(param, "FOUNDER") == 0) {

	ci->flags |= CI_TOPICLOCK;
        ci->topic_allow = 15;
	notice(s_ChanServ, source,
                "Topic lock option is now set to \2FOUNDER\2.");
        show_u_next_db(u, s_ChanServ);

    } else if (stricmp(param, "OFF") == 0) {

	ci->flags &= ~CI_TOPICLOCK;
        ci->topic_allow = 0;
	notice(s_ChanServ, source,
		"Topic lock option is now \2OFF\2.");
        show_u_next_db(u, s_ChanServ);

    } else {

	notice(s_ChanServ, source,
                "Syntax: \2SET\2 %s TOPICLOCK [VOP|HOP|AOP|SOP|CFOUNDER|FOUNDER|OFF]", chan);
	notice(s_ChanServ, source,
		"\2/msg %s HELP SET TOPICLOCK\2 for more information.",	s_ChanServ);
    }
}

/*************************************************************************/

static void do_set_private(User *u, ChannelInfo *ci, char *param)
{
    char *source = u->nick;
    char *chan = ci->name;

    if (stricmp(param, "ON") == 0) {

	ci->flags |= CI_PRIVATE;
	notice(s_ChanServ, source, "Private option is now \2ON\2.");
        show_u_next_db(u, s_ChanServ);

    } else if (stricmp(param, "OFF") == 0) {

	ci->flags &= ~CI_PRIVATE;
	notice(s_ChanServ, source, "Private option is now \2OFF\2.");
        show_u_next_db(u, s_ChanServ);

    } else {

	notice(s_ChanServ, source,
                "Syntax: \2SET\2 %s PRIVATE [ON|OFF]", chan);
	notice(s_ChanServ, source,
		"\2/msg %s HELP SET PRIVATE\2 for more information.",
		s_ChanServ);
    }
}

/*************************************************************************/

static void do_set_opguard(User *u, ChannelInfo *ci, char *param)
{
    char *source = u->nick, *chan = ci->name;

    if (!stricmp(param, "ON")) {

	ci->flags |= CI_OPGUARD;
	notice(s_ChanServ, source,
		"Secure ops option for %s is now \2ON\2.",chan);
        show_u_next_db(u, s_ChanServ);

    } else if (!stricmp(param, "OFF")) {

	ci->flags &= ~CI_OPGUARD;
	notice(s_ChanServ, source,
		"Op Guard option for %s is now \2OFF\2.",chan);
        show_u_next_db(u, s_ChanServ);

    } else {

	notice(s_ChanServ, source,
                "Syntax: \2SET\2 %s OPGUARD [ON|OFF]", chan);
	notice(s_ChanServ, source,
		"\2/msg %s HELP SET OPGUARD\2 for more information.",
		s_ChanServ);
    }
}

/*************************************************************************/

static void do_set_restricted(User *u, ChannelInfo *ci, char *param)
{
    char *source = u->nick, *chan = ci->name;

    if (stricmp(param, "ON") == 0) {

	ci->flags |= CI_RESTRICTED;
	notice(s_ChanServ, source,
		"Restricted access option is now \2ON\2.");
        show_u_next_db(u, s_ChanServ);

    } else if (stricmp(param, "OFF") == 0) {

	ci->flags &= ~CI_RESTRICTED;
	notice(s_ChanServ, source,
		"Restricted access option is now \2OFF\2.");
        show_u_next_db(u, s_ChanServ);

    } else {

	notice(s_ChanServ, source,
                "Syntax: \2SET\2 %s RESTRICTED [ON|OFF]", chan);
	notice(s_ChanServ, source,
		"\2/msg %s HELP SET RESTRICTED\2 for more information.",
		s_ChanServ);
    }
}

/*************************************************************************/

static void do_set_ident(User *u, ChannelInfo *ci, char *param)
{
    char *source = u->nick, *chan = ci->name;

    if (stricmp(param, "ON") == 0) {

	ci->flags |= CI_IDENT;
	notice(s_ChanServ, source,
		"Ident option is now \2ON\2.");

        if (ci->flags & CI_UNSECURE) {
             ci->flags &= ~CI_UNSECURE;
             notice(s_ChanServ, source,
                 "UNSECURE setting automatically disabled when IDENT set to ON");
        }

        show_u_next_db(u, s_ChanServ);

    } else if (stricmp(param, "OFF") == 0) {

	ci->flags &= ~CI_IDENT;
	notice(s_ChanServ, source,
		"Ident option is now \2OFF\2.");
        show_u_next_db(u, s_ChanServ);

    } else {

	notice(s_ChanServ, source,
                "Syntax: \2SET\2 %s IDENT [ON|OFF]", chan);
	notice(s_ChanServ, source,
		"\2/msg %s HELP SET IDENT\2 for more information.",
		s_ChanServ);
    }
}

/*************************************************************************/

static void do_set_unsecure(User *u, ChannelInfo *ci, char *param)
{
    char *source = u->nick, *chan = ci->name;

    if (stricmp(param, "ON") == 0) {

        if (ci->flags & CI_IDENT) {
            notice(s_ChanServ, source,
                 "Can't set UNSECURE when IDENT is already set!");
            return;
        }

	ci->flags |= CI_UNSECURE;
	notice(s_ChanServ, source,
		"Channel %s is now UNSECURE.", chan);
        show_u_next_db(u, s_ChanServ);

    } else if (stricmp(param, "OFF") == 0) {

	ci->flags &= ~CI_UNSECURE;
	notice(s_ChanServ, source,
		"Channel %s now has regular security options enabled",
                         chan);
        show_u_next_db(u, s_ChanServ);

    } else {

	notice(s_ChanServ, source,
                "Syntax: \2SET\2 %s UNSECURE [ON|OFF]", chan);
	notice(s_ChanServ, source,
		"\2/msg %s HELP SET UNSECURE\2 for more information.",
		s_ChanServ);
    }
}

/*************************************************************************/
static void do_cfounder(const char *source)
{
    char *chan = strtok(NULL, " ");
    char *cmd  = strtok(NULL, " ");
    char *nick = strtok(NULL, " ");
    char *s    = "1";
    ChannelInfo *ci;
    NickInfo *ni;
    User *u;
    short ulev;
    int is_list = (cmd && !stricmp(cmd, "LIST"));
    ChanAccess *access;
    int i, f=0, c=0, z;

    if (!cmd || ((!stricmp(cmd, "LIST") == 0) && (!stricmp(cmd, "WIPE") == 0)
             && (!stricmp(cmd, "CLEAN") == 0) && !nick)) {
         notice(s_ChanServ, source,
           "Syntax: \2CFOUNDER\2 <#channel> [ADD|DEL|LIST|WIPE|CLEAN] [<nick>]");
         return;
    }

    if (!(ci = cs_findchan(chan))) {

        notice(s_ChanServ, source, "Channel %s is not registered.", chan);

    } else if (!(u = finduser(source))) {

        notice(s_ChanServ, source, "Access denied.");
        log("%s: ACCESS command from nonexistent user %s", s_ChanServ, source);

    } else if (stricmp(cmd, "LIST") == 0) {

        if (!(is_oper(source)) &&
             (((is_list && !check_access(u, ci, CA_ACCESS_LIST))
             || (!is_list && !check_access(u, ci, CA_ACCESS_CHANGE)))
             && !(is_list && is_services_admin(source)))) {

           notice(s_ChanServ, source, "Access Denied");

        } else {

	int j = 0;
        notice(s_ChanServ, source, "CFOUNDER list for \2%s\2:", chan);
        for (access = ci->access, i = 0; i < ci->accesscount; ++access, ++i) {
            if ((nick && !match_wild(nick, access->name)) ||
                                        (13 != access->level))
                continue;
            if (ni = findnick(access->name))
                s = ni->last_usermask;
            else
                s = NULL;
	    ++j;
            notice(s_ChanServ, source, " %d)  %s%s%s%s",
                        j, access->name,
                        s ? " (" : "", s ? s : "", s ? ")" : "");
        }
	notice(s_ChanServ,source,"End of List");
      }

    } else if (((is_list && !check_access(u, ci, CA_ACCESS_LIST))
             || (!is_list && !check_access(u, ci, CA_ACCESS_CHANGE)))
            && !(is_list && is_services_root(source)))
    {
        notice(s_ChanServ, source, "Access denied.");

    } else if (s && 13 >= (ulev = get_access(u, ci))) {

        notice(s_ChanServ, source, "Access denied.");

    } else if (stricmp(cmd, "ADD") == 0) {
        char *mask = nick;

        if (readonly) {
            notice(s_ChanServ, source,
                "Sorry, channel access list modification is temporarily disabled");
            return;
        }
        ni = findnick(nick);
        if (!ni) {
            if (NS_DACCESS == 1) {
           	 notice(s_ChanServ, source,
                "Adding access mask have been disabled by Admin");
           	 notice(s_ChanServ, source, "Nick %s is not registered.", nick);
            	return;
           }
           if (strlen(mask) > 60) {
               notice(s_ChanServ, source, "ChanServ access mask too large!");
               wallops(s_ChanServ, "%s tried to add a HUGE mask to the %s Co-Founder list", source, chan);
               return;

           } else if (mask && !strchr(mask, '@') || !strchr(mask, '!')) {
               notice(s_ChanServ, source, "ChanServ access mask must be in nick!user@host or nick format");
               notice(s_ChanServ, source,
                    "\2/msg %s HELP CFOUNDER\2 for more information.", s_ChanServ);
               return;

          } else if (ci->flags & CI_IDENT) {
              notice(s_ChanServ, source, "Channel %s has the security option set. You may not add unregistered nicks when this option is enabled", ci->name);
              return;
          }

        }
        if (ni && ni->forbid) {
           notice(s_ChanServ, source, "Forbidden nick %s not permitted to be added to channel access lists", ni->nick);
           return;
        }
        for (access = ci->access, i = 0; i < ci->accesscount; ++access, ++i) {
            if (access->in_use && stricmp(access->name, nick) == 0) {
                if (access->level >= ulev) {
                    notice(s_ChanServ, source, "Permission denied.");
                    return;
                }
                if (access->level == 13) {
                    notice(s_ChanServ, source,
                        "\2%s\2 is already a CFOUNDER on \2%s\2.",
                        access->name, chan);
                    return;
                }
                if (ni && (ni->flags & NI_NOOP)) {
                    notice(s_ChanServ, source, "Since %s is already on the %s access list, regardless of NOOP setting, adding to Co-Founder list",
                       ni->nick, chan);
		    if (finduser(ni->nick))
		    if (ni->flags & NI_IDENTIFIED) 
			if (ni->flags & NI_AUTH)
		        notice(s_ChanServ, ni->nick, "\2%s\2 try to added you as Co Founder in channel \2%s\2.", source, chan);
		}
                else {
                	if (access->level == 3) {
                    	notice(s_ChanServ, source,
                        	"Move \2%s\2 from VOP to CFOUNDER on \2%s\2.",
                        	access->name, chan);
	                } else if (access->level == 5) {
                    	notice(s_ChanServ, source,
                        	"Move \2%s\2 from AOP to CFOUNDER on \2%s\2.",
                        	access->name, chan);
	                } else if (access->level == 4) {
                    	notice(s_ChanServ, source,
                        	"Move \2%s\2 from HOP to CFOUNDER on \2%s\2.",
                        	access->name, chan);
	                } else if (access->level == 10) {
                    	notice(s_ChanServ, source,
                        	"Move \2%s\2 from SOP to CFOUNDER on \2%s\2.",
                        	access->name, chan);
			} else 
                   notice(s_ChanServ, source,
                        "\2%s\2 was changed to a Co-Founder on \2%s\2.",
                        access->name, chan);
                access->level = 13;
	if (finduser(ni->nick))
	    if (ni->flags & NI_IDENTIFIED) {
		if (is_on_chan(ni->nick, chan))
			if (!is_chanhop(ni->nick, chan))
			change_cmode (s_ChanServ, chan, "+o", ni->nick);
		if (ni->flags & NI_AUTH)
	        notice(s_ChanServ, ni->nick, "\2%s\2 added you as Co Founder in channel \2%s\2.", source, chan);
	    }
		}
                show_next_db(source, s_ChanServ);
                return;
            }
        }
        if ((CHAN_ACCESS_MAX > 0) && (ci->accesscount > CHAN_ACCESS_MAX-1)) {
	    notice(s_ChanServ, source,
                    "Sorry, you may only have %d channel access enteries.",
                    CHAN_ACCESS_MAX);
            wallops(s_ChanServ, "%s tried to add another nick to the access list on %s",
                   source, chan);
	    return;
	}
        if (ni && (ni->flags & NI_NOOP)) {
             notice(s_ChanServ, source, "Sorry, %s has the NOOP option set.", ni->nick);
	    if (finduser(ni->nick))
	    if (ni->flags & NI_IDENTIFIED) 
		if (ni->flags & NI_AUTH)
	        notice(s_ChanServ, ni->nick, "\2%s\2 try to added you as Co Founder in channel \2%s\2.", source, chan);
             return;
        }
        for (access = ci->access, i = 0; i < ci->accesscount; ++access, ++i) {
            if (!access->in_use)
                break;
        }
        if (i == ci->accesscount) {
            ++ci->accesscount;
            ci->access =
                srealloc(ci->access, sizeof(ChanAccess) * ci->accesscount);
            access = &ci->access[ci->accesscount-1];
        }
        if (ni)
           access->name = sstrdup(ni->nick);
        else
           access->name = sstrdup(mask);
        access->in_use = 1;
        access->level = 13;
        notice(s_ChanServ, source,
                "\2%s\2 added to \2%s\2 as a Co-Founder.",
                access->name, chan);
        show_next_db(source, s_ChanServ);
	if (finduser(ni->nick))
	    if (ni->flags & NI_IDENTIFIED) {
		if (is_on_chan(ni->nick, chan))
			if (!is_chanhop(ni->nick, chan))
			change_cmode (s_ChanServ, chan, "+o", ni->nick);
		if (ni->flags & NI_AUTH)
	        notice(s_ChanServ, ni->nick, "\2%s\2 added you as Co Founder in channel \2%s\2.", source, chan);
	    }
    } else if (stricmp(cmd, "DEL") == 0) {

        if (readonly) {
            notice(s_ChanServ, source,
                "Sorry, channel access list modification is temporarily disabled");
            return;
        }
        /* Special case: is it a number?  Only do search if it isn't. */
        if (strspn(nick, "1234567890") == strlen(nick) &&
                                (i = atoi(nick)) > 0 && i <= ci->accesscount) {
            int j=0;
            for (access = ci->access, z = 0; z < ci->accesscount;
			++access, ++z) {
                if (!(13 == access->level))
                    continue;
	        j += 1;
                if (i == j) {
		    access = &ci->access[z];
		    break;
		}
            }
            if (!(i == j)) {
                notice(s_ChanServ, source,
                        "No such entry (#%d) on %s access list.", i, chan);
                return;
            }
        } else {
/*
            ni = findnick(nick);
            if (!ni) {
                notice(s_ChanServ, source,
                        "Nick \2%s\2 not registered.", nick);
                return;
            }
*/
            for (access = ci->access, z = 0; z < ci->accesscount;
                                                        ++access, ++z) {
                if ((stricmp(access->name, nick) == 0)
                        && (access->level == 13))
                    break;
            }
            if (z == ci->accesscount) {
                notice(s_ChanServ, source,
                        "\2%s\2 not found on %s Co-Founder list.", nick, chan);
                return;
            }
        }
        if (get_access(u, ci) <= access->level) {
            notice(s_ChanServ, source, "Permission denied.");
        } else {
            notice(s_ChanServ, source,
                "\2%s\2 deleted from %s access list.", access->name, chan);
            show_next_db(source, s_ChanServ);
            free(access->name);
            access->in_use = 0;
            access->name = NULL;
              --ci->accesscount;
              if (z < ci->accesscount)
                  bcopy(access+1, access, sizeof(ChanAccess)
                                           * (ci->accesscount - z));

        }

    } else if (stricmp(cmd, "CLEAN") == 0) {
        int check = 0;
        check = ci->accesscount;
        for (access = ci->access, i = 0; i < check; ++access, ++i) {
              if ((13 == access->level) && (!strchr(access->name, '@'))
                 && (!findnick(access->name))) {
                     free(access->name);
                     access->name = NULL;
                     access->in_use = 0;
                     ++f;
                     --ci->accesscount;
                     if ((i-f) < ci->accesscount)
                         bcopy(access+1, access, sizeof(ChanAccess)
                                        * (ci->accesscount - (i-f)));
                     --access;
              }
        }
        if (!(f > 0))
            notice(s_ChanServ, source,
               "No Expired nicknames found in the CFOUNDER list on \2%s\2",
                chan);
        else {
           notice(s_ChanServ, source,
               "%d expired nicknames in the CFOUNDER list on \2%s\2 removed",
                f, chan);
           show_next_db(source, s_ChanServ);
       }
       f=0;


    } else if (stricmp(cmd, "WIPE") == 0) {
        int check = 0;
        check = ci->accesscount;
        for (access = ci->access, i = 0; i < check; ++access, ++i) {
              if (13 == access->level) {
                  free(access->name);
                  access->name = NULL;
                  access->in_use = 0;
                  ++f;
                  --ci->accesscount;
                  if ((i-f) < ci->accesscount)
                      bcopy(access+1, access, sizeof(ChanAccess)
                                              * (ci->accesscount - (i-f)));
                  --access;
              }
        }
        if (!(f > 0))
            notice(s_ChanServ, source,
               "No enteries found in the CFOUNDER list on \2%s\2",
                chan);

        else {
           notice(s_ChanServ, source,
               "%d enter%s in the CFOUNDER list on \2%s\2 removed",
                f, f==1 ? "y" : "ies", chan);
           show_next_db(source, s_ChanServ);
       }
       f=0;


    } else {
        notice(s_ChanServ, source,
                "Syntax: \2CFOUNDER\2 %s [ADD|DEL|LIST|WIPE|CLEAN] [<nick>]\2",chan);
        notice(s_ChanServ, source,
                "\2/msg %s HELP SOP\2 for more information.", s_ChanServ);
    }
}

static void do_avoice(const char *source)
{
    char *chan = strtok(NULL, " ");
    char *cmd  = strtok(NULL, " ");
    char *nick = strtok(NULL, " ");
    char *s    = "1";
    ChannelInfo *ci;
    NickInfo *ni;
    User *u;
    short ulev;
    int is_list = (cmd && !stricmp(cmd, "LIST"));
    ChanAccess *access;
    int i, f=0, c=0, z;

    if (!cmd || ((!stricmp(cmd, "LIST") == 0) && (!stricmp(cmd, "WIPE") == 0)
             && (!stricmp(cmd, "CLEAN") == 0) && !nick)) {
         notice(s_ChanServ, source,
           "Syntax: \2VOP\2 <#channel> [ADD|DEL|LIST|WIPE|CLEAN] [<nick>]");
         return;
    }

    if (!(ci = cs_findchan(chan))) {

        notice(s_ChanServ, source, "Channel %s is not registered.", chan);

    } else if (!(u = finduser(source))) {

        notice(s_ChanServ, source, "Access denied.");
        log("%s: ACCESS command from nonexistent user %s", s_ChanServ, source);

    } else if (stricmp(cmd, "LIST") == 0) {

        if (!(is_oper(source)) &&
             (((is_list && !check_access(u, ci, CA_ACCESS_LIST))
             || (!is_list && !check_access(u, ci, CA_ACCESS_CHANGE)))
             && !(is_list && is_services_admin(source)))) {

           notice(s_ChanServ, source, "Access Denied");

        } else {

	int j=0;
        notice(s_ChanServ, source, "VOP list for \2%s\2:", chan);
        for (access = ci->access, i = 0; i < ci->accesscount; ++access, ++i) {
            if ((nick && !match_wild(nick, access->name)) ||
                                        (3 != access->level))
                continue;
            if (ni = findnick(access->name))
                s = ni->last_usermask;
            else
                s = NULL;
	    ++j;
            notice(s_ChanServ, source, " %d)  %s%s%s%s",
                        j, access->name,
                        s ? " (" : "", s ? s : "", s ? ")" : "");
        }
        notice(s_ChanServ,source,"End of List");
      }

    } else if (((is_list && !check_access(u, ci, CA_ACCESS_LIST))
             || (!is_list && !check_access(u, ci, CA_ACCESS_CHANGE)))
            && !(is_list && is_services_admin(source)))
    {
        notice(s_ChanServ, source, "Access denied.");

    } else if (s && 5 >= (ulev = get_access(u, ci))) {

        notice(s_ChanServ, source, "Access denied.");

    } else if (stricmp(cmd, "ADD") == 0) {
        char *mask = nick;

        if (readonly) {
            notice(s_ChanServ, source,
                "Sorry, channel access list modification is temporarily disabled");
            return;
        }
        ni = findnick(nick);
        if (!ni) {
            if (NS_DACCESS == 1) {
           	 notice(s_ChanServ, source,
                "Adding access mask have been disabled by Admin");
           	 notice(s_ChanServ, source, "Nick %s is not registered.", nick);
            	return;
           }
           if (strlen(mask) > 60) {
               notice(s_ChanServ, source, "ChanServ access mask too large!");
               wallops(s_ChanServ, "%s tried to add a HUGE mask to the %s VOP list", source, chan);
               return;

           } else if (mask && !strchr(mask, '@') || !strchr(mask, '!')) {
               notice(s_ChanServ, source, "ChanServ access mask must be in nick!user@host or nick format");
               notice(s_ChanServ, source,
                    "\2/msg %s HELP VOP\2 for more information.", s_ChanServ);
               return;

          } else if (ci->flags & CI_IDENT) {
              notice(s_ChanServ, source, "Channel %s has the security option set. You may not add unregistered nicks when this option is enabled", ci->name);
              return;
          }

        }
        if (ni && ni->forbid) {
           notice(s_ChanServ, source, "Forbidden nick %s not permitted to be added to channel access lists", ni->nick);
           return;
        }
        for (access = ci->access, i = 0; i < ci->accesscount; ++access, ++i) {
            if (access->in_use && stricmp(access->name, nick) == 0) {
                if (access->level >= ulev) {
                    notice(s_ChanServ, source, "Permission denied.");
                    return;
                }
                if (access->level == 3) {
                    notice(s_ChanServ, source,
                        "\2%s\2 is already an Auto-Voice on \2%s\2.",
                        access->name, chan);
                    return;
                }
                  
                if (ni && (ni->flags & NI_NOOP)) {
                    notice(s_ChanServ, source, "Since %s is already on the %s access list, regardless of NOOP setting, adding to VOP list",
                       ni->nick, chan);
		    if (finduser(ni->nick))
		    if (ni->flags & NI_IDENTIFIED) 
			if (ni->flags & NI_AUTH)
		        notice(s_ChanServ, ni->nick, "\2%s\2 try to added you as VOP in channel \2%s\2.", source, chan);
		}
/*	        if (ni->flags & NI_AUTH) {
		    do_add_auth(u->nick, access->name, chan, 3);
		    return;
		} */

                else {
                	if (access->level == 4) {
                    	notice(s_ChanServ, source,
                        	"Move \2%s\2 from HOP to VOP on \2%s\2.",
                        	access->name, chan);
	                } else if (access->level == 5) {
                    	notice(s_ChanServ, source,
                        	"Move \2%s\2 from AOP to VOP on \2%s\2.",
                        	access->name, chan);
	                } else if (access->level == 10) {
                    	notice(s_ChanServ, source,
                        	"Move \2%s\2 from SOP to VOP on \2%s\2.",
                        	access->name, chan);
	                } else if (access->level == 13) {
                    	notice(s_ChanServ, source,
                        	"Move \2%s\2 from CFOUNDER to VOP on \2%s\2.",
                        	access->name, chan);
			} else 
		        notice(s_ChanServ, source,
                        "\2%s\2 was changed to an VOP on \2%s\2.",
                        access->name, chan);
                access->level = 3;
	if (finduser(ni->nick))
		    if (ni->flags & NI_IDENTIFIED) {
			if (is_on_chan(ni->nick, chan))
			if (!is_voiced(ni->nick, chan))
				change_cmode (s_ChanServ, chan, "+v", ni->nick);
			if (ni->flags & NI_AUTH)
		        notice(s_ChanServ, ni->nick, "\2%s\2 added you as VOP in channel \2%s\2.", source, chan);
		    }
		}
                show_next_db(source, s_ChanServ);
                return;
            }
        }
        if ((CHAN_ACCESS_MAX > 0) && (ci->accesscount > CHAN_ACCESS_MAX-1)) {
	    notice(s_ChanServ, source,
                    "Sorry, you may only have %d channel access enteries.",
                    CHAN_ACCESS_MAX);
            wallops(s_ChanServ, "%s tried to add another nick to the access list on %s",
                   source, chan);
	    return;
	}
        if (ni && (ni->flags & NI_NOOP)) {
             notice(s_ChanServ, source, "Sorry, %s has the NOOP Option set",
                       ni->nick);
	    if (finduser(ni->nick))
	    if (ni->flags & NI_IDENTIFIED) 
		if (ni->flags & NI_AUTH)
	        notice(s_ChanServ, ni->nick, "\2%s\2 try to added you as VOP in channel \2%s\2.", source, chan);
             return;
        }

/*	if (ni->flags & NI_AUTH) {
	    do_add_auth(access->name, ni->nick, chan, 3);
	    return;
	} */

        for (access = ci->access, i = 0; i < ci->accesscount; ++access, ++i) {
            if (!access->in_use)
                break;
        }
        if (i == ci->accesscount) {
            ++ci->accesscount;
            ci->access =
                srealloc(ci->access, sizeof(ChanAccess) * ci->accesscount);
            access = &ci->access[ci->accesscount-1];
        }
        if (ni)
           access->name = sstrdup(ni->nick);
        else
           access->name = sstrdup(mask);
        access->in_use = 1;
        access->level = 3;
        notice(s_ChanServ, source,
                "\2%s\2 added to \2%s\2 as an Auto-Voice.",
                access->name, chan);
        show_next_db(source, s_ChanServ);
	if (finduser(ni->nick))
		    if (ni->flags & NI_IDENTIFIED) {
			if (is_on_chan(ni->nick, chan))
			if (!is_voiced(ni->nick, chan))
			change_cmode (s_ChanServ, chan, "+v", ni->nick);
			if (ni->flags & NI_AUTH)
		        notice(s_ChanServ, ni->nick, "\2%s\2 added you as VOP in channel \2%s\2.", source, chan);
		    }

    } else if (stricmp(cmd, "DEL") == 0) {

        if (readonly) {
            notice(s_ChanServ, source,
                "Sorry, channel access list modification is temporarily disabled");
            return;
        }
        /* Special case: is it a number?  Only do search if it isn't. */
        if (strspn(nick, "1234567890") == strlen(nick) &&
                                (i = atoi(nick)) > 0 && i <= ci->accesscount) {
            int j=0;
            for (access = ci->access, z = 0; z < ci->accesscount;
                        ++access, ++z) {
                if (!(3 == access->level))
                    continue;
                j += 1;
                if (i == j) {
                    access = &ci->access[z];
                    break;
                }
            }
            if (!(i == j)) {
                notice(s_ChanServ, source,
                        "No such entry (#%d) on %s access list.", i, chan);
                return;
            }
        } else {
/*
            ni = findnick(nick);
            if (!ni) {
                notice(s_ChanServ, source,
                        "Nick \2%s\2 not registered.", nick);
                return;
            }
*/
            for (access = ci->access, z = 0; z < ci->accesscount;
                                                        ++access, ++z) {
                if ((stricmp(access->name, nick) == 0)
                        && (access->level == 3))
                    break;
            }
            if (z == ci->accesscount) {
                notice(s_ChanServ, source,
                        "\2%s\2 not found on %s VOP list.", nick, chan);
                return;
            }
        }
        if (get_access(u, ci) <= access->level) {
            notice(s_ChanServ, source, "Permission denied.");
        } else {
            notice(s_ChanServ, source,
                "\2%s\2 deleted from %s access list.", access->name, chan);
            show_next_db(source, s_ChanServ);
            free(access->name);
            access->in_use = 0;
            access->name = NULL;
              --ci->accesscount;
              if (z < ci->accesscount)
                  bcopy(access+1, access, sizeof(ChanAccess)
                                           * (ci->accesscount-z));
        }


    } else if (stricmp(cmd, "CLEAN") == 0) {
        int check = 0;
        check = ci->accesscount;
        for (access = ci->access, i = 0; i < check; ++access, ++i) {
              if ((3 == access->level) && (!strchr(access->name, '@'))
                 && (!findnick(access->name))) {
                     free(access->name);
                     access->name = NULL;
                     access->in_use = 0;
                     ++f;
                     --ci->accesscount;
                     if ((i-f) < ci->accesscount)
                         bcopy(access+1, access, sizeof(ChanAccess)
                                        * (ci->accesscount - (i-f)));
                     --access;
              }
        }
        if (!(f > 0))
            notice(s_ChanServ, source,
               "No Expired nicknames found in the VOP list on \2%s\2",
                chan);
        else {
           notice(s_ChanServ, source,
               "%d expired nicknames in the VOP list on \2%s\2 removed",
                f, chan);
           show_next_db(source, s_ChanServ);
       }
       f=0;


    } else if (stricmp(cmd, "WIPE") == 0) {
        int check = 0;
        check = ci->accesscount;
        for (access = ci->access, i = 0; i < check; ++access, ++i) {
              if (3 == access->level) {
                  free(access->name);
                  access->name = NULL;
                  access->in_use = 0;
                  ++f;
                  --ci->accesscount;
                  if ((i-f) < ci->accesscount)
                      bcopy(access+1, access, sizeof(ChanAccess)
                                              * (ci->accesscount - (i-f)));
                  --access;
              }
        }
        if (!(f > 0))
            notice(s_ChanServ, source,
               "No enteries found in the VOP list on \2%s\2",
                chan);

        else {
           notice(s_ChanServ, source,
               "%d entr%s in the VOP list on \2%s\2 removed",
                f, f==1 ? "y" : "ies", chan);
           show_next_db(source, s_ChanServ);
       }
       f=0;


    } else {

        notice(s_ChanServ, source,
                "Syntax: \2VOP\2 %s [ADD|DEL|LIST|WIPE|CLEAN] [<nick>]",chan);
        notice(s_ChanServ, source,
                "\2/msg %s HELP VOP\2 for more information.", s_ChanServ);
    }
}

static void do_hop(const char *source)
{
    char *chan = strtok(NULL, " ");
    char *cmd  = strtok(NULL, " ");
    char *nick = strtok(NULL, " ");
    char *s    = "1";
    ChannelInfo *ci;
    NickInfo *ni;
    User *u;
    short ulev;
    int is_list = (cmd && !stricmp(cmd, "LIST"));
    ChanAccess *access;
    int i, f=0, c=0, z;

    if (!cmd || ((!stricmp(cmd, "LIST") == 0) && (!stricmp(cmd, "WIPE") == 0)
             && (!stricmp(cmd, "CLEAN") == 0) && !nick)) {
         notice(s_ChanServ, source,
           "Syntax: \2HOP\2 <#channel> [ADD|DEL|LIST|WIPE|CLEAN] [<nick>]");
         return;
    }

    if (!(ci = cs_findchan(chan))) {

        notice(s_ChanServ, source, "Channel %s is not registered.", chan);

    } else if (!(u = finduser(source))) {

        notice(s_ChanServ, source, "Access denied.");
        log("%s: ACCESS command from nonexistent user %s", s_ChanServ, source);

    } else if (stricmp(cmd, "LIST") == 0) {

        if (!(is_oper(source)) &&
             (((is_list && !check_access(u, ci, CA_ACCESS_LIST))
             || (!is_list && !check_access(u, ci, CA_ACCESS_CHANGE)))
             && !(is_list && is_services_admin(source)))) {

           notice(s_ChanServ, source, "Access Denied");

        } else {

	int j=0;
        notice(s_ChanServ, source, "HOP list for \2%s\2:", chan);
        for (access = ci->access, i = 0; i < ci->accesscount; ++access, ++i) {
            if ((nick && !match_wild(nick, access->name)) ||
                                        (4 != access->level))
                continue;
            if (ni = findnick(access->name))
                s = ni->last_usermask;
            else
                s = NULL;
	    ++j;
            notice(s_ChanServ, source, " %d)  %s%s%s%s",
                        j, access->name,
                        s ? " (" : "", s ? s : "", s ? ")" : "");
        }
        notice(s_ChanServ,source,"End of List");
      }

    } else if (((is_list && !check_access(u, ci, CA_ACCESS_LIST))
             || (!is_list && !check_access(u, ci, CA_ACCESS_CHANGE)))
            && !(is_list && is_services_admin(source)))
    {
        notice(s_ChanServ, source, "Access denied.");

    } else if (s && 5 >= (ulev = get_access(u, ci))) {

        notice(s_ChanServ, source, "Access denied.");

    } else if (stricmp(cmd, "ADD") == 0) {
        char *mask = nick;

        if (readonly) {
            notice(s_ChanServ, source,
                "Sorry, channel access list modification is temporarily disabled");
            return;
        }
        ni = findnick(nick);
        if (!ni) {
            if (NS_DACCESS == 1) {
           	 notice(s_ChanServ, source,
                "Adding access mask have been disabled by Admin");
           	 notice(s_ChanServ, source, "Nick %s is not registered.", nick);
            	return;
           }
           if (strlen(mask) > 60) {
               notice(s_ChanServ, source, "ChanServ access mask too large!");
               wallops(s_ChanServ, "%s tried to add a HUGE mask to the %s HOP list", source, chan);
               return;

           } else if (mask && !strchr(mask, '@') || !strchr(mask, '!')) {
               notice(s_ChanServ, source, "ChanServ access mask must be in nick!user@host or nick format");
               notice(s_ChanServ, source,
                    "\2/msg %s HELP HOP\2 for more information.", s_ChanServ);
               return;

          } else if (ci->flags & CI_IDENT) {
              notice(s_ChanServ, source, "Channel %s has the security option set. You may not add unregistered nicks when this option is enabled", ci->name);
              return;
          }

        }
        if (ni && ni->forbid) {
           notice(s_ChanServ, source, "Forbidden nick %s not permitted to be added to channel access lists", ni->nick);
           return;
        }
        for (access = ci->access, i = 0; i < ci->accesscount; ++access, ++i) {
            if (access->in_use && stricmp(access->name, nick) == 0) {
                if (access->level >= ulev) {
                    notice(s_ChanServ, source, "Permission denied.");
                    return;
                }
                if (access->level == 4) {
                    notice(s_ChanServ, source,
                        "\2%s\2 is already an Auto-Halfop on \2%s\2.",
                        access->name, chan);
                    return;
                }
                if (ni && (ni->flags & NI_NOOP)) {
                    notice(s_ChanServ, source, "Since %s is already on the %s access list, regardless of NOOP setting, adding to HOP list",
                       ni->nick, chan);
		    if (finduser(ni->nick))
		    if (ni->flags & NI_IDENTIFIED) 
			if (ni->flags & NI_AUTH)
		        notice(s_ChanServ, ni->nick, "\2%s\2 try to added you as HOP in channel \2%s\2.", source, chan);
		}
                else {
                	if (access->level == 3) {
                    	notice(s_ChanServ, source,
                        	"Move \2%s\2 from VOP to HOP on \2%s\2.",
                        	access->name, chan);
	                } else if (access->level == 5) {
                    	notice(s_ChanServ, source,
                        	"Move \2%s\2 from AOP to HOP on \2%s\2.",
                        	access->name, chan);
	                } else if (access->level == 10) {
                    	notice(s_ChanServ, source,
                        	"Move \2%s\2 from SOP to HOP on \2%s\2.",
                        	access->name, chan);
	                } else if (access->level == 13) {
                    	notice(s_ChanServ, source,
                        	"Move \2%s\2 from CFOUNDER to HOP on \2%s\2.",
                        	access->name, chan);
			} else 
                   notice(s_ChanServ, source,
                        "\2%s\2 was changed to an HOP on \2%s\2.",
                        access->name, chan);
                access->level = 4;

	if (finduser(ni->nick))
	    if (ni->flags & NI_IDENTIFIED) {
		if (is_on_chan(ni->nick, chan))
			if (!is_chanhop(ni->nick, chan))
			change_cmode (s_ChanServ, chan, "+h", ni->nick);
		if (ni->flags & NI_AUTH)
	        notice(s_ChanServ, ni->nick, "\2%s\2 added you as HOP in channel \2%s\2.", source, chan);
	    }
		}
                show_next_db(source, s_ChanServ);
                return;
            }
        }
        if ((CHAN_ACCESS_MAX > 0) && (ci->accesscount > CHAN_ACCESS_MAX-1)) {
	    notice(s_ChanServ, source,
                    "Sorry, you may only have %d channel access enteries.",
                    CHAN_ACCESS_MAX);
            wallops(s_ChanServ, "%s tried to add another nick to the access list on %s",
                   source, chan);
	    return;
	}
        if (ni && (ni->flags & NI_NOOP)) {
             notice(s_ChanServ, source, "Sorry, %s has the NOOP Option set",
                       ni->nick);
	    if (finduser(ni->nick))
	    if (ni->flags & NI_IDENTIFIED) 
		if (ni->flags & NI_AUTH)
	        notice(s_ChanServ, ni->nick, "\2%s\2 try to added you as HOP in channel \2%s\2.", source, chan);
             return;
        }


        for (access = ci->access, i = 0; i < ci->accesscount; ++access, ++i) {
            if (!access->in_use)
                break;
        }
        if (i == ci->accesscount) {
            ++ci->accesscount;
            ci->access =
                srealloc(ci->access, sizeof(ChanAccess) * ci->accesscount);
            access = &ci->access[ci->accesscount-1];
        }
        if (ni)
           access->name = sstrdup(ni->nick);
        else
           access->name = sstrdup(mask);
        access->in_use = 1;
        access->level = 4;
        notice(s_ChanServ, source,
                "\2%s\2 added to \2%s\2 as an Auto-Halfop.",
                access->name, chan);
        show_next_db(source, s_ChanServ);
	if (finduser(ni->nick))
	    if (ni->flags & NI_IDENTIFIED) {
		if (is_on_chan(ni->nick, chan))
			if (!is_chanhop(ni->nick, chan))
			change_cmode (s_ChanServ, chan, "+h", ni->nick);
		if (ni->flags & NI_AUTH)
	        notice(s_ChanServ, ni->nick, "\2%s\2 added you as HOP in channel \2%s\2.", source, chan);
	    }
    } else if (stricmp(cmd, "DEL") == 0) {

        if (readonly) {
            notice(s_ChanServ, source,
                "Sorry, channel access list modification is temporarily disabled");
            return;
        }
        /* Special case: is it a number?  Only do search if it isn't. */
        if (strspn(nick, "1234567890") == strlen(nick) &&
                                (i = atoi(nick)) > 0 && i <= ci->accesscount) {
            int j=0;
            for (access = ci->access, z = 0; z < ci->accesscount;
                        ++access, ++z) {
                if (!(4 == access->level))
                    continue;
                j += 1;
                if (i == j) {
                    access = &ci->access[z];
                    break;
                }
            }
            if (!(i == j)) {
                notice(s_ChanServ, source,
                        "No such entry (#%d) on %s access list.", i, chan);
                return;
            }
        } else {
/*
            ni = findnick(nick);
            if (!ni) {
                notice(s_ChanServ, source,
                        "Nick \2%s\2 not registered.", nick);
                return;
            }
*/
            for (access = ci->access, z = 0; z < ci->accesscount;
                                                        ++access, ++z) {
                if ((stricmp(access->name, nick) == 0)
                        && (access->level == 4))
                    break;
            }
            if (z == ci->accesscount) {
                notice(s_ChanServ, source,
                        "\2%s\2 not found on %s HOP list.", nick, chan);
                return;
            }
        }
        if (get_access(u, ci) <= access->level) {
            notice(s_ChanServ, source, "Permission denied.");
        } else {
            notice(s_ChanServ, source,
                "\2%s\2 deleted from %s access list.", access->name, chan);
            show_next_db(source, s_ChanServ);
            free(access->name);
            access->in_use = 0;
            access->name = NULL;
              --ci->accesscount;
              if (z < ci->accesscount)
                  bcopy(access+1, access, sizeof(ChanAccess)
                                           * (ci->accesscount-z));
        }


    } else if (stricmp(cmd, "CLEAN") == 0) {
        int check = 0;
        check = ci->accesscount;
        for (access = ci->access, i = 0; i < check; ++access, ++i) {
              if ((4 == access->level) && (!strchr(access->name, '@'))
                 && (!findnick(access->name))) {
                     free(access->name);
                     access->name = NULL;
                     access->in_use = 0;
                     ++f;
                     --ci->accesscount;
                     if ((i-f) < ci->accesscount)
                         bcopy(access+1, access, sizeof(ChanAccess)
                                        * (ci->accesscount - (i-f)));
                     --access;
              }
        }
        if (!(f > 0))
            notice(s_ChanServ, source,
               "No Expired nicknames found in the HOP list on \2%s\2",
                chan);
        else {
           notice(s_ChanServ, source,
               "%d expired nicknames in the HOP list on \2%s\2 removed",
                f, chan);
           show_next_db(source, s_ChanServ);
       }
       f=0;


    } else if (stricmp(cmd, "WIPE") == 0) {
        int check = 0;
        check = ci->accesscount;
        for (access = ci->access, i = 0; i < check; ++access, ++i) {
              if (4 == access->level) {
                  free(access->name);
                  access->name = NULL;
                  access->in_use = 0;
                  ++f;
                  --ci->accesscount;
                  if ((i-f) < ci->accesscount)
                      bcopy(access+1, access, sizeof(ChanAccess)
                                              * (ci->accesscount - (i-f)));
                  --access;
              }
        }
        if (!(f > 0))
            notice(s_ChanServ, source,
               "No enteries found in the HOP list on \2%s\2",
                chan);

        else {
           notice(s_ChanServ, source,
               "%d entr%s in the HOP list on \2%s\2 removed",
                f, f==1 ? "y" : "ies", chan);
           show_next_db(source, s_ChanServ);
       }
       f=0;


    } else {

        notice(s_ChanServ, source,
                "Syntax: \2HOP\2 %s [ADD|DEL|LIST|WIPE|CLEAN] [<nick>]",chan);
        notice(s_ChanServ, source,
                "\2/msg %s HELP HOP\2 for more information.", s_ChanServ);
    }
}



static void do_sop(const char *source)
{
    char *chan = strtok(NULL, " ");
    char *cmd  = strtok(NULL, " ");
    char *nick = strtok(NULL, " ");
    char *s    = "1";
    ChannelInfo *ci;
    NickInfo *ni;
    User *u;
    short ulev;
    int is_list = (cmd && !stricmp(cmd, "LIST"));
    ChanAccess *access;
    int i, f=0, c=0, z;

    if (!cmd || ((!stricmp(cmd, "LIST") == 0) && (!stricmp(cmd, "WIPE") == 0)
             && (!stricmp(cmd, "CLEAN") == 0) && !nick)) {
         notice(s_ChanServ, source,
           "Syntax: \2SOP\2 <#channel> [ADD|DEL|LIST|WIPE|CLEAN] [<nick>]");
         return;
    }

    if (!(ci = cs_findchan(chan))) {

        notice(s_ChanServ, source, "Channel %s is not registered.", chan);

    } else if (!(u = finduser(source))) {

        notice(s_ChanServ, source, "Access denied.");
        log("%s: ACCESS command from nonexistent user %s", s_ChanServ, source);

    } else if (stricmp(cmd, "LIST") == 0) {

        if (!(is_oper(source)) &&
             (((is_list && !check_access(u, ci, CA_ACCESS_LIST))
             || (!is_list && !check_access(u, ci, CA_ACCESS_CHANGE)))
             && !(is_list && is_services_admin(source)))) {

           notice(s_ChanServ, source, "Access Denied");

        } else {


	int j=0;
        notice(s_ChanServ, source, "SOP list for \2%s\2:", chan);
        for (access = ci->access, i = 0; i < ci->accesscount; ++access, ++i) {
            if ((nick && !match_wild(nick, access->name)) ||
                                        (10 != access->level))
                continue;
            if (ni = findnick(access->name))
                s = ni->last_usermask;
            else
                s = NULL;
	    ++j;
            notice(s_ChanServ, source, " %d)  %s%s%s%s",
                        j, access->name,
                        s ? " (" : "", s ? s : "", s ? ")" : "");
        }
        notice(s_ChanServ,source,"End of List");

      }

    } else if (((is_list && !check_access(u, ci, CA_ACCESS_LIST))
             || (!is_list && !check_access(u, ci, CA_ACCESS_CHANGE)))
            && !(is_list && is_services_admin(source)))
    {
        notice(s_ChanServ, source, "Access denied.");

    } else if (s && 10 >= (ulev = get_access(u, ci))) {

        notice(s_ChanServ, source, "Access denied.");

    } else if (stricmp(cmd, "ADD") == 0) {
        char *mask = nick;

        if (readonly) {
            notice(s_ChanServ, source,
                "Sorry, channel access list modification is temporarily disabled");
            return;
        }
        ni = findnick(nick);
        if (!ni) {
            if (NS_DACCESS == 1) {
           	 notice(s_ChanServ, source,
                "Adding access mask have been disabled by Admin");
           	 notice(s_ChanServ, source, "Nick %s is not registered.", nick);
            	return;
           }
           if (strlen(mask) > 60) {
               notice(s_ChanServ, source, "ChanServ access mask too large!");
               wallops(s_ChanServ, "%s tried to add a HUGE mask to the %s SOP list", source, chan);
               return;

           } else if (mask && !strchr(mask, '@') || !strchr(mask, '!')) {
               notice(s_ChanServ, source, "ChanServ access mask must be in nick!user@host or nick format");
               notice(s_ChanServ, source,
                    "\2/msg %s HELP SOP\2 for more information.", s_ChanServ);
               return;
          } else if (ci->flags & CI_IDENT) {
              notice(s_ChanServ, source, "Channel %s has the security option set. You may not add unregistered nicks when this option is enabled", ci->name);
              return;
          }
        }

        if (ni && ni->forbid) {
           notice(s_ChanServ, source, "Forbidden nick %s not permitted to be added to channel access lists", ni->nick);
           return;
        }

        for (access = ci->access, i = 0; i < ci->accesscount; ++access, ++i) {
            if (access->in_use && stricmp(access->name, nick) == 0) {
                if (access->level >= ulev) {
                    notice(s_ChanServ, source, "Permission denied.");
                    return;
                }
                if (access->level == 10) {
                    notice(s_ChanServ, source,
                        "\2%s\2 is already a SOP on \2%s\2.",
                        access->name, chan);
                    return;
                }

                if (ni && (ni->flags & NI_NOOP)) {
                    notice(s_ChanServ, source, "Since %s is already on the %s access list, regardless of NOOP setting, adding to SOP list",
                       ni->nick, chan);
		    if (finduser(ni->nick))
		    if (ni->flags & NI_IDENTIFIED) 
			if (ni->flags & NI_AUTH)
		        notice(s_ChanServ, ni->nick, "\2%s\2 try to added you as SOP in channel \2%s\2.", source, chan);
		}
                else {
                	if (access->level == 3) {
                    	notice(s_ChanServ, source,
                        	"Move \2%s\2 from VOP to HOP on \2%s\2.",
                        	access->name, chan);
	                } else if (access->level == 5) {
                    	notice(s_ChanServ, source,
                        	"Move \2%s\2 from AOP to SOP on \2%s\2.",
                        	access->name, chan);
	                } else if (access->level == 4) {
                    	notice(s_ChanServ, source,
                        	"Move \2%s\2 from HOP to SOP on \2%s\2.",
                        	access->name, chan);
	                } else if (access->level == 13) {
                    	notice(s_ChanServ, source,
                        	"Move \2%s\2 from CFOUNDER to SOP on \2%s\2.",
                        	access->name, chan);
			} else 
                    notice(s_ChanServ, source,
                        "\2%s\2 was changed to a SOP on \2%s\2.",
                        access->name, chan);
	        access->level = 10;
	if (finduser(ni->nick))
	    if (ni->flags & NI_IDENTIFIED) {
		if (is_on_chan(ni->nick, chan))
			if (!is_chanhop(ni->nick, chan))
			change_cmode (s_ChanServ, chan, "+o", ni->nick);
		if (ni->flags & NI_AUTH)
	        notice(s_ChanServ, ni->nick, "\2%s\2 added you as SOP in channel \2%s\2.", source, chan);
	    }
		}
                show_next_db(source, s_ChanServ);
                return;
            }
        }
        if ((CHAN_ACCESS_MAX > 0) && (ci->accesscount > CHAN_ACCESS_MAX-1)) {
	    notice(s_ChanServ, source,
                    "Sorry, you may only have %d channel access enteries.",
                    CHAN_ACCESS_MAX);
            wallops(s_ChanServ, "%s tried to add another nick to the access list on %s",
                   source, chan);
	    return;
	}
        if (ni && (ni->flags & NI_NOOP)) {
             notice(s_ChanServ, source, "Sorry, %s has the NOOP option set.", ni->nick);
	    if (finduser(ni->nick))
	    if (ni->flags & NI_IDENTIFIED) 
		if (ni->flags & NI_AUTH)
	        notice(s_ChanServ, ni->nick, "\2%s\2 try to added you as SOP in channel \2%s\2.", source, chan);
             return;
        }
        for (access = ci->access, i = 0; i < ci->accesscount; ++access, ++i) {
            if (!access->in_use)
                break;
        }
        if (i == ci->accesscount) {
            ++ci->accesscount;

            ci->access =
                srealloc(ci->access, sizeof(ChanAccess) * ci->accesscount);
            access = &ci->access[ci->accesscount-1];
        }

        if (ni)
           access->name = sstrdup(ni->nick);
        else
           access->name = sstrdup(mask);
        access->in_use = 1;
        access->level = 10;

        notice(s_ChanServ, source,
                "\2%s\2 added to \2%s\2 as a SOP.",
                access->name, chan);
        show_next_db(source, s_ChanServ);

	if (finduser(ni->nick))
	    if (ni->flags & NI_IDENTIFIED) {
		if (is_on_chan(ni->nick, chan))
			if (!is_chanhop(ni->nick, chan))
			change_cmode (s_ChanServ, chan, "+o", ni->nick);
		if (ni->flags & NI_AUTH)
	        notice(s_ChanServ, ni->nick, "\2%s\2 added you as SOP in channel \2%s\2.", source, chan);
	    }

    } else if (stricmp(cmd, "DEL") == 0) {

        if (readonly) {
            notice(s_ChanServ, source,
                "Sorry, channel access list modification is temporarily disabled");
            return;
        }
        /* Special case: is it a number?  Only do search if it isn't. */
        if (strspn(nick, "1234567890") == strlen(nick) &&
                                (i = atoi(nick)) > 0 && i <= ci->accesscount) {
            int j=0;
            for (access = ci->access, z = 0; z < ci->accesscount;
                        ++access, ++z) {
                if (!(10 == access->level))
                    continue;
                j += 1;
                if (i == j) {
                    access = &ci->access[z];
                    break;
                }
            }
            if (!(i == j)) {
                notice(s_ChanServ, source,
                        "No such entry (#%d) on %s access list.", i, chan);
                return;
            }
        } else {
/*
            ni = findnick(nick);
            if (!ni) {
                notice(s_ChanServ, source,
                        "Nick \2%s\2 not registered.", nick);
                return;
            }
*/
            for (access = ci->access, z = 0; z < ci->accesscount;
                                                        ++access, ++z) {
                if ((stricmp(access->name, nick) == 0)
                        && (access->level == 10))
                    break;
            }
            if (z == ci->accesscount) {
                notice(s_ChanServ, source,
                        "\2%s\2 not found on %s SOP list.", nick, chan);
                return;
            }
        }
        if (get_access(u, ci) <= access->level) {
            notice(s_ChanServ, source, "Permission denied.");
        } else {
            notice(s_ChanServ, source,
                "\2%s\2 deleted from %s access list.", access->name, chan);
            show_next_db(source, s_ChanServ);
            free(access->name);
            access->in_use = 0;
            access->name = NULL;
              --ci->accesscount;
              if (z < ci->accesscount)
                  bcopy(access+1, access, sizeof(ChanAccess)
                                        * (ci->accesscount - z));
        }


    } else if (stricmp(cmd, "CLEAN") == 0) {
        int check = 0;
        check = ci->accesscount;
        for (access = ci->access, i = 0; i < check; ++access, ++i) {
              if ((10 == access->level) && (!strchr(access->name, '@'))
                 && (!findnick(access->name))) {
                     free(access->name);
                     access->name = NULL;
                     access->in_use = 0;
                     ++f;
                     --ci->accesscount;
                     if ((i-f) < ci->accesscount)
                         bcopy(access+1, access, sizeof(ChanAccess)
                                        * (ci->accesscount - (i-f)));
                     --access;
              }
        }
        if (!(f > 0))
            notice(s_ChanServ, source,
               "No Expired nicknames found in the SOP list on \2%s\2",
                chan);
        else {
           notice(s_ChanServ, source,
               "%d expired nicknames in the SOP list on \2%s\2 removed",
                f, chan);
           show_next_db(source, s_ChanServ);
       }
       f=0;



    } else if (stricmp(cmd, "WIPE") == 0) {
        int check = 0;
        check = ci->accesscount;
        for (access = ci->access, i = 0; i < check; ++access, ++i) {
              if (10 == access->level) {
                  free(access->name);
                  access->name = NULL;
                  access->in_use = 0;
                  ++f;
                  --ci->accesscount;
                  if ((i-f) < ci->accesscount)
                      bcopy(access+1, access, sizeof(ChanAccess)
                                              * (ci->accesscount - (i-f)));
                  --access;
              }
        }
        if (!(f > 0))
            notice(s_ChanServ, source,
               "No enteries found in the SOP list on \2%s\2",
                chan);


        else {
           notice(s_ChanServ, source,
               "%d entr%s in the SOP list on \2%s\2 removed",
                f, f==1 ? "y" : "ies", chan);
           show_next_db(source, s_ChanServ);
       }
       f=0;





    } else {

        notice(s_ChanServ, source,
                "Syntax: \2SOP\2 %s [ADD|DEL|LIST|WIPE|CLEAN] [<nick>]",chan);
        notice(s_ChanServ, source,
                "\2/msg %s HELP SOP\2 for more information.", s_ChanServ);
    }
}


static void do_aop(const char *source)
{
    char *chan = strtok(NULL, " ");
    char *cmd  = strtok(NULL, " ");
    char *nick = strtok(NULL, " ");
    char *s    = "1";
    ChannelInfo *ci;
    NickInfo *ni;
    User *u;
    short ulev;
    int is_list = (cmd && !stricmp(cmd, "LIST"));
    ChanAccess *access;
    int i, f=0, c=0, z;

    if (!cmd || ((!stricmp(cmd, "LIST") == 0) && (!stricmp(cmd, "WIPE") == 0)
             && (!stricmp(cmd, "CLEAN") == 0) && !nick)) {
         notice(s_ChanServ, source,
           "Syntax: \2AOP <#channel> [ADD|DEL|LIST|WIPE|CLEAN] [<nick>]");
         return;
    }

    if (!(ci = cs_findchan(chan))) {

        notice(s_ChanServ, source, "Channel %s is not registered.", chan);

    } else if (!(u = finduser(source))) {

        notice(s_ChanServ, source, "Access denied.");
        log("%s: ACCESS command from nonexistent user %s", s_ChanServ, source);


    } else if (stricmp(cmd, "LIST") == 0) {

        if (!(is_oper(source)) &&
             (((is_list && !check_access(u, ci, CA_ACCESS_LIST))
             || (!is_list && !check_access(u, ci, CA_ACCESS_CHANGE)))
             && !(is_list && is_services_admin(source)))) {

           notice(s_ChanServ, source, "Access Denied");

        } else {

	int j=0;
        notice(s_ChanServ, source, "AOP list for \2%s\2:", chan);
        for (access = ci->access, i = 0; i < ci->accesscount; ++access, ++i) {
            if ((nick && !match_wild(nick, access->name)) ||
                                        (5 != access->level))
                continue;
            if (ni = findnick(access->name))
                s = ni->last_usermask;
            else
                s = NULL;
	    ++j;
            notice(s_ChanServ, source, " %d)  %s%s%s%s",
                        j, access->name,
                        s ? " (" : "", s ? s : "", s ? ")" : "");
        }

        notice(s_ChanServ,source,"End of List");

      }


    } else if (((is_list && !check_access(u, ci, CA_ACCESS_LIST))
             || (!is_list && !check_access(u, ci, CA_ACCESS_CHANGE)))
            && !(is_list && is_services_admin(source)))
    {
        notice(s_ChanServ, source, "Access denied.");

    } else if (s && 5 >= (ulev = get_access(u, ci))) {

        notice(s_ChanServ, source, "Access denied.");

    } else if (stricmp(cmd, "ADD") == 0) {
        char *mask = nick;

        if (readonly) {
            notice(s_ChanServ, source,
                "Sorry, channel access list modification is temporarily disabled");
            return;
        }
        ni = findnick(nick);
        if (!ni) {
            if (NS_DACCESS == 1) {
           	 notice(s_ChanServ, source,
                "Adding access mask have been disabled by Admin");
           	 notice(s_ChanServ, source, "Nick %s is not registered.", nick);
            	return;
           }
           if (strlen(mask) > 60) {
               notice(s_ChanServ, source, "ChanServ access mask too large!");
               wallops(s_ChanServ, "%s tried to add a HUGE mask to the %s AOP list", source, chan);
               return;

           } else if (mask && !strchr(mask, '@') || !strchr(mask, '!')) {
               notice(s_ChanServ, source, "ChanServ access mask must be in nick!user@host or nick format");
               notice(s_ChanServ, source,
                    "\2/msg %s HELP AOP\2 for more information.", s_ChanServ);
               return;
          } else if (ci->flags & CI_IDENT) {
              notice(s_ChanServ, source, "Channel %s has the security option set. You may not add unregistered nicks when this option is enabled", ci->name);
              return;
          }
      }
      if (ni && ni->forbid) {
           notice(s_ChanServ, source, "Forbidden nick %s not permitted to be added to channel access lists", ni->nick);
           return;
        }

        for (access = ci->access, i = 0; i < ci->accesscount; ++access, ++i) {
            if (access->in_use && stricmp(access->name, nick) == 0) {
                if (access->level >= ulev) {
                    notice(s_ChanServ, source, "Permission denied.");
                    return;
                }
                if (access->level == 5) {
                    notice(s_ChanServ, source,
                        "\2%s\2 is already an AOP on \2%s\2.",
                        access->name, chan);
                    return;
                }
                if (ni && (ni->flags & NI_NOOP)) {
                    notice(s_ChanServ, source, "Since %s is already on the %s access list, regardless of NOOP setting, adding to AOP list",
                       ni->nick, chan);
		    if (finduser(ni->nick))
		    if (ni->flags & NI_IDENTIFIED) 
			if (ni->flags & NI_AUTH)
		        notice(s_ChanServ, ni->nick, "\2%s\2 try to added you as AOP in channel \2%s\2.", source, chan);
		}
                else {
                	if (access->level == 3) {
                    	notice(s_ChanServ, source,
                        	"Move \2%s\2 from VOP to AOP on \2%s\2.",
                        	access->name, chan);
	                } else if (access->level == 4) {
                    	notice(s_ChanServ, source,
                        	"Move \2%s\2 from HOP to AOP on \2%s\2.",
                        	access->name, chan);
	                } else if (access->level == 10) {
                    	notice(s_ChanServ, source,
                        	"Move \2%s\2 from SOP to AOP on \2%s\2.",
                        	access->name, chan);
	                } else if (access->level == 13) {
                    	notice(s_ChanServ, source,
                        	"Move \2%s\2 from CFOUNDER to AOP on \2%s\2.",
                        	access->name, chan);
			} else 
                    notice(s_ChanServ, source,
                        "\2%s\2 was changed to an AOP on \2%s\2.",
                        access->name, chan);
                access->level = 5;
	if (finduser(ni->nick))
	    if (ni->flags & NI_IDENTIFIED) {
		if (is_on_chan(ni->nick, chan))
			if (!is_chanop(ni->nick, chan))
			change_cmode (s_ChanServ, chan, "+o", ni->nick);
		if (ni->flags & NI_AUTH)
	        notice(s_ChanServ, ni->nick, "\2%s\2 added you as AOP in channel \2%s\2.", source, chan);
	    }
		}
                show_next_db(source, s_ChanServ);
                return;
            }
        }
        if ((CHAN_ACCESS_MAX > 0) && (ci->accesscount > CHAN_ACCESS_MAX-1)) {
	    notice(s_ChanServ, source,
                    "Sorry, you may only have %d channel access enteries.",
                    CHAN_ACCESS_MAX);
            wallops(s_ChanServ, "%s tried to add another nick to the access list on %s",
                   source, chan);
	    return;
	}
        if (ni && (ni->flags & NI_NOOP)) {
             notice(s_ChanServ, source,
                 "Sorry, %s has the NOOP option set", ni->nick);
	    if (finduser(ni->nick))
	    if (ni->flags & NI_IDENTIFIED) 
		if (ni->flags & NI_AUTH)
	        notice(s_ChanServ, ni->nick, "\2%s\2 try to added you as AOP in channel \2%s\2.", source, chan);
             return;
        }


        for (access = ci->access, i = 0; i < ci->accesscount; ++access, ++i) {
            if (!access->in_use)
                break;
        }
        if (i == ci->accesscount) {
            ++ci->accesscount;
            ci->access =
                srealloc(ci->access, sizeof(ChanAccess) * ci->accesscount);
            access = &ci->access[ci->accesscount-1];
        }
        if (ni)
           access->name = sstrdup(ni->nick);
        else
           access->name = sstrdup(mask);
        access->in_use = 1;
        access->level = 5;
        notice(s_ChanServ, source,
                "\2%s\2 added to \2%s\2 as an AOP.",
                access->name, chan);
        show_next_db(source, s_ChanServ);
	if (finduser(ni->nick))
	    if (ni->flags & NI_IDENTIFIED) {
		if (is_on_chan(ni->nick, chan))
			if (!is_chanop(ni->nick, chan))
			change_cmode (s_ChanServ, chan, "+o", ni->nick);
		if (ni->flags & NI_AUTH)
	        notice(s_ChanServ, ni->nick, "\2%s\2 added you as AOP in channel \2%s\2.", source, chan);
	    }

    } else if (stricmp(cmd, "DEL") == 0) {

        if (readonly) {
            notice(s_ChanServ, source,
                "Sorry, channel access list modification is temporarily disabled");
            return;
        }
        /* Special case: is it a number?  Only do search if it isn't. */
        if (strspn(nick, "1234567890") == strlen(nick) &&
                                (i = atoi(nick)) > 0 && i <= ci->accesscount) {
            int j=0;
            for (access = ci->access, z = 0; z < ci->accesscount;
                        ++access, ++z) {
                if (!(5 == access->level))
                    continue;
                j += 1;
                if (i == j) {
                    access = &ci->access[z];
                    break;
                }
            }
            if (!(i == j)) {
                notice(s_ChanServ, source,
                        "No such entry (#%d) on %s access list.", i, chan);
                return;
            }
        } else {
/*
            ni = findnick(nick);
            if (!ni) {
                notice(s_ChanServ, source,
                        "Nick \2%s\2 not registered.", nick);
                return;
            }
*/
            for (access = ci->access, z = 0; z < ci->accesscount;
                                                        ++access, ++z) {
                if ((stricmp(access->name, nick) == 0)
			&& (access->level == 5))
                    break;
            }
            if (z == ci->accesscount) {
                notice(s_ChanServ, source,
                        "\2%s\2 not found on %s AOP list.", nick, chan);
                return;
            }
        }
        if (get_access(u, ci) <= access->level) {
            notice(s_ChanServ, source, "Permission denied.");
        } else {
            notice(s_ChanServ, source,
                "\2%s\2 deleted from %s access list.", access->name, chan);
            show_next_db(source, s_ChanServ);
            free(access->name);
            access->in_use = 0;
            access->name = NULL;
              --ci->accesscount;
              if (z < ci->accesscount)
                  bcopy(access+1, access, sizeof(ChanAccess)
                                        * (ci->accesscount - z));
        }


    } else if (stricmp(cmd, "CLEAN") == 0) {
        int check = 0;
        check = ci->accesscount;
        for (access = ci->access, i = 0; i < check; ++access, ++i) {
              if ((5 == access->level) && (!strchr(access->name, '@'))
                 && (!findnick(access->name))) {
                     free(access->name);
                     access->name = NULL;
                     access->in_use = 0;
                     ++f;
                     --ci->accesscount;
                     if ((i-f) < ci->accesscount)
                         bcopy(access+1, access, sizeof(ChanAccess)
                                        * (ci->accesscount - (i-f)));
                     --access;
              }
        }
        if (!(f > 0))
            notice(s_ChanServ, source,
               "No Expired nicknames found in the AOP list on \2%s\2",
                chan);
        else {
           notice(s_ChanServ, source,
               "%d expired nicknames in the AOP list on \2%s\2 removed",
                f, chan);
           show_next_db(source, s_ChanServ);
       }
       f=0;


    } else if (stricmp(cmd, "WIPE") == 0) {
        int check = 0;
        check = ci->accesscount;
        for (access = ci->access, i = 0; i < check; ++access, ++i) {
              if (5 == access->level) {
                  free(access->name);
                  access->name = NULL;
                  access->in_use = 0;
                  ++f;
                  --ci->accesscount;
                  if ((i-f) < ci->accesscount)
                      bcopy(access+1, access, sizeof(ChanAccess)
                                              * (ci->accesscount - (i-f)));
                  --access;
              }
        }
        if (!(f > 0))
            notice(s_ChanServ, source,
               "No enteries found in the AOP list on \2%s\2",
                chan);

        else {
           notice(s_ChanServ, source,
               "%d entr%s in the AOP list on \2%s\2 removed",
                f, f==1 ? "y" : "ies", chan);
           show_next_db(source, s_ChanServ);
       }
       f=0;

    } else {

        notice(s_ChanServ, source,
                "Syntax: \2AOP\2 %s [ADD|DEL|LIST|WIPE|CLEAN] [<nick>]",chan);
        notice(s_ChanServ, source,
                "\2/msg %s HELP AOP\2 for more information.", s_ChanServ);
    }
}

/*************************************************************************/

static void do_akick(const char *source)
{
    char *chan   = strtok(NULL, " ");
    char *cmd    = strtok(NULL, " ");
    char *mask   = strtok(NULL, " ");
    char *reason = strtok(NULL, "");
    char *t;
    ChannelInfo *ci;
    NickInfo *ni;
    User *u;
    int i, k=0;
    AutoKick *akick;
    Channel *c;
    struct c_userlist *u2;

    if (!cmd || (stricmp(cmd, "LIST") != 0 &&
       stricmp(cmd, "WIPE") != 0 && !mask)) {

	notice(s_ChanServ, source,
                "Syntax: \2AKICK\2 %s [ADD|DEL|WIPE|LIST] [<nick> or <usermask>]",
                chan ? chan : "<#channel>");
	notice(s_ChanServ, source,
		"\2/msg %s HELP AKICK\2 for more information.", s_ChanServ);

    } else if (!(ci = cs_findchan(chan))) {

	notice(s_ChanServ, source, "Channel %s is not registered.", chan);

    } else if (stricmp(cmd, "LIST") == 0) {

       if (!(is_oper(source)) &&
        ((!(u = finduser(source)) || !check_access(u, ci, CA_AKICK))
             && !(stricmp(cmd, "LIST") == 0 && is_services_admin(source)))) {

           notice(s_ChanServ, source, "Access Denied");

        } else {
 
	notice(s_ChanServ, source, "AKICK list for %s:", chan);
	for (akick = ci->akick, i = 0; i < ci->akickcount; ++akick, ++i) {
	    if (mask && !match_wild(mask, akick->name))
		continue;
	    if (ni = findnick(akick->name))
		t = ni->last_usermask;
	    else
                t = NULL;
	    notice(s_ChanServ, source, " %d)  %s%s%s%s%s%s%s",
			i+1, akick->name,
			t ? " (" : "", t ? t : "", t ? ")" : "",
			akick->reason ? " (" : "",
			akick->reason ? akick->reason : "",
			akick->reason ? ")" : "");
	}
        notice(s_ChanServ,source,"End of List");
      }

    } else if ((!(u = finduser(source)) || !check_access(u, ci, CA_AKICK))
	       && !(stricmp(cmd, "LIST") == 0 && is_services_admin(source))) {

	notice(s_ChanServ, source, "Access denied.");

    } else if (stricmp(cmd, "WIPE") == 0) {

      User *u = finduser(source);

      if (get_access(u, ci) < 13)  {
          notice(s_ChanServ, source, "Access Denied");
          return;
      }

      for (akick = ci->akick, i = 0; ci->akickcount; ++akick, ++i) {
           free(akick->name);
           if (akick->reason)
               free(akick->reason);

             ++k;
             --ci->akickcount;
             if (!ci->akickcount) {
                 free(ci->akick);
                 ci->akick = NULL;
           }
      }

      notice(s_ChanServ, source, "%d enteries deleted from the %s AKICK list",
         k, chan);

    } else if (stricmp(cmd, "ADD") == 0) {

	NickInfo *ni = findnick(mask);
	char *nick, *user, *host;

	if (readonly) {
	    notice(s_ChanServ, source,
		"Sorry, channel AutoKick list modification is temporarily disabled.");
	    return;
	}

        if ((AKICK_MAX != 0) && (ci->akickcount > AKICK_MAX-1)) {
	    notice(s_ChanServ, source,
                    "Sorry, you may only have %d AutoKick masks for a channel.",
		    AKICK_MAX);
            wallops(s_ChanServ,
                 "%s tried to add another akick to %s", source, chan);
	    return;
	}

	if (!ni) {
	    split_usermask(mask, &nick, &user, &host);
	    mask = smalloc(strlen(nick) + strlen(user) + strlen(host) + 3);
	    sprintf(mask, "%s!%s@%s", nick, user, host);
	    free(nick);
	    free(user);
	    free(host);
	}

	for (akick = ci->akick, i = 0; i < ci->akickcount; ++akick, ++i) {
	    if ((akick->is_nick ? stricmp(akick->name,mask) :
						strcmp(akick->name,mask)) == 0) {
		notice(s_ChanServ, source,
			"\2%s\2 already exists on %s AKICK list.",
			akick->name, chan);
		return;
	    }
	}

	++ci->akickcount;
	ci->akick = srealloc(ci->akick, sizeof(AutoKick) * ci->akickcount);
	akick = &ci->akick[ci->akickcount-1];
	akick->name = ni ? sstrdup(mask) : mask;
	akick->is_nick = (ni != NULL);
	akick->pad = 0;
	if (reason)
	    akick->reason = sstrdup(reason);
	else
	    akick->reason = NULL;
	notice(s_ChanServ, source,
		"\2%s\2 added to %s AKICK list.", akick->name, chan);
        show_next_db(source, s_ChanServ);


    } else if (stricmp(cmd, "DEL") == 0) {

	if (readonly) {
	    notice(s_ChanServ, source,
		"Sorry, channel AutoKick list modification is temporarily disabled.");
	    return;
	}

	/* Special case: is it a number?  Only do search if it isn't. */
	if (strspn(mask, "1234567890") == strlen(mask) &&
				(i = atoi(mask)) > 0 && i <= ci->akickcount) {
	    --i;
	    akick = &ci->akick[i];
	} else {
	    /* First try for an exact match; then, a case-insensitive one. */
	    for (akick = ci->akick, i = 0; i < ci->akickcount; ++akick, ++i) {
		if (strcmp(akick->name, mask) == 0)
		    break;
	    }
	    if (i == ci->akickcount) {
		for (akick = ci->akick, i = 0; i < ci->akickcount; ++akick, ++i) {
		    if (stricmp(akick->name, mask) == 0)
			break;
		}
	    }
	    if (i == ci->akickcount) {
		notice(s_ChanServ, source,
			"\2%s\2 not found on %s AKICK list.", mask, chan);
		return;
	    }
	}
	notice(s_ChanServ, source,
		"\2%s\2 deleted from %s AKICK list.", akick->name, chan);
        show_next_db(source, s_ChanServ);
        if (readonly)
             notice(s_ChanServ, source,
                   "Database changes temporarily disabled. Changes will not be saved!");

	free(akick->name);
	if (akick->reason)
	    free(akick->reason);
	--ci->akickcount;
	if (i < ci->akickcount)
	    bcopy(akick+1, akick, sizeof(AutoKick) * (ci->akickcount-i));
	if (ci->akickcount)
	    ci->akick = srealloc(ci->akick, sizeof(AutoKick) * ci->akickcount);
	else {
	    free(ci->akick);
	    ci->akick = NULL;
	}

    } else {

	notice(s_ChanServ, source,
                "Syntax: \2AKICK\2 %s [ADD|DEL|LIST|WIPE] [<nick> or <usermask>]",
		chan);
	notice(s_ChanServ, source,
		"\2/msg %s HELP AKICK\2 for more information.", s_ChanServ);

    }
}

/*************************************************************************/

static void do_info(const char *source)
{
    char *chan = strtok(NULL, " ");
    ChannelInfo *ci;
    Channel *chan_s;
    NickInfo *ni;
    struct tm tm;
    char s[BUFSIZE], *t, *end, timebuf[64];
    time_t tbuff;   

    if (!chan) {

        notice(s_ChanServ, source, "Syntax: \2INFO\2 <#channel>");
	notice(s_ChanServ, source,
		"\2/msg %s HELP INFO\2 for more information.", s_ChanServ);

    } else if (!(ci = cs_findchan(chan))) {

	notice(s_ChanServ, source, "Channel \2%s\2 is not registered.", chan);

    } else if (ci->flags & CI_VERBOTEN) {

	notice(s_ChanServ, source,
		"Channel \2%s\2 may not be registered or used.", chan);

    } else {

	notice(s_ChanServ, source, "Information for channel \2%s\2:", chan);

	if (((ni = findnick(ci->founder)) && (!(ni->flags & NI_NOHOST)) || (is_services_admin(source))))
	    t = ni->last_usermask;
	else
	    t = NULL;
	notice(s_ChanServ, source,
		"        Founder: %s%s%s%s",
			ci->founder, t ? " (" : "", t ? t : "", t ? ")" : "");
	notice(s_ChanServ, source,
		"    Description: %s", ci->desc);
        if (ci->welcome)
            notice(s_ChanServ, source,
                    "    Welcome MSG: %s", ci->welcome);

	tm = *localtime(&ci->time_registered);
        strftime(timebuf, sizeof(timebuf), "%b %d %H:%M:%S %Y %Z", &tm);
        timebuf[sizeof(timebuf)-1] = 0;

	notice(s_ChanServ, source,
		"     Registered: %s", timebuf);

	tm = *localtime(&ci->last_used);
        strftime(timebuf, sizeof(timebuf), "%b %d %H:%M:%S %Y %Z", &tm);
        timebuf[sizeof(timebuf)-1] = 0;

	notice(s_ChanServ, source,
		"      Last used: %s", timebuf);

        tbuff = time(NULL);
	tm = *localtime(&tbuff);
        strftime(timebuf, sizeof(timebuf), "%b %d %H:%M:%S %Y %Z", &tm);
        timebuf[sizeof(timebuf)-1] = 0;

	notice(s_ChanServ, source,
		"       Time Now: %s", timebuf);

	if (ci->last_topic) {
	    notice(s_ChanServ, source,
		"     Last topic: %s", ci->last_topic);
	    notice(s_ChanServ, source,
		"   Topic set by: %s\n", ci->last_topic_setter);
	}
        if (ci->flags & CI_MEMO_AV) {
            notice(s_ChanServ, source,
                "     Memo Level: VOP");
        } else if (ci->flags & CI_MEMO_NONE) {
            notice(s_ChanServ, source,
                "     Memo Level: Disabled by Founder");
        } else if (ci->flags & CI_MEMO_HOP) {
            notice(s_ChanServ, source,
                "     Memo Level: HOP");
        } else if (ci->flags & CI_MEMO_AOP) {
            notice(s_ChanServ, source,
                "     Memo Level: AOP");
        } else if (ci->flags & CI_MEMO_SOP) {
            notice(s_ChanServ, source,
                "     Memo Level: SOP");
        } else if (ci->flags & CI_MEMO_CF) {
            notice(s_ChanServ, source,
                "     Memo Level: CFOUNDER");
        } else if (ci->flags & CI_MEMO_FR) {
            notice(s_ChanServ, source,
                "     Memo Level: FOUNDER");
        } else
            notice(s_ChanServ, source,
                "     Memo Level: VOP");

/*  most likely is a case of an older channel that was created
    before the implementation of channel memos. Once a channel memo
    is sent, or memo level set, it will automatically be set anyways.
*/

	if (ci->url)
	    notice(s_ChanServ, source,
		"            URL: %s\n", ci->url);
	if (ci->email && !(ci->flags & CI_PRIVATE))
	    notice(s_ChanServ, source,
		" E-mail address: %s\n", ci->email);
	if (!ci->flags)
	    strcpy(s, "None\n");
	else {
	    int need_comma = 0;
	    static const char commastr[] = ", ";
	    end = s;
	    if (ci->flags & CI_PRIVATE) {
		end += snprintf(end, sizeof(s)-(end-s), "Private");
		need_comma = 1;
	    }
	    if (ci->flags & CI_KEEPTOPIC) {
		end += snprintf(end, sizeof(s)-(end-s), "%sTopic Retention", need_comma ? commastr : "");
		need_comma = 1;
	    }
	    if (ci->flags & CI_TOPICLOCK) {
                if (ci->topic_allow == 3)
                    end += snprintf(end, sizeof(s)-(end-s), "%sTopic Lock(+v)", need_comma ? commastr : "");
                else if (ci->topic_allow == 5)
                    end += snprintf(end, sizeof(s)-(end-s), "%sTopic Lock(A)", need_comma ? commastr : "");
                else if (ci->topic_allow == 10)
                    end += snprintf(end, sizeof(s)-(end-s), "%sTopic Lock(S)", need_comma ? commastr : "");
                else if (ci->topic_allow == 13)
                    end += snprintf(end, sizeof(s)-(end-s), "%sTopic Lock(CF)", need_comma ? commastr : "");
                else if (ci->topic_allow == 15)
                    end += snprintf(end, sizeof(s)-(end-s), "%sTopic Lock(F)", need_comma ? commastr : "");
		need_comma = 1;
	    }
	    if (ci->flags & CI_OPGUARD) {
		end += snprintf(end, sizeof(s)-(end-s), "%sOp Guard", need_comma ? commastr : "");
		need_comma = 1;
	    }
	    if (ci->flags & CI_RESTRICTED) {
		end += snprintf(end, sizeof(s)-(end-s), "%sRestricted Access", need_comma ? commastr : "");
		need_comma = 1;
	    }
	    if (ci->flags & CI_IDENT) {
		end += snprintf(end, sizeof(s)-(end-s), "%sIdent", need_comma ? commastr : "");
		need_comma = 1;
	    }
	    if (ci->flags & CI_AUTH) {
		end += snprintf(end, sizeof(s)-(end-s), "%sAuth", need_comma ? commastr : "");
		need_comma = 1;
	    }
	}
	notice(s_ChanServ, source, "        Options: %s", s);
	end = s;
	*end = 0;
	if (ci->mlock_on || ci->mlock_key || ci->mlock_limit)
            end += sprintf(end, "+%s%s%s%s%s%s%s%s%s%s%s%s%s",
				(ci->mlock_on & CMODE_I) ? "i" : "",
				(ci->mlock_key         ) ? "k" : "",
				(ci->mlock_limit       ) ? "l" : "",
				(ci->mlock_on & CMODE_M) ? "m" : "",
				(ci->mlock_on & CMODE_N) ? "n" : "",
				(ci->mlock_on & CMODE_P) ? "p" : "",
				(ci->mlock_on & CMODE_S) ? "s" : "",
				(ci->mlock_on & CMODE_T) ? "t" : "",
				(ci->mlock_on & CMODE_C) ? "c" : "",
				(ci->mlock_on & CMODE_H) ? "H" : "",
			/*	(ci->mlock_on & CMODE_G) ? "G" : "",
				(ci->mlock_on & CMODE_ST) ? "S" : "",
				(ci->mlock_on & CMODE_NK) ? "K" : "", */
				(ci->mlock_on & CMODE_NI) ? "V" : "",
				(ci->mlock_on & CMODE_AD) ? "A" : "",
				(ci->mlock_on & CMODE_OP) ? "O" : "",
				(ci->mlock_on & CMODE_Q) ? "Q" : "",
				(ci->mlock_on & CMODE_RN) ? "R" : "",
				(ci->mlock_on & CMODE_r) ? "r" : "");
	if (ci->mlock_off)
	    end += sprintf(end, "-%s%s%s%s%s%s%s%s%s%s%s%s%s",
				(ci->mlock_off & CMODE_I) ? "i" : "",
				(ci->mlock_off & CMODE_K) ? "k" : "",
				(ci->mlock_off & CMODE_L) ? "l" : "",
				(ci->mlock_off & CMODE_M) ? "m" : "",
				(ci->mlock_off & CMODE_N) ? "n" : "",
				(ci->mlock_off & CMODE_P) ? "p" : "",
				(ci->mlock_off & CMODE_S) ? "s" : "",
				(ci->mlock_off & CMODE_T) ? "t" : "",
				(ci->mlock_off & CMODE_C) ? "c" : "",
				(ci->mlock_off & CMODE_H) ? "H" : "",
			/*	(ci->mlock_off & CMODE_G) ? "G" : "",
				(ci->mlock_off & CMODE_ST) ? "S" : "",
				(ci->mlock_off & CMODE_NK) ? "K" : "", */
				(ci->mlock_off & CMODE_NI) ? "V" : "",
				(ci->mlock_off & CMODE_AD) ? "A" : "",
				(ci->mlock_off & CMODE_OP) ? "O" : "",
				(ci->mlock_off & CMODE_Q) ? "Q" : "",
				(ci->mlock_off & CMODE_RN) ? "R" : "",
				(ci->mlock_off & CMODE_r) ? "r" : "");
	if (s)
	    notice(s_ChanServ, source, "      Mode lock: %s", s);
        chan_s = findchan(chan);
        if (is_services_admin(source)) {
            if (!chan_s && ci->mlock_key)
                notice(s_ChanServ, source, "\2SRA\2: Channel Key = %s",
                           ci->mlock_key);
            else {
                if (chan_s && chan_s->key)
                   notice(s_ChanServ, source, "\2SRA\2: Channel Key = %s",
                           chan_s->key);
            }
        }
        if (is_oper(source)) {

            if (ci->flags & CI_MARKCHAN)
                notice(s_ChanServ, source,
                   " * Channel has been marked by: %s", ci->mark);

            if (ci->flags & CI_HELDCHAN)
                notice(s_ChanServ, source,
                   " * Channel has been held by: %s", ci->hold);

            if (ci->flags & CI_FREEZECHAN)
                notice(s_ChanServ, source,
                   " * Channel has been frozen by: %s", ci->freeze);

        } else {

            if (ci->flags & CI_MARKCHAN)
                notice(s_ChanServ, source,
                   " * This Channel has been marked.");

            if (ci->flags & CI_HELDCHAN)
                notice(s_ChanServ, source,
                   " * This Channel has been held.");

            if (ci->flags & CI_FREEZECHAN)
                notice(s_ChanServ, source,
                   " * This Channel has been frozen.");
        }


    }
}

/*************************************************************************/

static void do_list(const char *source)
{
    char *pattern = strtok(NULL, " ");
    ChannelInfo *ci;
    int nchans, i;
    char buf[BUFSIZE];

    if (!pattern) {

        notice(s_ChanServ, source, "Syntax: \2LIST\2 <pattern>");
	notice(s_ChanServ, source,
		"\2/msg %s HELP LIST\2 for more information.", s_ChanServ);

    } else {

	nchans = 0;
	notice(s_ChanServ, source, "List of entries matching \2%s\2:", pattern);
	for (i = 33; i < 256; ++i) {
	    for (ci = chanlists[i]; ci; ci = ci->next) {
		if (ci->flags & (CI_PRIVATE | CI_VERBOTEN))
		    continue;
		if (strlen(ci->name)+strlen(ci->desc) > sizeof(buf))
		    continue;
		snprintf(buf, sizeof(buf), "%-20s  %s", ci->name, ci->desc);
		if (stricmp(pattern, ci->name) == 0 ||
					match_wild_nocase(pattern, buf)) {
		    if (++nchans <= 50)
			notice(s_ChanServ, source, "    %s", buf);
		}
	    }
	}
	notice(s_ChanServ, source, "End of list - %d/%d matches shown.",
					nchans>50 ? 50 : nchans, nchans);
    }

}

/*************************************************************************/

static void do_invite(const char *source)
{
    char *chan = strtok(NULL, " ");
    char *nick;
    User *u = finduser(source);
    ChannelInfo *ci;

    if (!chan) {

	notice(s_ChanServ, source,
                "Syntax: \2INVITE\2 <#channel>");
	notice(s_ChanServ, source,
		"\2/msg %s HELP INVITE\2 for more information.", s_ChanServ);

    } else if (!findchan(chan)) {

	notice(s_ChanServ, source, "Channel %s does not exist.", chan);

    } else if (!(ci = cs_findchan(chan))) {

	notice(s_ChanServ, source, "Channel %s is not registered.", chan);

    } else if (!u || !check_access(u, ci, CA_INVITE)) {

	notice(s_ChanServ, source, "Access denied.");

    } else

           send_cmd(s_ChanServ, "INVITE %s %s", source, chan);
}

/*************************************************************************/

static void do_unban(const char *source)
{
    char *chan = strtok(NULL, " ");
    User *u = finduser(source);
    ChannelInfo *ci;
    Channel *c;
    int i;
    char *av[3];

    if (!chan) {

        notice(s_ChanServ, source, "Syntax: \2UNBAN\2 <#channel>");
	notice(s_ChanServ, source,
		"\2/msg %s HELP UNBAN\2 for more information.", s_ChanServ);

    } else if (!(c = findchan(chan))) {

	notice(s_ChanServ, source, "Channel %s does not exist.", chan);

    } else if (!(ci = cs_findchan(chan))) {

	notice(s_ChanServ, source, "Channel %s is not registered.", chan);

    } else if (!u || !check_access(u, ci, CA_UNBAN)) {

	notice(s_ChanServ, source, "Access denied.");

    } else if (ci->flags & CI_FREEZECHAN) {
        notice(s_ChanServ, source, "Channel %s is FROZEN.", chan);

    } else {

	av[0] = chan;
	av[1] = sstrdup("-b");
	for (i = c->bancount-1; i > -1; --i) {
	    if (match_usermask(c->bans[i], u)) {
		send_cmd(MODE_SENDER, "MODE %s -b %s", chan, c->bans[i]);
		av[2] = sstrdup(c->bans[i]);
		do_cmode(s_ChanServ, 3, av);
		free(av[2]);
	    }
	}
	free(av[1]);
	notice(s_ChanServ, source, "You have been unbanned from %s.", chan);

    }
}

/*************************************************************************/

static void do_clear(const char *source)
{
    char *chan = strtok(NULL, " ");
    char *what = strtok(NULL, " ");
    char *reason = strtok(NULL, " ");
    User *u = finduser(source);
    Channel *c;
    ChannelInfo *ci;

    if (!what) {

	notice(s_ChanServ, source,
                "Syntax: \2CLEAR\2 <#channel> <what> [<reason>]");
	notice(s_ChanServ, source,
		"\2/msg %s HELP CLEAR\2 for more information.", s_ChanServ);


    } else if (!(c = findchan(chan))) {

	notice(s_ChanServ, source, "Channel %s does not exist.", chan);

    } else if (!(ci = cs_findchan(chan))) {

	notice(s_ChanServ, source, "Channel %s is not registered.", chan);

    } else if (ci->flags & CI_FREEZECHAN) {
        notice(s_ChanServ, source, "Channel %s is Frozen.", chan);

    } else if (!u || !check_access(u, ci, CA_CLEAR)) {

	notice(s_ChanServ, source, "Access denied.");

    } else if (stricmp(what, "bans") == 0) {

	char *av[3];

	while (c->bancount > 0) {
	    av[0] = sstrdup(chan);
	    av[1] = sstrdup("-b");
	    av[2] = sstrdup(c->bans[0]);
	    send_cmd(MODE_SENDER, "MODE %s %s :%s", av[0], av[1], av[2]);
	    do_cmode(s_ChanServ, 3, av);
	    free(av[2]);
	    free(av[1]);
	    free(av[0]);
	}
	notice(s_ChanServ, source, "All bans on %s have been removed.", chan);
        slog("CS C %s (%s!%s@%s) [Bans]",
              chan, source, u->username, u->host);

    } else if (stricmp(what, "modes") == 0) {

	char *av[3];

	av[0] = chan;
	av[1] = sstrdup("-mintpslk");
	if (c->key)
	    av[2] = sstrdup(c->key);
	else
	    av[2] = sstrdup("");
	send_cmd(MODE_SENDER, "MODE %s %s :%s", av[0], av[1], av[2]);
	do_cmode(s_ChanServ, 3, av);
	free(av[2]);
	free(av[1]);
	notice(s_ChanServ, source, "All modes on %s have been reset.", chan);
        slog("CS C %s (%s!%s@%s) [Modes]",
              chan, source, u->username, u->host);

    } else if (stricmp(what, "ops") == 0) {

	char *av[3];
	struct c_userlist *cu, *next;

	for (cu = c->chanops; cu; cu = next) {
            next = cu->next;
            if (stricmp(cu->user->nick, source) != 0) {
     	       av[0] = sstrdup(chan);
	       av[1] = sstrdup("-o");
	       av[2] = sstrdup(cu->user->nick);
	       send_cmd(MODE_SENDER, "MODE %s %s :%s", av[0], av[1], av[2]);
	       do_cmode(s_ChanServ, 3, av);
	       free(av[2]);
	       free(av[1]);
	       free(av[0]);
            }
	}
	notice(s_ChanServ, source, "Mode +o cleared from %s.", chan);
        slog("CS C %s (%s!%s@%s) [Ops]",
              chan, source, u->username, u->host);

    } else if (stricmp(what, "voices") == 0) {

	char *av[3];
	struct c_userlist *cu, *next;

	for (cu = c->voices; cu; cu = next) {
	    next = cu->next;
	    av[0] = sstrdup(chan);
	    av[1] = sstrdup("-v");
	    av[2] = sstrdup(cu->user->nick);
	    send_cmd(MODE_SENDER, "MODE %s %s :%s", av[0], av[1], av[2]);
	    do_cmode(s_ChanServ, 3, av);
	    free(av[2]);
	    free(av[1]);
	    free(av[0]);
	}
	notice(s_ChanServ, source, "Mode +v cleared from %s.", chan);
        slog("CS C %s (%s!%s@%s) [Voices]",
              chan, source, u->username, u->host);

    } else if (stricmp(what, "users") == 0) {

	char *av[3];
	struct c_userlist *cu, *next_cu;
	char buf[256];
        Timeout *t;


	snprintf(buf, sizeof(buf), "CLEAR USERS command from %s", source);
        send_cmd(s_ChanServ, "JOIN %s", chan);
        send_cmd(s_ChanServ, "MODE %s +b *!*@*", chan);

	for (cu = c->users; cu; cu = next_cu) {
	    next_cu = cu->next;
            if (stricmp(cu->user->nick, source) != 0) {
  	       av[0] = sstrdup(chan);
	       av[1] = sstrdup(cu->user->nick);
               if (!reason)
	          av[2] = sstrdup(buf);
               else
                  av[2] = sstrdup(reason);
	       send_cmd(s_ChanServ, "KICK %s %s :%s", av[0], av[1], av[2]);
	       do_kick(s_ChanServ, 3, av);
	       free(av[2]);
	       free(av[1]);
	       free(av[0]);
            }
	}
	notice(s_ChanServ, source, "All users kicked from %s.", chan);
        slog("CS C %s (%s!%s@%s) [Users]",
              chan, source, u->username, u->host);

        t = add_timeout(CHANNEL_INHABIT, timeout_unban, 0);
        t->data = sstrdup(chan);

        t = add_timeout(CHANNEL_INHABIT+1, timeout_leave, 0);
        t->data = sstrdup(chan);


    } else {

	notice(s_ChanServ, source,
                "Syntax: \2CLEAR <#channel> <what> [<reason>]");
	notice(s_ChanServ, source,
		"\2/msg %s HELP CLEAR\2 for more information.", s_ChanServ);

    }
}

/*************************************************************************/

static void do_count(const char *source)
{
    char *chan = strtok(NULL, " ");
    char *s    = "1";
    ChannelInfo *ci;
    NickInfo *ni;
    AutoKick *akick;
    User *u;
    short ulev;
    int is_list = (chan && !stricmp(chan, "LIST"));
    ChanAccess *access;
    int i;

    if (!chan) {
	notice(s_ChanServ, source, "Usage: \2/msg %s COUNT \37channel\37\2", s_ChanServ);
	return;
    }
    if (!(ci = cs_findchan(chan))) {

        notice(s_ChanServ, source, "Channel %s is not registered.", chan);

    } else if (!(u = finduser(source))) {

        notice(s_ChanServ, source, "Access denied.");
        log("%s: ACCESS command from nonexistent user %s", s_ChanServ, source);


    } else if (!(is_oper(source)) &&
             (((is_list && !check_access(u, ci, CA_ACCESS_LIST))
             || (!is_list && !check_access(u, ci, CA_ACCESS_CHANGE)))
             && !(is_list && is_services_admin(source)))) {

        notice(s_ChanServ, source, "Access denied.");

    } else if (s && 3 > (ulev = get_access(u, ci)) && !(is_oper(source))) {

        notice(s_ChanServ, source, "Access denied.");

    } else {
        int v=0;
	int h=0;
        int a=0;
        int so=0;
        int f=0;
        for (access = ci->access, i = 0; i < ci->accesscount; ++access, ++i) {
            if (3 == access->level)
               ++v;
            if (4 == access->level)
               ++h;
            if (5 == access->level)
               ++a;
            if (10 == access->level)
               ++so;
            if (13 == access->level)
               ++f;

        }
        notice(s_ChanServ, source, "\2%s Count:\2 VOP: %d, HOP: %d, AOP: %d, SOP: %d, CF: %d AKICK: %d", chan, v, h, a, so, f, ci->akickcount);
        
    }
}


/*************************************************************************/

static void do_getpass(const char *source)
{
    char *chan = strtok(NULL, " ");
    User *u = finduser(source);
    ChannelInfo *ci;

#ifdef USE_ENCRYPTION
    notice(s_ChanServ, source,
		"GETPASS command unavailable when encryption is in use.");
#else
    if (!chan) {
        notice(s_ChanServ, source, "Syntax: \2GETPASS\2 <#channel>");
    } else if (!(ci = cs_findchan(chan))) {
	notice(s_ChanServ, source, "Channel %s is not registered.", chan);
        slog("CS *G %s (%s!%s@%s) [UNREGISTERED]",
          chan, u->nick, u->username, u->host);
        do_break_log("CS_G", "CS *G %s (%s!%s@%s) [UNREGISTERED]",
          chan, u->nick, u->username, u->host);
	return;
    } else if (!u) {
	notice(s_ChanServ, source, "Could not find your user record!");
    } else if (ci->flags & CI_MARKCHAN) {
	notice(s_ChanServ,source,"Channel \2%s\2 is marked!",chan);
        slog("CS *G %s (%s!%s@%s) [MARKED]",
          chan, u->nick, u->username, u->host);
        do_break_log("CS_G", "CS *G %s (%s!%s@%s) [MARKED]",
          chan, u->nick, u->username, u->host);
	wallops(s_ChanServ,
                "\2Warning:\2 %s attempted to use GETPASS on marked channel %s"
		,u->nick,chan);
	log("%s: %s!%s@%s attempted GETPASS on %s (Fail)"
		,s_ChanServ, source, u->username, u->host, chan);
    } else {
        slog("CS G %s (%s!%s@%s)",
          chan, u->nick, u->username, u->host);
        do_break_log("CS_G", "CS G %s (%s!%s@%s)",
          chan, u->nick, u->username, u->host);
	log("%s: %s!%s@%s used GETPASS on %s",
		s_ChanServ, source, u->username, u->host, chan);
        wallops(s_ChanServ, "%s used GETPASS on channel %s",
		source, chan);
	notice(s_ChanServ, source, "Password for %s is: \2%s\2",
		chan, ci->founderpass);
    }
#endif
}

/*************************************************************************/

static void do_sendpass(const char *source)
{
    char *chan = strtok(NULL, " ");
    User *u = finduser(source);
    ChannelInfo *ci;

#ifdef USE_ENCRYPTION
    notice(s_ChanServ, source,
		"SENDPASS command unavailable when encryption is in use.");
#else
    if (!chan) {
        notice(s_ChanServ, source, "Syntax: \2SENDPASS\2 <#channel>");
    } else if (!(ci = cs_findchan(chan))) {
	notice(s_ChanServ, source, "Channel %s is not registered.", chan);
        slog("CS *S %s (%s!%s@%s) [UNREGISTERED]",
          chan, u->nick, u->username, u->host);
        do_break_log("CS_G", "CS *S %s (%s!%s@%s) [UNREGISTERED]",
          chan, u->nick, u->username, u->host);
	return;
    } else if (!u) {
	notice(s_ChanServ, source, "Could not find your user record!");
    } else if (ci->flags & CI_MARKCHAN) {
	notice(s_ChanServ,source,"Channel \2%s\2 is marked!",chan);
        slog("CS *S %s (%s!%s@%s) [MARKED]",
          chan, u->nick, u->username, u->host);
        do_break_log("CS_G", "CS *S %s (%s!%s@%s) [MARKED]",
          chan, u->nick, u->username, u->host);
	log("%s: %s!%s@%s attempted SENDPASS on %s (Fail)"
		,s_ChanServ, source, u->username, u->host, chan);
    } else {
        sendpass_cs(source, ci);
    }
#endif
}


/*************************************************************************/

static void do_suspend(const char *source)
{
  Timeout *t;
  char *cmd, *chan, *ndelta, *s;
  C_SuspendData *c_spn, *prev, *id;
  int i, delta, temp_time;
  C_SuspendData **whichlist = &c_suspend[tolower(chan[0])];
  struct c_userlist *cu, *next_cu;
  char *av[3];

  cmd = strtok(NULL, " ");

  if (!cmd) {
     notice(s_ChanServ, source,
           "Syntax: \2SUSPEND\2 [ADD|DEL|LIST] [<#channel>] [<time>]");
     return;
  }

  if (stricmp(cmd, "LIST") == 0) {

	int sent_header = 0;
        int temp_time;

        for (i = 0; i < MAX_SUSPEND; i++) {
           for (id = c_suspend[i]; id; id = id->next) {
              C_SuspendData *c_spn;
              if (c_spn && NULL != (c_spn = get_c_suspend(id->chan))) {
                  if (!sent_header) {
                      notice(s_ChanServ, source, "Channel Suspend list:");
                      sent_header = 1;
                  }
                  temp_time = id->time - time(NULL);

                  notice(s_ChanServ, source,
                      "Chan: %s  Time: %d minute%s, %d second%s", id->chan,
                      temp_time/60, temp_time/60==1 ? "" : "s",
                      temp_time%60, temp_time%60==1 ? "" : "s");
              }
           }
            
	}
	if (!sent_header)
            notice(s_ChanServ, source, "Channel Suspend list is empty.");

  } else if (!is_services_oper(source)) {
       notice(s_ChanServ, source, "Access Denied");

  } else if (stricmp(cmd, "DEL") == 0) {

     chan = strtok(NULL, " ");

     if (!chan) {
        notice(s_ChanServ, source, "Syntax: \2SUSPEND DEL\2 <#channel>");
        return;
     }

     for (i = 0; i < 256; i++) {
        for (id = c_suspend[i]; id; id = id->next) {
            if (stricmp(id->chan, chan) == 0) {
               temp_time = 0 - id->time;
               add_c_suspend(chan, temp_time);
               notice(s_ChanServ, source, "%s removed from SUSPEND list", chan);
               wallops(s_ChanServ, "%s had me no longer suspend channel %s",
                   source, chan);
               return;
            }
        }
     }

  } else if (stricmp(cmd, "ADD") == 0) {

      Channel *c;
      chan = strtok(NULL, " ");
      ndelta = strtok(NULL, " ");

      if (!chan || !ndelta) {
         notice(s_ChanServ, source,
            "Syntax: \2SUSPEND ADD\2 <#channel> [<time>]");
         return;
      }

      c = findchan(chan);
      delta = atoi(ndelta);

      if (!(delta > 0)) {
         notice(s_ChanServ, source, "IGNORE: Value must be greater than 0");
         return;
      }

      for (i = 0; c_suspend[i]; i++) {
          if (!c_suspend[i])
          break;
      }

      if ((i < MAX_SUSPEND) || (MAX_SUSPEND == 0)) {
          temp_time = delta * 60;
          add_c_suspend(chan, temp_time);
          notice(s_ChanServ, source, "%s added to SUSPEND list for %d minute%s",
                   chan, temp_time/60, temp_time/60==1 ? "" : "s");
          wallops(s_ChanServ, "%s had me suspend channel %s for +%d minute%s",
                   source, chan, temp_time/60, temp_time/60==1 ? "" : "s");


        send_cmd(s_ChanServ, "JOIN %s", chan);

        if (c) {

            for (cu = c->users; cu; cu = next_cu) {
               next_cu = cu->next;
               av[0] = sstrdup(chan);
               av[1] = sstrdup(cu->user->nick);
               av[2] = sstrdup("Channel Suspended");
               send_cmd(s_ChanServ, "MODE %s +b *!*@*", av[0], av[1]);
               send_cmd(s_ChanServ, "KICK %s %s :%s", av[0], av[1], av[2]);
               do_kick(s_ChanServ, 3, av);
               free(av[2]);
               free(av[1]);
               free(av[0]);
            }
        }

        t = add_timeout(CHANNEL_INHABIT, timeout_unban, 0);
        t->data = sstrdup(chan);

        t = add_timeout(CHANNEL_INHABIT+1, timeout_leave, 0);
        t->data = sstrdup(chan);

      } else {
        notice(s_ChanServ, source, "SUSPEND list full");
        wallops(SERVER_NAME, "\2WARNING\2: Suspend list full! (%d)",
           MAX_SUSPEND);
      }
  }
}

/*************************************************************************/

static void do_freeze(const char *source)
{
    ChannelInfo *ci;
    char *chan = strtok(NULL, " ");
    User *u = finduser(source);

        if ((!chan) || (stricmp(chan, "-") == 0)) {	
            notice(s_ChanServ,source,"Syntax: \2FREEZE\2 <#channel>");
            notice(s_ChanServ,source,
                "\2/msg %s OHELP FREEZE\2 for more information",s_ChanServ);
            return;
        }
        if (*chan == '-') {
           *chan++;
           do_unfreeze(source, chan);

        } else if (!(ci = cs_findchan(chan))) {
            notice(s_ChanServ, source, "Channel %s is not registered.",chan);
        } else if (!(ci->flags & CI_FREEZECHAN)) {
            ci->flags |= CI_FREEZECHAN;
            slog("CS +Z %s (%s!%s@%s)",
                chan, u->nick, u->username, u->host);
            do_break_log("CS_F", "CS +Z %s (%s!%s@%s)",
                chan, u->nick, u->username, u->host);
	    wallops(s_ChanServ,
                "%s set the FREEZE flag for \2%s\2",u->nick,chan);
           if (ci->freeze)
                free(ci->freeze);
           ci->freeze = sstrdup(u->nick);

	} else {
            notice(s_ChanServ,source,"Channel %s already FROZEN.",chan);
	}
}

static void do_unfreeze(const char *source, const char *chan)
{
    ChannelInfo *ci;
    User *u = finduser(source);

        if (!chan) {	
            notice(s_ChanServ,source,"Syntax: \2FREEZE\2 -<#channel>");
            notice(s_ChanServ,source,
                "\2/msg %s OHELP UNFREEZE\2 for more information",s_ChanServ);
        } else if (!(ci = cs_findchan(chan))) {
            notice(s_ChanServ, source, "Channel %s is not registered.",chan);
        } else if (ci->flags & CI_FREEZECHAN) {
            ci->flags &= ~CI_FREEZECHAN;
            slog("CS -Z %s (%s!%s@%s)",
                chan, u->nick, u->username, u->host);
            do_break_log("CS_F", "CS -Z %s (%s!%s@%s)",
                chan, u->nick, u->username, u->host);
	    wallops(s_ChanServ,
                "%s UNFROZE channel \2%s\2",u->nick,chan);
           if (ci->freeze)
               free(ci->freeze);
           ci->freeze = NULL;

	} else {
            notice(s_ChanServ,source,"Channel %s is not FROZEN.",chan);
	}
}



/*************************************************************************/

static void do_forbid(const char *source)
{
    ChannelInfo *ci;
    char *chan = strtok(NULL, " ");
    User *u = finduser(source);

    if (!chan) {
        notice(s_ChanServ, source, "Syntax: \2FORBID\2 [-]<#channel>");
        notice(s_ChanServ, source,
              "\2/MSG %s OHELP FORBID\2 fo more information.",
              s_ChanServ);
	return;
    }
    if (readonly) {
	notice(s_ChanServ, source,
	    "Warning: Services is in read-only mode.  Changes will not be saved.");
    }

    if (*chan == '-') {
        *chan++;
        if (chan != NULL)
           do_unforbid(source, chan);
        else
           notice(s_ChanServ, source, "Syntax: FORBID [-]<#channel>");
        return;
    }

    if (ci = cs_findchan(chan)) {
        if (ci->forbid) {
           notice(s_ChanServ, source, "Channel %s already forbidden", chan);
           return;
        }
	delchan(ci);
    }
    if (ci = makechan(chan)) {
        slog("CS +F %s (%s!%s@%s)",
          chan, u->nick, u->username, u->host);
        do_break_log("CS_F", "CS +F %s (%s!%s@%s)",
          chan, u->nick, u->username, u->host);
	log("%s: %s set FORBID for channel %s", s_ChanServ, source, chan);
	ci->flags |= CI_VERBOTEN;
        if (ci->forbid)
             free(ci->forbid);
        ci->forbid = sstrdup(u->nick);
        wallops(s_ChanServ, "%s has forbidden the channel \2%s\2",
             source, chan);
	notice(s_ChanServ, source,
		"Channel \2%s\2 has been marked FORBIDden.", chan);
    } else {
        slog("CS *F %s (%s!%s@%s) [Unknown, Possible Services Bug]",
          chan, u->nick, u->username, u->host);
	log("%s: Valid FORBID for %s by %s failed", s_ChanServ,
		chan, source);
	notice(s_ChanServ, source,
		"Couldn't FORBID channel \2%s\2!", chan);
    }
}



static void do_unforbid(const char *source, const char *chan)
{
    ChannelInfo *ci;
    User *u = finduser(source);

    if (readonly) {
	notice(s_ChanServ, source,
	    "Warning: Services is in read-only mode.  Changes will not be saved.");
    }

    if (ci = cs_findchan(chan)) {
       if (!(ci->flags & CI_VERBOTEN)) {
           slog("CS -F* %s (%s!%s@%s)",
                   chan,u->nick,u->username,u->host);
           log("%s: %s tried to set unforbidden flag for non forbidden chan %s",
                   s_ChanServ, source, chan);
           notice(s_ChanServ, source, "%s is not set as FORBIDDEN", chan);
           return;
        }
        slog("CS -F %s (%s!%s@%s) [Dropped]",
                chan,u->nick,u->username,u->host);
        do_break_log("CS_F", "CS -F %s (%s!%s@%s) [Dropped]",
            chan, u->nick, u->username, u->host);
        log("%s: %s set UNFORBID for chan %s", s_ChanServ, source, chan);
        if (ci->forbid)
            free(ci->forbid);
        ci->forbid = NULL;
        delchan(ci);
        wallops(s_ChanServ, "%s UNFORBID %s [Automatically Dropped]", source, chan);
        notice(s_ChanServ, source, "Channel \2%s\2 is now UNFORBIDDEN.", chan);

    } else {
        slog("CS -F* %s (%s!%s@%s) [Unknown, Possible Services Bug]",
          chan, u->nick, u->username, u->host);
        log("%s: Valid UNFORBID for %s by %s failed", s_ChanServ,
		chan, source);
	notice(s_ChanServ, source,
                "Couldn't UNFORBID channel \2%s\2!", chan);
    }
}


/*************************************************************************/

static void do_why(const char *source)
{
    NickInfo *ni, *ni2;
    ChanAccess *access;
    ChannelInfo *ci;
    User *u, *u2 = finduser(source);
    char *nick, *chan, *whom;
    char **naccess;
    int ulev = 0;
    int i, i2, x;
    int recognized=0, identified=0, acc=0;


    chan = strtok(NULL, " ");
    nick = strtok(NULL, " ");

    if (!nick || strtok(NULL, " ")) {
	notice(s_ChanServ, source,
                "Syntax: \2WHY\2 <#channel> <nickname>");
	notice(s_ChanServ, source,
                "\2/msg %s HELP WHY\2 for more information.", s_ChanServ);
	return;
    }

    if (!(ci = cs_findchan(chan))) {
        notice(s_ChanServ, source, "Channel %s is not registered!", chan);
        return;
    }

    if (u2 && !is_oper(u2->nick) && 3 > (ulev = get_access(u2, ci))) {
        notice(s_ChanServ, source, "Access Denied");
        return;
    }
    ni = findnick(nick);

    if (u = finduser(nick)) {

    for (access = ci->access, i = 0; i < ci->accesscount; ++access,
                                                                  ++i) {
       if (stricmp(access->name, nick) == 0) {
            if (ni && (ni->flags & NI_IDENTIFIED)) {
                 identified++;
                 whom = access->name;

            } else if (ni && (ni->flags & NI_RECOGNIZED)) {
                 recognized++;
                 whom = access->name;
                 acc = access->level;
            }
       } else {
             ni2 = findnick(access->name);
             if (ni2) {

                for (naccess = ni2->access, x=0; x < ni2->accesscount;
                                                ++naccess, ++x) {
                     if (is_on_access(u, ni2)) {
                         if (access->level > acc) {

                            whom = access->name;
                            identified = 0;
                            recognized++;
                         }
                     }
                  }
              }

          }

       }            


       for (i=0; i < MAX_IDS && u->id_nicks[i]; i++) {
         for (access = ci->access, i2 = 0; i2 < ci->accesscount;
                                           ++access, ++i2) {
             ni = findnick(u->id_nicks[i]);
             if (ni && (ni->flags & NI_IDENTIFIED)
                && stricmp(ni->nick, access->name) == 0) {
                    whom = access->name;
                    if (access->level > acc) {
                       identified = 1;
                       whom = access->name;
                    }
                       
             }
         }
       }

       ni = findnick(nick);
 
       ulev = get_access(u, ci);

        if (ulev < 0) {
            notice(s_ChanServ, source,
                   "%s does not have access to channel %s. "
                   "Reason: Banned", nick, chan);
        } else if (ulev == 0) {
            notice(s_ChanServ, source,
                   "%s does not have access to channel %s. "
                   "Reason: Basic User", nick, chan);
        } else if (ulev == 15) {
             if (is_identified(u, ci)) {
                  notice(s_ChanServ, source,
                      "%s has founder access to channel %s. "
                      "Reason: Identification to the channel.",
                       nick, chan);
             } else
                notice(s_ChanServ, source,
                       "%s has founder access to channel %s. "
                       "Reason: %s.",
                       nick, chan,
                       (ni && (ni->flags & NI_IDENTIFIED)) ?
                            "Identification to the founder's nick" :
                            "Recognition to the founder's nick");

        } else if (ulev > 0) {
            if (!whom) {
                notice(s_ChanServ, source,
                       "%s has %s%s%s%s access to channel %s. "
                       "Reason: User Host matches channel access mask.",
                       nick,
                        (ulev == 3) ? "VOP" : "",
			(ulev == 4) ? "HOP" : "",
                        (ulev == 5) ? "AOP" : "",
                        (ulev == 10) ? "SOP" : "",
                        (ulev == 13) ? "Co-Founder" : "",
                        chan);
            } else {
                notice(s_ChanServ, source, 
                       "%s has %s%s%s%s access to channel %s. "
                       "Reason: %s to the nick %s.",
                        nick,
                        (ulev == 3) ? "VOP" : "",
			(ulev == 4) ? "HOP" : "",
                        (ulev == 5) ? "AOP" : "",
                        (ulev == 10) ? "SOP" : "",
                        (ulev == 13) ? "Co-Founder" : "",
                        chan,
                        (identified)
                             ? "Identification" : "Recognition",
                        whom);
            }
        } else {
            notice(s_ChanServ, source, "Channel: \2%s\2 Nick: \2%s\2 (%s@%s) Access: Unknown", chan, nick, u->username, u->host);
        }
    } else {
       notice(s_ChanServ, source, "Nick %s not online", nick);
    }
}

static void do_flist(const char *source)
{
    char *cmd = strtok(NULL, " ");
    ChannelInfo *ci;
    int i;

    if (!cmd) {
        notice(s_ChanServ,source,"Syntax: \2FLIST\2 [HELD|MARKED|FORBIDDEN|FROZEN]");
	notice(s_ChanServ,source,
	    "\2/msg %s OHELP FLIST\2 for more information",s_ChanServ);

    } else if (stricmp(cmd, "HELD") == 0) {
	notice(s_ChanServ,source,"\2HELD\2 channel listing:");
        for (i = 0; i < 256; ++i) {
            for (ci = chanlists[i]; ci; ci = ci->next) {
              if (ci->hold) {
                 notice(s_ChanServ,source,"%-20s %s",ci->name,ci->desc);
                 notice(s_ChanServ, source,"Set by: %s",ci->hold);
              }
	    }
	}
	notice(s_ChanServ,source,"\2End of List\2");
    } else if (stricmp(cmd, "MARKED") == 0) {
        notice(s_ChanServ,source,"\2MARKED\2 channel listing:");
        for (i = 0; i < 256; ++i) {
            for (ci = chanlists[i]; ci; ci = ci->next) {
              if (ci->mark) {
                 notice(s_ChanServ,source,"%-20s  %s",ci->name,ci->desc);
                 notice(s_ChanServ, source,"Set by: %s",ci->mark);
              }
            }
        }
        notice(s_ChanServ,source,"\2End of List\2");
    } else if (stricmp(cmd, "FROZEN") == 0) {
        notice(s_ChanServ,source,"\2FROZEN\2 channel listing:");
        for (i = 0; i < 256; ++i) {
            for (ci = chanlists[i]; ci; ci = ci->next) {
               if (ci->freeze) {
                  notice(s_ChanServ,source,"%-20s  %s",ci->name, ci->desc);
                 notice(s_ChanServ, source,"Set by: %s",ci->freeze);
              }
            }
        }
        notice(s_ChanServ,source,"\2End of List\2");

    } else if (stricmp(cmd, "FORBIDDEN") == 0) {
        notice(s_ChanServ,source,"\2FORBIDDEN\2 channel listing:");
        for (i = 0; i < 256; ++i) {
            for (ci = chanlists[i]; ci; ci = ci->next) {
               if (ci->forbid) {
                 notice(s_ChanServ,source,"%s",ci->name);
                 notice(s_ChanServ, source,"Set by: %s",ci->forbid);
              }
            }
        }
        notice(s_ChanServ,source,"\2End of List\2");
    }
}

static void do_ostopic(const char *source)
{
    char *chan = strtok(NULL, " ");
    char *itopic = strtok(NULL, "");


    if (!chan || !itopic) {
       notice(s_ChanServ, source, "Syntax: \2TOPIC\2 <#channel> <topic>");

    } else if (!(findchan(chan))) {
       notice(s_ChanServ, source, "Channel %s doesn't exist!", chan);

    } else {

    send_cmd(s_ChanServ, "TOPIC %s %s 0 :%s (%s)", chan,
                source, itopic, source);
  }

}



static void do_level(const char *source)
{
    char *chan = strtok(NULL," ");
    char *who = strtok(NULL," ");
    ChannelInfo *ci;
    User *u = finduser(source);

    if (!chan || !who) {
        notice(s_ChanServ,source,"Syntax: \2LEVEL\2 <#channel> "
                "<to-whom>");
        notice (s_ChanServ,source,"Type: \2/msg %s HELP LEVEL\2 "
		"for more information.",s_ChanServ);
	return;
    }
    ci = cs_findchan(chan);
    if (!ci) {
	notice(s_ChanServ,source,"%s isnt registered.",chan);
	return;
    }
    if (!u)
	return;
    if (get_access(u,ci) < 13) {
	notice(s_ChanServ,source,"Permission Denied.");
	return;
    }
    if (stricmp(who,"OPERS") == 0) {
	if (ci->flags & CI_OPERONLY)
	    ci->flags &= ~CI_OPERONLY;
	if (ci->flags & CI_SOPONLY)
	    ci->flags &= ~CI_SOPONLY;
	if (ci->flags & CI_SAONLY)
	    ci->flags &= ~CI_SAONLY;
	if (ci->flags & CI_SRAONLY)
	    ci->flags &= ~CI_SRAONLY;
	if (ci->flags & CI_CODERONLY)
	    ci->flags &= ~CI_CODERONLY;
        if (ci->flags & CI_ABUSEONLY)
            ci->flags &= ~CI_ABUSEONLY;
	ci->flags |= CI_OPERONLY;
        notice(s_ChanServ,source,"LEVEL for \2%s\2 set to \2%s\2",
		chan,who);
	return;
    } else if (stricmp(who,"SOP") == 0) {
        if (ci->flags & CI_OPERONLY)
            ci->flags &= ~CI_OPERONLY;
        if (ci->flags & CI_SOPONLY)
            ci->flags &= ~CI_SOPONLY;
        if (ci->flags & CI_SAONLY)
            ci->flags &= ~CI_SAONLY;
        if (ci->flags & CI_SRAONLY)
            ci->flags &= ~CI_SRAONLY;
        if (ci->flags & CI_CODERONLY)
            ci->flags &= ~CI_CODERONLY;
        if (ci->flags & CI_ABUSEONLY)
            ci->flags &= ~CI_ABUSEONLY;
        ci->flags |= CI_SOPONLY;
        notice(s_ChanServ,source,"LEVEL for \2%s\2 set to \2%s\2",
                chan,who);
        return;
    } else if (stricmp(who,"SA") == 0) {
        if (ci->flags & CI_OPERONLY)
            ci->flags &= ~CI_OPERONLY;
        if (ci->flags & CI_SOPONLY)
            ci->flags &= ~CI_SOPONLY;
        if (ci->flags & CI_SAONLY)
            ci->flags &= ~CI_SAONLY;
        if (ci->flags & CI_SRAONLY)
            ci->flags &= ~CI_SRAONLY;
        if (ci->flags & CI_CODERONLY)
            ci->flags &= ~CI_CODERONLY;
        if (ci->flags & CI_ABUSEONLY)
            ci->flags &= ~CI_ABUSEONLY;
        ci->flags |= CI_SAONLY;
        notice(s_ChanServ,source,"LEVEL for \2%s\2 set to \2%s\2",
                chan,who);
        return;
    } else if (stricmp(who,"SRA") == 0) {
        if (ci->flags & CI_OPERONLY)
            ci->flags &= ~CI_OPERONLY;
        if (ci->flags & CI_SOPONLY)
            ci->flags &= ~CI_SOPONLY;
        if (ci->flags & CI_SAONLY)
            ci->flags &= ~CI_SAONLY;
        if (ci->flags & CI_SRAONLY)
            ci->flags &= ~CI_SRAONLY;
        if (ci->flags & CI_CODERONLY)
            ci->flags &= ~CI_CODERONLY;
        if (ci->flags & CI_ABUSEONLY)
            ci->flags &= ~CI_ABUSEONLY;
        ci->flags |= CI_SRAONLY;
        notice(s_ChanServ,source,"LEVEL for \2%s\2 set to \2%s\2",
                chan,who);
        return;
    } else if (stricmp(who,"MASTERS") == 0) {
        if (ci->flags & CI_OPERONLY)
            ci->flags &= ~CI_OPERONLY;
        if (ci->flags & CI_SOPONLY)
            ci->flags &= ~CI_SOPONLY;
        if (ci->flags & CI_SAONLY)
            ci->flags &= ~CI_SAONLY;
        if (ci->flags & CI_SRAONLY)
            ci->flags &= ~CI_SRAONLY;
        if (ci->flags & CI_CODERONLY)
            ci->flags &= ~CI_CODERONLY;
	if (ci->flags & CI_ABUSEONLY)
	    ci->flags &= ~CI_ABUSEONLY;
        ci->flags |= CI_CODERONLY;
        notice(s_ChanServ,source,"LEVEL for \2%s\2 set to \2%s\2",
                chan,who);
        return;
    } else if (stricmp(who,"ABUSE") == 0) {
        if (ci->flags & CI_OPERONLY)
            ci->flags &= ~CI_OPERONLY;
        if (ci->flags & CI_SOPONLY)
            ci->flags &= ~CI_SOPONLY;
        if (ci->flags & CI_SAONLY)
            ci->flags &= ~CI_SAONLY;
        if (ci->flags & CI_SRAONLY)
            ci->flags &= ~CI_SRAONLY;
        if (ci->flags & CI_CODERONLY)
            ci->flags &= ~CI_CODERONLY;
        if (ci->flags & CI_ABUSEONLY)
            ci->flags &= ~CI_ABUSEONLY;
        ci->flags |= CI_ABUSEONLY;
        notice(s_ChanServ,source,"LEVEL for \2%s\2 set to \2%s\2",
                chan,who);
        return;
    } else if (stricmp(who,"NONE") == 0) {
        if (ci->flags & CI_OPERONLY)
            ci->flags &= ~CI_OPERONLY;
        if (ci->flags & CI_SOPONLY)
            ci->flags &= ~CI_SOPONLY;
        if (ci->flags & CI_SAONLY)
            ci->flags &= ~CI_SAONLY;
        if (ci->flags & CI_SRAONLY)
            ci->flags &= ~CI_SRAONLY;
        if (ci->flags & CI_CODERONLY)
            ci->flags &= ~CI_CODERONLY;
        if (ci->flags & CI_ABUSEONLY)
            ci->flags &= ~CI_ABUSEONLY;
        notice(s_ChanServ,source,"LEVEL for \2%s\2 set to \2%s\2",
                chan,who);
        return;
    } else
        notice(s_ChanServ,source,"Unknown LEVEL who setting");
}

void reset_all_levels()
{
    int i;
    ChannelInfo *ci;

        for (i = 33; i < 256; ++i) {
            for (ci = chanlists[i]; ci; ci = ci->next) {
		reset_levels(ci);
	    }
	}
}

/*************************************************************************/

static void do_unhold(const char *source, const char *chan)
{
     User *u = finduser(source);
     ChannelInfo *ci;

     if (!(ci = cs_findchan(chan))) {
          notice(s_ChanServ, source, "Channel %s is not registered.",chan);
     } else if (ci->flags & CI_HELDCHAN) {
          ci->flags &= ~CI_HELDCHAN;
          if (ci->hold)
              free(ci->hold);
          ci->hold = NULL;

          slog("CS -H %s (%s!%s@%s)",
              chan,u->nick,u->username,u->host);
          do_break_log("CS_F", "CS -H %s (%s!%s@%s)",
              chan, u->nick, u->username, u->host);
          wallops(s_ChanServ,
              "%s unset the HELD flag for \2%s\2",u->nick,chan);
     } else {
          notice(s_ChanServ,source,"Channel \2%s\2 is not held.",chan);
     }
}

/*************************************************************************/

static void do_unmark(const char *source, const char *chan)
{
     User *u = finduser(source);
     ChannelInfo *ci;

     if (!(ci = cs_findchan(chan))) {
          notice(s_ChanServ, source, "Channel %s is not registered.",chan);
     } else if (ci->flags & CI_MARKCHAN) {
          ci->flags &= ~CI_MARKCHAN;
         if (ci->mark)
             free(ci->mark);
          ci->mark = NULL;
          slog("CS -M %s (%s!%s@%s)",
              chan,u->nick,u->username,u->host);
          do_break_log("CS_F", "CS -M %s (%s!%s@%s)",
              chan, u->nick, u->username, u->host);
          wallops(s_ChanServ,
              "%s unset the MARK flag for \2%s\2",u->nick,chan);

     } else {
          notice(s_ChanServ,source,"Channel \2%s\2 is not marked.",chan);
     }

}

/*************************************************************************/

static void do_memolev(User *u, ChannelInfo *ci, char *param)
{
   if (!param) {
      notice(s_ChanServ, u->nick,
        "Syntax: \2MEMO\2 <#channel> <level>");
      notice(s_ChanServ, u->nick,
         "For Help, Type \"/msg %s HELP SET MEMO\" for help.",
           s_MemoServ);
      return;
   }

   if (get_access(u, ci) < 13) {
      notice(s_ChanServ, u->nick, "Access Denied");
      return;
   }

   if (!(stricmp(param, "NONE") == 0) && (stricmp(param, "VOP") == 0)
      && (stricmp(param, "HOP") == 0) && (stricmp(param, "AOP") == 0) 
      && (stricmp(param, "SOP") == 0)
      && (stricmp(param, "CFOUNDER") == 0) && (stricmp(param, "CF") == 0)
      && (stricmp(param, "FOUNDER") == 0)) {
          notice(s_ChanServ, u->nick,
             "\2MEMO\2 options may only be NONE, VOP, AOP, SOP, CFOUNDER, or FOUNDER");
          return;
   }

   if (ci->flags & CI_MEMO_AV)
       ci->flags &= ~CI_MEMO_AV;
   if (ci->flags & CI_MEMO_AOP)
       ci->flags &= ~CI_MEMO_AOP;
  if (ci->flags & CI_MEMO_SOP)
      ci->flags &= ~CI_MEMO_SOP;
  if (ci->flags & CI_MEMO_CF)
      ci->flags &= ~CI_MEMO_CF;
  if (ci->flags & CI_MEMO_FR)
      ci->flags &= ~CI_MEMO_FR;
  if (ci->flags & CI_MEMO_NONE)
      ci->flags &= ~CI_MEMO_NONE;

   if (stricmp(param, "NONE") == 0) {
       ci->flags |= CI_MEMO_NONE;
       notice(s_ChanServ, u->nick,
          "Channel Memos for %s are now disabled", ci->name);

   } else if (stricmp(param, "VOP") == 0) {
       ci->flags |= CI_MEMO_AV;
       notice(s_ChanServ, u->nick,
          "Memo level for %s is now set to \2%s\2", ci->name, param);

   } else if (stricmp(param, "AOP") == 0) {
       ci->flags |= CI_MEMO_AOP;
       notice(s_ChanServ, u->nick,
          "Memo level for %s is now set to \2%s\2", ci->name, param);

   } else if (stricmp(param, "SOP") == 0) {
       ci->flags |= CI_MEMO_SOP;
       notice(s_ChanServ, u->nick,
          "Memo level for %s is now set to \2%s\2", ci->name, param);

   } else if (stricmp(param, "CFOUNDER") == 0 || stricmp(param, "CF") == 0) {
       ci->flags |= CI_MEMO_CF;
       notice(s_ChanServ, u->nick,
          "Memo level for %s is now set to \2%s\2", ci->name, param);

   } else if (stricmp(param, "FOUNDER") == 0) {
       ci->flags |= CI_MEMO_FR;
       notice(s_ChanServ, u->nick,
          "Memo level for %s is now set to \2%s\2", ci->name, param);
   }
}

/*************************************************************************/

static void do_wipe(const char *source)
{
   ChannelInfo *ci;
   char *chan = strtok(NULL, " ");
   char *what = strtok(NULL, " ");
   ChanAccess *access;
   AutoKick *akick;
   int i, c=0, check=0;

   if (!chan || !what) {
      notice(s_ChanServ, source,
         "Syntax: \2WIPE\2 <#channel> [CFOUNDER|SOP|AOP|HOP|VOP|AKICK|ALL]");
      return;

   }
   if (!(ci = cs_findchan(chan))) {
       notice(s_ChanServ, source,
          "Channel %s is not registered", chan);
       return;
   }

   check = ci->accesscount;

   if ((stricmp(what, "AKICK") == 0) || (stricmp(what, "ALL") == 0)) {
       int check2 = 0;
       check2 = ci->akickcount;
       for (akick = ci->akick, i = 0; i < check2; ++akick, ++i) {
           free(akick->name);
           akick->name = NULL;
           if (akick->reason)
               free(akick->reason);
           ++c;
           --ci->akickcount;
           if ((i-c) < ci->akickcount)
                bcopy(akick+1, akick, sizeof(AutoKick)
                                           * (ci->akickcount - (i-c)));
           --akick;
       }
   }

   for (access = ci->access, i = 0; i < check; ++access,++i) {
       if (stricmp(what, "AKICK") == 0)
           return;
       if (stricmp(what, "CFOUNDER") == 0) {
            if (13 != access->level)
               continue;
            free(access->name);
            access->name = NULL;
            access->in_use = 0;
            ++c;
            --ci->accesscount;
            if ((i-c) < ci->accesscount)
                 bcopy(access+1, access, sizeof(ChanAccess)
                                           * (ci->accesscount - (i-c)));
            --access;

       } else if (stricmp(what, "SOP") == 0) {
            if (10 != access->level)
                 continue;
            free(access->name);
            access->name = NULL;
            access->in_use = 0;
            ++c;
            --ci->accesscount;
            if ((i-c) < ci->accesscount)
                 bcopy(access+1, access, sizeof(ChanAccess)
                                           * (ci->accesscount - (i-c)));
            --access;

       } else if (stricmp(what, "AOP") == 0) {
            if (5 != access->level)
                 continue;
            free(access->name);
            access->name = NULL;
            access->in_use = 0;
            ++c;
            --ci->accesscount;
            if ((i-c) < ci->accesscount)
                 bcopy(access+1, access, sizeof(ChanAccess)
                                           * (ci->accesscount - (i-c)));
            --access;

       } else if (stricmp(what, "HOP") == 0) {
            if (4 != access->level)
                 continue;
            free(access->name);
            access->name = NULL;
            access->in_use = 0;
            ++c;
            --ci->accesscount;
            if ((i-c) < ci->accesscount)
                 bcopy(access+1, access, sizeof(ChanAccess)
                                           * (ci->accesscount - (i-c)));
            --access;

       } else if (stricmp(what, "VOP") == 0) {
            if (3 != access->level)
                 continue;
            free(access->name);
            access->name = NULL;
            access->in_use = 0;
            ++c;
            --ci->accesscount;
            if ((i-c) < ci->accesscount)
                 bcopy(access+1, access, sizeof(ChanAccess)
                                           * (ci->accesscount - (i-c)));
            --access;


       } else if (stricmp(what, "ALL") == 0) {
            free(access->name);
            access->name = NULL;
            access->in_use = 0;
            ++c;
            --ci->accesscount;
            if ((i-c) < ci->accesscount)
                 bcopy(access+1, access, sizeof(ChanAccess)
                                           * (ci->accesscount - (i-c)));
            --access;

       } else {
           notice(s_ChanServ, source,
               "Syntax: \2WIPE\2 <#channel> [CFOUNDER|SOP|AOP|HOP|VOP|AKICK|ALL]");
       }
   }
   if (stricmp(what, "AKICK") == 0)
         wallops(s_ChanServ,
            "%s had me wipe the akick list on %s [%d deleted]",
            source, chan, c);
   else if (stricmp(what, "CFOUNDER") == 0)
         wallops(s_ChanServ,
            "%s had me wipe the CFOUNDER list on %s [%d deleted]",
            source, chan, c);
   else if (stricmp(what, "SOP") == 0)
         wallops(s_ChanServ,
            "%s had me wipe the SOP list on %s [%d deleted]",
            source, chan, c);
   else if (stricmp(what, "AOP") == 0)
         wallops(s_ChanServ,
            "%s had me wipe the AOP list on %s [%d deleted]",
            source, chan, c);
   else if (stricmp(what, "HOP") == 0)
         wallops(s_ChanServ,
            "%s had me wipe the HOP list on %s [%d deleted]",
            source, chan, c);
   else if (stricmp(what, "VOP") == 0)
         wallops(s_ChanServ,
            "%s had me wipe the VOP list on %s [%d deleted]",
            source, chan, c);
   else if (stricmp(what, "ALL") == 0)
         wallops(s_ChanServ,
            "%s had me wipe ALL access lists on %s [%d deleted]",
            source, chan, c);
}

/*************************************************************************/

int check_opguard(User *user, const char *chan)
{
   ChannelInfo *ci;

    if (!(ci = cs_findchan(chan)))
        return 0;

    if (ci->flags & CI_OPGUARD) {
         if (!is_on_id_list(user->nick, user->nick))
              return 0;
         else if (check_access(user, ci, CA_AUTOVOICE))
              return 1;
         else
              return 0;
    } else

       return 1;
}

/*************************************************************************/

int check_invite(User *user, const char *chan)
{
   ChannelInfo *ci;
   struct u_chanlist *c;
   char *av[3];
   Timeout *t;
   int i;

    if (!(ci = cs_findchan(chan)))
        return 0;

    if (get_access(user, ci) > 0)
        return 1;

    if (ci->mlock_on & CMODE_I) {
        send_cmd(s_ChanServ, "JOIN %s", chan);
        send_cmd(s_ChanServ, "MODE %s +b *!*@*", chan);
        send_cmd(s_ChanServ, "KICK %s %s :Sorry, Channel is invite only", 
                chan, user->nick);
        t = add_timeout(CHANNEL_INHABIT, timeout_leave, 0);
        t->data = sstrdup(chan); 

        av[0] = sstrdup(chan);
        av[1] = sstrdup(user->nick);
        do_kick(s_ChanServ, 3, av);
        for (c = user->chans; c && stricmp(chan, c->chan->name) != 0;
                                                               c = c->next)
            ;
        if (c)
           chan_deluser(user, c->chan);
        free(av[0]); free(av[1]);
        return 0;

    }
   return 1;

}

#endif	/* !SKELETON */

/**********************************************************/
void do_cs_join(const char *chan) {
   ChannelInfo *ci = cs_findchan (chan);
	if((ci) && (ci->flags & CI_CHANJOIN)) {
		send_cmd (s_ChanServ, "JOIN %s", chan);
		send_cmd (s_ChanServ, "MODE %s +oa %s", chan, s_ChanServ);
	}
}
/***********************************************************/
void do_cs_part(const char *chan) {
   ChannelInfo *ci = cs_findchan (chan);
	if((ci) && (ci->flags & CI_CHANJOIN)) {
		send_cmd (s_ChanServ, "PART %s", chan);
	}
}
/***********************************************************/
static void do_setpass(const char *source)
{
    ChannelInfo *ci;
    char *chan = strtok(NULL, " ");
    char *newpass = strtok(NULL, " ");
    User *u = finduser(source);

    if ((!chan) || (!newpass)) {
        notice(s_ChanServ, source, "Syntax: \2SETPASS\2 <#channel> <password>");
        notice(s_ChanServ, source,
              "\2/MSG %s OHELP SETPASS\2 fo more information.",
              s_ChanServ);
	return;
    }
    if (readonly) {
	notice(s_ChanServ, source,
	    "Warning: Services is in read-only mode.  Changes will not be saved.");
    }

    if (!(ci = cs_findchan(chan)))
        if (ci->forbid) {
           notice(s_ChanServ, source, "Channel %s is not registered", chan);
           return;
        }

    strscpy(ci->founderpass, newpass, PASSMAX);
    notice(s_ChanServ, u->nick,	"%s password changed to \2%s\2.", ci->name, ci->founderpass);
    show_u_next_db(u, s_ChanServ);
}
/***********************************************************************/
void do_doop(const char *source)
{
    User *u = finduser(source);
    ChannelInfo *ci;
    Channel *c;
    NickInfo *ni;
    char *av[3];
    int p=0, i=0, x=0, acc=0;
    struct u_chanlist *c1, *c2;

        if (u->chans != NULL) {
            char buf[512];
            *buf = 0;
            c1 = u->chans;
            while (c1) {
                c2 = c1->next;
                if (*buf)
                    strcat(buf, ", ");
                strcat(buf, c1->chan->name);

     	  if (!(ci = cs_findchan(c1->chan->name))) {
	  } else {
	  if (ci->flags & CI_FREEZECHAN) {
	  } else {
	  acc = get_access(u, ci);

	  if (acc == 3) {
		if (!is_voiced(source, c1->chan->name))
		     change_cmode (s_ChanServ, c1->chan->name, "+v", source);
          } else if (acc == 4) {
		if (!is_chanhop(source, c1->chan->name))
		     change_cmode (s_ChanServ, c1->chan->name, "+h", source);
          } else if (acc > 4) {
		if (!is_chanop(source, c1->chan->name))
	     	 if ((ci->mark) && (CHANX == 1) && (ci->hold)) {
  			   change_cmode (s_ServicesX, c1->chan->name, "+o", source);
	   		if (acc > 12)
  			   change_cmode (s_ServicesX, c1->chan->name, "+a", source);
		 } else {
		   	   change_cmode (s_ChanServ, c1->chan->name, "+o", source);
			if (acc > 12)
		   	   change_cmode (s_ChanServ, c1->chan->name, "+a", source);
		 }
	  } } }		
              c1 = c2;
        } }
}

/************************************************************************/
void do_glist(const char *source)
{
    User *u = finduser(source);
    ChannelInfo *ci;
    int i;
    int acc = 0;
    int totalaccess = 0;

	notice(s_NickServ,source,"\2Your current access is:\2");

        for (i = 0; i < 256; ++i) {
            for (ci = chanlists[i]; ci; ci = ci->next) {
	      acc = 0;
	      acc = get_access(u, ci);
              if (acc == 3) {
                 notice(s_NickServ, source,"Vop         in %s", ci->name);
		 ++totalaccess;
              } else if (acc == 4) {
                 notice(s_NickServ, source,"Hop         in %s", ci->name);
		 ++totalaccess;
              } else if (acc == 5) {
		 ++totalaccess;
                 notice(s_NickServ, source,"Aop         in %s", ci->name);
              } else if (acc == 10) {
		 ++totalaccess;
                 notice(s_NickServ, source,"\2Sop\2        in \37%s\37", ci->name);
              } else if (acc == 13) {
		 ++totalaccess;
                 notice(s_NickServ, source,"\2Co Founder\2 in \37%s\37", ci->name);
              } else if (acc == 15) {
                 notice(s_NickServ, source,"\2Founder\2    in \37%s\37", ci->name);
		 ++totalaccess;
	      } else { }
	    }
	}
	notice(s_NickServ,source,"There are %d access found. --\2End of List\2",totalaccess);
}
/************************************************************************/
void do_aglist(const char *source)
{
    char *nick = strtok(NULL, " ");
    NickInfo *ni;
    ChannelInfo *ci;
    User *u;
    int i;
    int acc = 0;
    int totalaccess = 0;

	if (!nick) {
		notice(s_Agent,source,"Syntax: LISTCHANS <Nick>");	
		return;
	}

	ni = findnick(nick);
	if (u = finduser(nick)) {
	if (!ni) {
		notice(s_Agent,source,"Nick \2%s\2 is not registered",nick);
	} else {
	notice(s_Agent,source,"Current access for nick \2%s\2 is:",nick);
        for (i = 0; i < 256; ++i) {
            for (ci = chanlists[i]; ci; ci = ci->next) {
	      acc = 0;
	      acc = get_access(u, ci);
              if (acc == 3) {
                 notice(s_Agent, source,"Vop         in %s", ci->name);
		 ++totalaccess;
              } else if (acc == 4) {
                 notice(s_Agent, source,"Hop         in %s", ci->name);
		 ++totalaccess;
              } else if (acc == 5) {
		 ++totalaccess;
                 notice(s_Agent, source,"Aop         in %s", ci->name);
              } else if (acc == 10) {
		 ++totalaccess;
                 notice(s_Agent, source,"\2Sop\2        in \37%s\37", ci->name);
              } else if (acc == 13) {
		 ++totalaccess;
                 notice(s_Agent, source,"\2Co Founder\2 in \37%s\37", ci->name);
              } else if (acc == 15) {
                 notice(s_Agent, source,"\2Founder\2    in \37%s\37", ci->name);
		 ++totalaccess;
	      } else { }
	    }
	}
	notice(s_Agent,source,"There are %d access found. --\2End of List\2",totalaccess);
	} 
	} else { 
		notice(s_Agent,source,"%s is not online",nick);
	}
}
/************************************************************************/
static void do_set_chanjoin(User *u, ChannelInfo *ci, char *param)
{
    char *source = u->nick, *chan = ci->name;

    if (!stricmp(param, "ON")) {

	ci->flags |= CI_CHANJOIN;
	notice(s_ChanServ, source,
		"ChanServ will join %s.",chan);
        show_u_next_db(u, s_ChanServ);
	do_cs_join(chan);
    } else if (!stricmp(param, "OFF")) {

	ci->flags &= ~CI_CHANJOIN;
	notice(s_ChanServ, source,
		"Disable joining of ChanServ in %s.",chan);
	do_cs_part(chan);
        show_u_next_db(u, s_ChanServ);

    } else {

	notice(s_ChanServ, source,
                "Syntax: \2SET\2 %s JOIN [ON|OFF]", chan);
	notice(s_ChanServ, source,
		"\2/msg %s HELP SET JOIN\2 for more information.",
		s_ChanServ);
    }
}
/************************************************************************/
static void do_set_auth(User *u, ChannelInfo *ci, char *param)
{
    char *source = u->nick;
    char *chan = ci->name;

    if (stricmp(param, "ON") == 0) {

	ci->flags |= CI_AUTH;
	notice(s_ChanServ, source, "Authorize from founder require before adding any access.");
        show_u_next_db(u, s_ChanServ);

    } else if (stricmp(param, "OFF") == 0) {

	ci->flags &= ~CI_AUTH;
	notice(s_ChanServ, source, "Disable Authorize.");
        show_u_next_db(u, s_ChanServ);

    } else {

	notice(s_ChanServ, source, "Syntax: \2SET\2 %s AUTH [ON|OFF]", chan);
	notice(s_ChanServ, source, "\2/msg %s HELP SET AUTH\2 for more information.", s_ChanServ);
    }
}
/**********************************************************************/
static void do_voice (const char *source)
{
    char *chan = strtok (NULL, " ");
    char *op_params = strtok (NULL, " ");
    User *u = finduser (source);
    ChannelInfo *ci;

    if (!chan)
    {
        notice(s_ChanServ, source,"Syntax: \2VOICE\2 <#channel> <nick>");
        notice(s_ChanServ, source,"\2/msg %s HELP VOICE\2 for more information.", s_ChanServ);
    }

    else if (!findchan (chan))
        notice(s_ChanServ, source,
              "Channel %s does not currently exist (empty)", chan);

    else if (!is_on_chan((op_params) ? op_params : source, chan))
        if (op_params)
            notice (s_ChanServ, source, "User %s is not in %s", op_params, chan);
	else
            notice (s_ChanServ, source, "User %s is not in %s", source, chan);
    
    else if (!(ci = cs_findchan (chan)))
        notice (s_ChanServ, source, "Channel %s is not registered.", chan);

    else if (ci->flags & CI_VERBOTEN)
        notice (s_ChanServ, source, "Channel %s is forbid channel.", chan);

    else if (is_voiced((op_params) ? op_params : source, chan))
        if (op_params)
            notice (s_ChanServ, source, "%s already got voiced in %s", op_params, chan);
        else
            notice (s_ChanServ, source, "You already got voice in %s", chan);

    else if (!u || (get_access(u, ci) < 3))
            notice (s_ChanServ, source, "Access denied.");

    else if (ci->flags & CI_FREEZECHAN)
            notice (s_ChanServ, source, "This channel is freezed.");

    else if (!op_params)
    {
        change_cmode (s_ChanServ, chan, "+v", source);
	send_cmd(s_ChanServ, "NOTICE @%%+%s :%s used ChanServ VOICE: %s",
              chan, u->nick, source);
    }

    else if (finduser (op_params))
    {
        change_cmode (s_ChanServ, chan, "+v", finduser(op_params)->nick);
        notice (s_ChanServ, op_params, "You are were voiced in %s by %s", chan, source);
	send_cmd(s_ChanServ, "NOTICE @%%+%s :%s used ChanServ VOICE: %s",
              chan, u->nick, op_params);
    }

    else
        notice (s_ChanServ, source, "Channel \2%s\2 is not currently in use.", op_params);
}
/****************************************************************************/
static void do_devoice (const char *source)
{
    char *chan = strtok (NULL, " ");
    char *op_params = strtok (NULL, " ");
    User *u = finduser (source);
    ChannelInfo *ci;

    if (!chan)
    {
        notice(s_ChanServ, source,"Syntax: \2DEVOICE\2 <#channel> <nick>");
        notice(s_ChanServ, source,"\2/msg %s HELP DEVOICE\2 for more information.", s_ChanServ);
    }

    else if (!findchan (chan))
        notice(s_ChanServ, source,
              "Channel %s does not currently exist (empty)", chan);

    else if (!is_on_chan((op_params) ? op_params : source, chan))
        if (op_params)
            notice (s_ChanServ, source, "User %s is not in %s", op_params, chan);
	else
            notice (s_ChanServ, source, "User %s is not in %s", source, chan);
    
    else if (!(ci = cs_findchan (chan)))
        notice (s_ChanServ, source, "Channel %s is not registered.", chan);

    else if (ci->flags & CI_VERBOTEN)
        notice (s_ChanServ, source, "Channel %s is forbid channel.", chan);

    else if (!is_voiced((op_params) ? op_params : source, chan))
        if (op_params)
            notice (s_ChanServ, source, "%s don't have voiced in %s yet.", op_params, chan);
        else
            notice (s_ChanServ, source, "You don't have voiced in %s yet.", chan);

    else if (!u || (get_access(u, ci) < 3))
            notice (s_ChanServ, source, "Access denied.");

    else if (ci->flags & CI_FREEZECHAN)
            notice (s_ChanServ, source, "This channel is freezed.");

    else if (!op_params)
    {
        change_cmode (s_ChanServ, chan, "-v", source);
	send_cmd(s_ChanServ, "NOTICE @%%+%s :%s used ChanServ DEVOICE: %s",
              chan, u->nick, source);
    }

    else if (finduser (op_params))
    {
        change_cmode (s_ChanServ, chan, "-v", finduser(op_params)->nick);
        notice (s_ChanServ, op_params, "You are were devoiced in %s by %s", chan, source);
	send_cmd(s_ChanServ, "NOTICE @%%+%s :%s used ChanServ DEVOICE: %s",
              chan, u->nick, op_params);
    }

    else
        notice (s_ChanServ, source, "Channel \2%s\2 is not currently in use.", op_params);
}

/****************************************************************************/
static void do_halfop (const char *source)
{
    char *chan = strtok (NULL, " ");
    char *op_params = strtok (NULL, " ");
    User *u = finduser (source);
    ChannelInfo *ci;

    if (!chan)
    {
        notice(s_ChanServ, source,"Syntax: \2HALFOP\2 <#channel> <nick>");
        notice(s_ChanServ, source,"\2/msg %s HELP HALFOP\2 for more information.", s_ChanServ);
    }

    else if (!findchan (chan))
        notice(s_ChanServ, source,
              "Channel %s does not currently exist (empty)", chan);

    else if (!is_on_chan((op_params) ? op_params : source, chan))
        if (op_params)
            notice (s_ChanServ, source, "User %s is not in %s", op_params, chan);
	else
            notice (s_ChanServ, source, "User %s is not in %s", source, chan);
    
    else if (!(ci = cs_findchan (chan)))
        notice (s_ChanServ, source, "Channel %s is not registered.", chan);

    else if (ci->flags & CI_VERBOTEN)
        notice (s_ChanServ, source, "Channel %s is forbid channel.", chan);

    else if (is_chanhop((op_params) ? op_params : source, chan))
        if (op_params)
            notice (s_ChanServ, source, "%s already got halfop in %s", op_params, chan);
        else
            notice (s_ChanServ, source, "You already got halfop in %s", chan);

    else if (!u || (get_access(u, ci) < 4))
            notice (s_ChanServ, source, "Access denied.");

    else if (ci->flags & CI_FREEZECHAN)
            notice (s_ChanServ, source, "This channel is freezed.");

    else if (!op_params)
    {
        change_cmode (s_ChanServ, chan, "+h", source);
	send_cmd(s_ChanServ, "NOTICE @%%+%s :%s used ChanServ HALFOP: %s",
              chan, u->nick, source);
    }

    else if (finduser (op_params))
    {
        change_cmode (s_ChanServ, chan, "+h", finduser(op_params)->nick);
        notice (s_ChanServ, op_params, "You are were halfop in %s by %s", chan, source);
	send_cmd(s_ChanServ, "NOTICE @%%+%s :%s used ChanServ HALFOP: %s",
              chan, u->nick, op_params);
    }

    else
        notice (s_ChanServ, source, "Channel \2%s\2 is not currently in use.", op_params);
}
/****************************************************************************/
static void do_dehalfop (const char *source)
{
    char *chan = strtok (NULL, " ");
    char *op_params = strtok (NULL, " ");
    User *u = finduser (source);
    ChannelInfo *ci;

    if (!chan)
    {
        notice(s_ChanServ, source,"Syntax: \2DEHALFOP\2 <#channel> <nick>");
        notice(s_ChanServ, source,"\2/msg %s HELP DEHALFOP\2 for more information.", s_ChanServ);
    }

    else if (!findchan (chan))
        notice(s_ChanServ, source,
              "Channel %s does not currently exist (empty)", chan);

    else if (!is_on_chan((op_params) ? op_params : source, chan))
        if (op_params)
            notice (s_ChanServ, source, "User %s is not in %s", op_params, chan);
	else
            notice (s_ChanServ, source, "User %s is not in %s", source, chan);
    
    else if (!(ci = cs_findchan (chan)))
        notice (s_ChanServ, source, "Channel %s is not registered.", chan);

    else if (ci->flags & CI_VERBOTEN)
        notice (s_ChanServ, source, "Channel %s is forbid channel.", chan);

    else if (!is_chanhop((op_params) ? op_params : source, chan))
        if (op_params)
            notice (s_ChanServ, source, "%s don't have halfop in %s yet.", op_params, chan);
        else
            notice (s_ChanServ, source, "You don't have halfop in %s yet.", chan);

    else if (!u || (get_access(u, ci) < 4))
            notice (s_ChanServ, source, "Access denied.");

    else if (ci->flags & CI_FREEZECHAN)
            notice (s_ChanServ, source, "This channel is freezed.");

    else if (!op_params)
    {
        change_cmode (s_ChanServ, chan, "-h", source);
	send_cmd(s_ChanServ, "NOTICE @%%+%s :%s used ChanServ DEHALFOP: %s",
              chan, u->nick, source);
    }

    else if (finduser (op_params))
    {
        change_cmode (s_ChanServ, chan, "-h", finduser(op_params)->nick);
        notice (s_ChanServ, op_params, "You are were dehalfop in %s by %s", chan, source);
	send_cmd(s_ChanServ, "NOTICE @%%+%s :%s used ChanServ DEHALFOP: %s",
              chan, u->nick, op_params);
    }

    else
        notice (s_ChanServ, source, "Channel \2%s\2 is not currently in use.", op_params);
}
/***************************************************************************/

/* 
** Thanks for Wrecked Services Source Code :)
** You have better idea skold :P
*/

static void do_op (const char *source)
{
    char *chan = strtok (NULL, " ");
    char *op_params = strtok (NULL, " ");
    User *u = finduser (source);
    ChannelInfo *ci;

    if (!chan)
    {
        notice(s_ChanServ, source,"Syntax: \2OP\2 <#channel> <nick>");
        notice(s_ChanServ, source,"\2/msg %s HELP OP\2 for more information.", s_ChanServ);
    }

    else if (!findchan (chan))
        notice(s_ChanServ, source,
              "Channel %s does not currently exist (empty)", chan);

    else if (!is_on_chan((op_params) ? op_params : source, chan))
        if (op_params)
            notice (s_ChanServ, source, "User %s is not in %s", op_params, chan);
	else
            notice (s_ChanServ, source, "User %s is not in %s", source, chan);
    
    else if (!(ci = cs_findchan (chan)))
        notice (s_ChanServ, source, "Channel %s is not registered.", chan);

    else if (ci->flags & CI_VERBOTEN)
        notice (s_ChanServ, source, "Channel %s is forbid channel.", chan);

    else if (is_chanop((op_params) ? op_params : source, chan))
        if (op_params)
            notice (s_ChanServ, source, "%s already got op in %s", op_params, chan);
        else
            notice (s_ChanServ, source, "You already got op in %s", chan);

    else if (!u || (get_access(u, ci) < 5))
            notice (s_ChanServ, source, "Access denied.");

    else if (ci->flags & CI_FREEZECHAN)
            notice (s_ChanServ, source, "This channel is freezed.");

    else if (!op_params)
    {
        change_cmode (s_ChanServ, chan, "+o", source);
	send_cmd(s_ChanServ, "NOTICE @%%+%s :%s used ChanServ OP: %s",
              chan, u->nick, source);
    }

    else if (finduser (op_params))
    {
        change_cmode (s_ChanServ, chan, "+o", finduser(op_params)->nick);
        notice (s_ChanServ, op_params, "You are were op in %s by %s", chan, source);
	send_cmd(s_ChanServ, "NOTICE @%%+%s :%s used ChanServ OP: %s",
              chan, u->nick, op_params);
    }

    else
        notice (s_ChanServ, source, "Channel \2%s\2 is not currently in use.", op_params);
}
/****************************************************************************/
static void do_deop (const char *source)
{
    char *chan = strtok (NULL, " ");
    char *op_params = strtok (NULL, " ");
    User *u = finduser (source);
    ChannelInfo *ci;

    if (!chan)
    {
        notice(s_ChanServ, source,"Syntax: \2DEOP\2 <#channel> <nick>");
        notice(s_ChanServ, source,"\2/msg %s HELP DEOP\2 for more information.", s_ChanServ);
    }

    else if (!findchan (chan))
        notice(s_ChanServ, source,
              "Channel %s does not currently exist (empty)", chan);

    else if (!is_on_chan((op_params) ? op_params : source, chan))
        if (op_params)
            notice (s_ChanServ, source, "User %s is not in %s", op_params, chan);
	else
            notice (s_ChanServ, source, "User %s is not in %s", source, chan);
    
    else if (!(ci = cs_findchan (chan)))
        notice (s_ChanServ, source, "Channel %s is not registered.", chan);

    else if (ci->flags & CI_VERBOTEN)
        notice (s_ChanServ, source, "Channel %s is forbid channel.", chan);

    else if (!is_chanop((op_params) ? op_params : source, chan))
        if (op_params)
            notice (s_ChanServ, source, "%s don't have op in %s yet.", op_params, chan);
        else
            notice (s_ChanServ, source, "You don't have op in %s yet.", chan);

    else if (!u || (get_access(u, ci) < 5))
            notice (s_ChanServ, source, "Access denied.");

    else if (ci->flags & CI_FREEZECHAN)
            notice (s_ChanServ, source, "This channel is freezed.");

    else if (!op_params)
    {
        change_cmode (s_ChanServ, chan, "-o", source);
	send_cmd(s_ChanServ, "NOTICE @%%+%s :%s used ChanServ DEOP: %s",
              chan, u->nick, source);
    }

    else if (finduser (op_params))
    {
        change_cmode (s_ChanServ, chan, "-o", finduser(op_params)->nick);
        notice (s_ChanServ, op_params, "You are were deop in %s by %s", chan, source);
	send_cmd(s_ChanServ, "NOTICE @%%+%s :%s used ChanServ DEOP: %s",
              chan, u->nick, op_params);
    }

    else
        notice (s_ChanServ, source, "Channel \2%s\2 is not currently in use.", op_params);
}
/***************************************************************************/
static void do_setfounder(const char *source)
{
    ChannelInfo *ci;
    NickInfo *ni;
    char *chan = strtok(NULL, " ");
    char *newfounder = strtok(NULL, " ");
    User *u = finduser(source);

    if (!chan || !newfounder) {
        notice(s_ChanServ, source, "Syntax: \2SETFOUNDER\2 <#channel> <nick>");
        notice(s_ChanServ, source,
              "\2/MSG %s OHELP SETFOUNDER\2 fo more information.",
              s_ChanServ);
	return;
    }
    if (readonly) {
	notice(s_ChanServ, source,
	    "Warning: Services is in read-only mode.  Changes will not be saved.");
    }

    if (!(ci = cs_findchan(chan)))
        if (ci->forbid) {
           notice(s_ChanServ, source, "Channel %s is not registered", chan);
           return;
        }
    ni = findnick(newfounder);

    if (!ni) {
           notice(s_ChanServ, source, "Nick %s is not registered.", newfounder);
	   return;
    }
    strscpy(ci->founder, newfounder, NICKMAX);
    notice(s_ChanServ, u->nick,	"%s founder transfer to \2%s\2.", ci->name, ci->founder);
    show_u_next_db(u, s_ChanServ);
}
